// |$Id: ba2s.c,v 1.1.1.1 2017/09/16 19:19:46 pm Exp $              |
/* |------------------------------------------------------------|
   | Basic Seismic Utilities (BSU)                              |
   | LANGUAGE: C                                                |
   | AUTHOR: P. MICHAELS    http://cgiss.boisestate.edu/~pm     | 
   | FUNCTION NAME: ba2s.c                                      |
   | DIRECTORY: bsu-2.0.2/src/C/bsegy                           |
   |------------------------------------------------------------|
   | purpose: convert ascii text file containing matrix of      |
   |          samples into multi-trace bsegy format file        |
   |------------------------------------------------------------|
   |  NOTICE:                                                   |
   |  Copyright (C) 2017 Paul Michaels                          |
   |  <pm@cgiss.boisestate.edu>                                 |
   |  This program is free software; you can                    |
   |  redistribute it and/or modify it under the terms          |
   |  of the GNU General Public License as published            |
   |  by the Free Software Foundation; either version           |
   |  3 of the License, or (at your option) any later           |
   |  version.  This program is distributed in the              |
   |  hope that it will be useful, but                          |
   |  WITHOUT ANY WARRANTY; without even the implied            |
   |  warranty of MERCHANTABILITY or FITNESS FOR A              |
   |  PARTICULAR PURPOSE.  See the GNU General Public           |
   |  License for more details.                                 |
   |  You should have received a copy of the GNU                |
   |  General Public License along with this program;           |
   |  if not, write to the Free Software Foundation,            |
   |  Inc., 675 Mass Ave, Cambridge, MA 02139, USA.             |
   |------------------------------------------------------------|
   |INLINE FUNCTIONS:                                           |
   |  getparm    get parameters from command line or interactive|
   |  outlst     echo check of input parameters to listing file |
   |  s1alloc    allocate memory for input trace                |
   |                                                            |
   |------------------------------------------------------------|
   |sublibC4 functions:                                         |
   |  c_bsegin   read seismic trace                             |
   |  c_bsegout  write a seismic trace                          |
   |  in_chk     scan input file for samples, number traces etc.|
   |  bargrid    write progress bar grid to screen              |
   |  exbar      write progress bar to screen                   |
   |  cr_labl    write copyright label to screen                |
   |  fcr_labl   write copyright label to file                  |
   |  names      create an output file name from input file name|
   |                                                            |
   |------------------------------------------------------------|
*/
      #include <stdio.h>
      #include <stdlib.h>
      #include <float.h>
      #include <math.h>
      #include <string.h>
      #include <time.h>
      #include "../include/c_bsegy.h"

//...parameter definitions
	#define MAXCHAN 480
	#define NDIM 32767

//...function prototypes
      #include "../include/sub4.h"

      int  getparm(int argc,char *argv[], int *iorder, int *ncol, int *nrow,
        int *isamin, char infile[],
        char ofile[], char lstfile[], char pid[]);

      void outlst(char infile[], int iorder, int ncol, int nrow, int isamin,
        char pid[], char ofile[], char lstfile[], FILE *h3);

      int readascii(FILE *h3, float s1[], int npts, int jrec, int ncol,
	 int iorder, char fs);	

      int tstfield(FILE *h1, char *fs);

      char *sfnsc(char *loc);

	
      int main(int argc, char *argv[])
     {

//...declare variables
      FILE              *h1, *h2, *h3;
//    int               jt,
      int               jrec,
                        npts,
                        ntraces=0,
                        ibar,
                        invbar;
//    float             fsamin,
      float             bar,
                        *s1;
      char              pid[5]="ba2s",   //process id name
                        ofile[13],
                        lstfile[13],
                        infile[14];
      struct header     hd;

//...specific to this application
	char		buff[10],
			blnk=' ';
		//	scrs[5];
/*	int		j,
		  	idsfc,
		  	nchan, */
	int		iorder,
			ncol,
			nrow,
			isamin;
/*	float		delayt,
			gaink,
			fold;
  	int		gidba[50],
  			nvsa[50]; */
	char		fs;	//field separator
//...get parameters
      if(getparm(argc,argv, &iorder, &ncol, &nrow, &isamin,
         infile,ofile,lstfile,pid)!=0)
      {
       goto quick_exit;
      }

//...open output listing file
        if( (h3=fopen(lstfile,"w"))==NULL )
        {
          printf("\n ABORT !! Error opening file %s",lstfile);
          goto quick_exit;
        }

//...echo check to listing file
        outlst(infile,iorder,ncol,nrow,isamin,pid,ofile,lstfile,h3);

//...assign npts and ntraces depending on iorder
	if (iorder == 0 ) {
	npts= nrow;
	ntraces=ncol;
		if (npts > NDIM ) {
		fprintf(stderr,"ABORT: npts=%d >%d  BSEGY Header Limit \n",npts,NDIM);
		return(1); } //endif
		if (ncol > MAXCHAN ) {
		fprintf(stderr,"ABORT: ncol=%d >%d ba2s Limit \n",ncol,MAXCHAN);
		return(1); } //endif
	 }
	else {
	npts= ncol;
	ntraces=nrow;
		if (npts > NDIM ) {
		fprintf(stderr,"ABORT: npts=%d >%d  BSEGY Header Limit \n",npts,NDIM);
		return(1); } //endif
		if (ncol > MAXCHAN ) {
		fprintf(stderr,"ABORT: ncol=%d >%d ba2s Limit \n",ncol,MAXCHAN);
		return(1); } //endif
	} //endif

//...open input file
        if( (h1=fopen(infile,"rb"))==NULL )
        {
          printf("\n ABORT !! Error opening file %s",infile);
          goto exit_3;
        }

//...test for field separator
	tstfield(h1,&fs);
/*
	if (fs  == ' ' ) fprintf(stdout,"\n Field Separator is space"); 
	if (fs  == ',' ) fprintf(stdout,"\n Field Separator is comma"); 
	if (fs  == 9 ) fprintf(stdout,"\n Field Separator is tab"); 
*/
//...open output file
        if( (h2=fopen(ofile,"wb"))==NULL )
        {
          printf("\n ABORT !! Error opening file %s",ofile);
          goto exit_1;
        }

//...assign default header values
	hd.idist=0;	//range, shot to receiver
	hd.length=(short) npts; //number of samples in trace
	hd.isi=(short) isamin;
	hd.duse=1;

	hd.smul1=1;	//multiplier for next values
	hd.irel=0;	//receiver elevation
	hd.ishe=0;	//shot elevation
	hd.ishd=0;	//shot depth
	hd.delr=0;	//datum elevation of receiver
	hd.dels=0;	//datum elevation of source
	hd.wds=0;	//water depth of source
	hd.wdr=0;	//water depth of receiver

	hd.smul2=1;	//multiplier for next values
	hd.ishlo=0;	//shot coordinate, sx
	hd.ishla=0;	//shot coordinate, sy
	hd.irlo=0;	//receiver coordinate, rx
	hd.irla=0;	//receiver coordinate, ry

	hd.cunits=1;	//units in meters
	hd.wvel=0;	//weathering velocity
	hd.swvel=0;	//subweathering velocity
	hd.utimes=0;	//uphole time at shot
	hd.utimeg=0;	//uphole time at group
	hd.sstati=0;	//shot static (ms)
	hd.gstati=0;	//group static (ms)
	hd.tstati=0;	//total static (ms)
	hd.istime=0;	//lag time A
	hd.ibtime=0;	//lag time B
	hd.mtimes=0;	//mute start time
	hd.mtimee=0;	//mute end time
	hd.tic=1;	//trace id code
	sprintf(buff,"%c%c%c%c",blnk,blnk,blnk,blnk);
	memcpy(&hd.deploy[0],&buff[0],4);	//deployment name
	memcpy(&hd.spname[0],&buff[0],4);	//shot name
	memcpy(&hd.rstnam[0],&buff[0],4);	//receiver site name
	memcpy(&hd.shotid[0],&buff[0],4);	//shot id
	memcpy(&hd.lineid[0],&buff[0],4);	//line name
	memcpy(&hd.geoor[0],&buff[0],4);	//geophone orientation
	hd.mst=0;	//first break pick time
	hd.shtazi=0;	//source azimuth East=90
	hd.charge=0;
	hd.shtver=180;	//source polarization down
	hd.azimut=0;
	hd.geoazi=0;	//geophone azimuth East=90
	hd.geover=0;

//...dynamic allocation of seimic trace memory
        s1=calloc(npts,sizeof(float));
          if(s1==NULL)
         {
          fprintf(stderr,"ABORT:s1 can't allocate enough trace memory");
          fprintf(stderr," npts = %d",npts);
          goto exit_s1;
         }
//-----------------main loop---------------------------
//...display progress bar grid

      bargrid(ntraces,&bar,&ibar,&invbar);

//...trace loop
	  int jstart=0;
	  fseek(h1,jstart,SEEK_SET);
      for (jrec=0;jrec<ntraces;jrec++)
     {

//...read trace
	readascii(h1,s1,npts,jrec,ncol,iorder,fs);

//...assign header values
	hd.nvs=1;	//vertical fold
	hd.gidb=0;	//instrument gain in dB
	hd.tsnl=jrec+1;		//trace sequence number in line
	hd.tsnt=jrec+1;		//trace sequence number in reel
	hd.tnofr=jrec+1;	//trace number original field record

//...output trace
        if(c_bsegout(jrec,s1,npts,&hd,h2)!=0)
       {
        puts("ABORT--c_bsegout");
        goto quit_it;
       }

//...display progress bar
      exbar(ntraces,bar,ibar,invbar,jrec+1);

     } //  end trace loop-----------------------------

      printf("\n   Listing ====>%s",lstfile);
      printf("\n   Output  ====>%s",ofile);
      printf("\n   Processed %d traces",ntraces);
      return(0);

quit_it:
        puts(" ");

        free(s1);
exit_s1:
        fclose(h2);
exit_1:
        fclose(h1);
exit_3:
        fclose(h3);

quick_exit:
	return(1);

     } //  end of main

/* |------------------------------------------------------------|
   | Basic Seismic Utilities (BSU)                              | 
   | LANGUAGE: C                                                |
   | AUTHOR: P. MICHAELS <pm@cgiss.boisestate.edu>              |
   | FUNCTION NAME: getparm.c                                   |
   | Copyright (C) 2017 P. Michaels  (See GNU License)          |
   |------------------------------------------------------------|
   |                                                            |
   | purpose: gets input parameters from command                |
   |          line or prompts user from consol.                 |
   |                                                            |
   |                                                            |
   |------------------------------------------------------------|
*/
      int  getparm(int argc,char *argv[], int *iorder, int *ncol, int *nrow, 
         int *isamin, char infile[],
         char ofile[], char lstfile[],char pid[])
     {
      int               i,
			nchar;
      char              sfx1[4]="seg",
			stmp[12],
                        sfx2[4]="lst";
      char		help[3]="-h";
      char              msg1[46],msg2[46];

//...test for command line help ?
      if(argc==2 && !strcmp(argv[1],help))
     {
//...screen header and copyright notice
      cr_labl(pid);

//...help pannel
      sprintf(msg1,"Convert ascii text file (matrix of samples)  ");
      sprintf(msg2,"to BSU (bsegy) format.  No geometry setting. ");
      hlp_labl(msg1,msg2);
      puts(" ");
      
//...help display
      puts(" ");
      printf("  ba2s  infile iorder ncol nrow dt \n");
      puts(" ");
      puts("     infile   =  input file name");
      puts("     iorder   =  1 each row is a trace, columns are time axis (int)");
      puts("              =  0 each col is a trace, rows are time axis (int) ");
      puts("     ncol     =  number of columns in matrix (int)");
      puts("     nrow     =  number of rows in matrix (int)");
      puts("     dt       =  sample interval in micro seconds (int)");
      puts(" ");
      return(1);
     }
      else
     {
//...get parameters------------------------------
//...pars off character parameters
      if(argc>1)
       strncpy(infile,argv[1],13);
       else
      {
       puts(" Enter INPUT file name ");
       scanf(" %13s",infile);
      }
	if (namecheck(12,infile) != 0 ) return (1);

//...convert string to integer
      if(argc>2)
       *iorder=atoi(argv[2]);
       else
      {
       puts(" Enter iorder (int)");
       puts("     iorder   =  1 each row is a trace, columns are time axis ");
       puts("              =  0 each col is a trace, rows are time axis ");
       scanf(" %d",iorder);
      }

//...convert string to integer
      if(argc>3)
       *ncol=atoi(argv[3]);
       else
      {
       puts(" Enter ncol (int)");
       scanf(" %d",ncol);
      }

//...convert string to integer
      if(argc>4)
       *nrow=atoi(argv[4]);
       else
      {
       puts(" Enter nrow (int)");
       scanf(" %d",nrow);
      }

//...convert string to integer
      if(argc>5)
       *isamin=atoi(argv[5]);
       else
      {
       puts(" Enter dt (int)");
//     scanf(" %f",isamin);
       scanf(" %d",isamin);
      }

//...name output files
	nchar=0;
     for (i=0;i<8;i++)
	{
 	  if(infile[i]!='.') 
	  { stmp[i]=infile[i];
	    nchar=nchar+1; }
	else
	  goto label1;
	}	//next i
label1:
     stmp[nchar]=0;
     sprintf(ofile,"%s.%s",stmp,sfx1);    
     sprintf(lstfile,"%s.%s",stmp,sfx2); 
       return(0);
     }  //  endif bhelp test
      } // end getparm

/* |------------------------------------------------------------|
   | Basic Seismic Utilities (BSU)                              | 
   | LANGUAGE: C                                                |
   | AUTHOR: P. MICHAELS <pm@cgiss.boisestate.edu>              |
   | FUNCTION NAME: outlst.c                                    |
   | Copyright (C) 2017 P. Michaels  (See GNU License)          |
   |------------------------------------------------------------|
   |                                                            |
   | purpose: prints copyright header, file names, and          |
   |          echo check of input parameters in newly           |
   |          opened output listing file.                       |
   |                                                            |
   |------------------------------------------------------------|
*/
      void outlst(char infile[], int iorder, int ncol, int nrow, int isamin,
        char pid[], char ofile[], char lstfile[], FILE *h3)
     {

//...print copyright label to outlst and screen

        fcr_labl(pid,h3);

//...echo check
      fputs("\n",h3);                      //blank line
      fprintf(h3,"\n     PROGRAM: ba2s.c    ");
      fputs("\n",h3);                      //blank line
      fputs("\n     FILES:       ",h3);
      fprintf(h3,"\n     Input   File= %s",infile);
      fprintf(h3,"\n     Output  File= %s",ofile);
      fprintf(h3,"\n     Listing File= %s",lstfile);
      fputs("\n",h3);                      //blank line
      fputs("\n     INPUT PARAMETERS: ",h3);
      fprintf(h3,"\n     iorder=%d ",iorder);
      fprintf(h3,"\n     ncol=%d ",ncol);
      fprintf(h3,"\n     nrow=%d ",nrow);
      fprintf(h3,"\n     dt=%d microseconds ",isamin);
      fprintf(h3,"\n      ");
      fputs("\n",h3);                      //blank line
     }  //  end outlst


/* |------------------------------------------------------------|
   | Basic Seismic Utilities (BSU)                              | 
   | LANGUAGE: C                                                |
   | AUTHOR: P. MICHAELS <pm@cgiss.boisestate.edu>              |
   | FUNCTION NAME: readascii                                   |
   | Copyright (C) 2017 P. Michaels  (See GNU License)          |
   |------------------------------------------------------------|
   |                                                            |
   | purpose: reads channel=jrec                        	|
   |          ncol=max number of columns in file                |
   |          iorder select row or col order                    |
   |------------------------------------------------------------|
*/
	int readascii(FILE *h1, float s1[], int npts, int jrec,
	 int ncol, int iorder, char fs)
	{
	#define MAXCOL 50000

	char	*loc;
	char	buff[MAXCOL];
	long	jstart=0;
	float   fval;
//	char	LF=10, chr;
	int	jfield, /* field of text file */
		j;  /* record of text file */

	if (iorder == 0 )
	{
	  fseek(h1,jstart,SEEK_SET);
	    for (j=0; j<npts; j++) {
 	    fgets(buff+1, MAXCOL-1, h1);  //read next record	
	    *buff=' '; //make sure space at beginning, don't want to miss first trace
	    loc=buff;
	loc=sfnsc(loc);  //search for next non-space character
/*       scan across row of text */
		for (jfield=0; jfield<ncol; jfield++) {
		sscanf(loc-1,"%f",&fval) ; 
		if (jfield == jrec) s1[j]=fval; 
		if (jfield==ncol-1) break;
		loc=sfnsc(loc);  //search for next non-space character
		}  //next jfield
	    } //next j time loop
	} // else iorder 
	else
	{
 	fgets(buff+1, MAXCOL-1, h1);	
	*buff=' '; //make sure space at beginning, don't want to miss first trace
	loc=buff;
	loc=sfnsc(loc);  //search for next non-space character
		for (jfield=0; jfield<ncol; jfield++)
		{
		sscanf(loc-1,"%f",&fval) ; 
		s1[jfield]=fval;
		if (jfield==ncol-1) break;
		loc=sfnsc(loc);  //search for next non-space character
		} //next jfield
	} //endif iorder
	return(0);
	} //end readascii
//
//FUNCTION: scan for first non space character in a row
	char *sfnsc(char *loc)
	{
	int	k,
		nchar;
	char	tst;

		nchar=strlen(loc);
		loc=strchr(loc,' ');
		for (k=0; k<nchar;k++) {
		 sscanf(loc,"%c",&tst);
		if (tst != ' ') break;
		 loc=loc+1;
		} //endfor
	return(loc);
	} //end sfnsc
//
//FUNCTION: test for field separator
	int tstfield(FILE *h1, char *fs)
	{
	long	jstart=0;
	char	buff[500];
	char	c=',',  	//comma
		s=' ', 		//space
		t=9; 		//tab
	char	*loc;

	fseek(h1,jstart,SEEK_SET);
	fgets(buff,500,h1);
	loc=buff;
	loc=strchr(buff+1,s);
	if (loc != NULL ) *fs=s;
//	fprintf(stdout,"\n loc %ld buff %ld loc-buff %ld",loc,buff,loc-buff);
	loc=buff;
	loc=strchr(buff+1,t);
	if (loc != NULL) *fs=t;	
//	fprintf(stdout,"\n loc %ld buff %ld loc-buff %ld",loc,buff,loc-buff);
	loc=buff;
	loc=strchr(buff+1,c);
	if (loc != NULL) *fs=c;
//	fprintf(stdout,"\n loc %ld buff %ld loc-buff %ld",loc,buff,loc-buff);
	return (0);
	} //end tstfield
