// |$Id: bsum.c,v 1.1.1.1 2017/09/16 19:19:46 pm Exp $          |
/* |------------------------------------------------------------|
   | Basic Seismic Utilities (BSU)                              |
   | LANGUAGE: C                                                |
   | AUTHOR: P. MICHAELS    http://cgiss.boisestate.edu/~pm     | 
   | FUNCTION NAME: bsum.c                                      |
   | DIRECTORY: bsu-2.0.2/src/C/bsegy                           |
   |------------------------------------------------------------|
   | purpose: Sum two bsegy data sets.                          |
   |------------------------------------------------------------|
   |  NOTICE:                                                   |
   |  Copyright (C) 2017 Paul Michaels                          |
   |  <pm@cgiss.boisestate.edu>                                 |
   |  This program is free software; you can                    |
   |  redistribute it and/or modify it under the terms          |
   |  of the GNU General Public License as published            |
   |  by the Free Software Foundation; either version           |
   |  3 of the License, or (at your option) any later           |
   |  version.  This program is distributed in the              |
   |  hope that it will be useful, but                          |
   |  WITHOUT ANY WARRANTY; without even the implied            |
   |  warranty of MERCHANTABILITY or FITNESS FOR A              |
   |  PARTICULAR PURPOSE.  See the GNU General Public           |
   |  License for more details.                                 |
   |  You should have received a copy of the GNU                |
   |  General Public License along with this program;           |
   |  if not, write to the Free Software Foundation,            |
   |  Inc., 675 Mass Ave, Cambridge, MA 02139, USA.             |
   |------------------------------------------------------------|
   |INLINE FUNCTIONS:                                           |
   |  getparm    get parameters from command line or interactive|
   |  outlst     echo check of input parameters to listing file |
   |                                                            |
   |------------------------------------------------------------|
   |sublibC4 functions:                                         |
   |  c_bsegin   read seismic trace                             |
   |  c_bsegout  write a seismic trace                          |
   |  in_chk     scan input file for samples, number traces etc.|
   |  bargrid    write progress bar grid to screen              |
   |  exbar      write progress bar to screen                   |
   |  cr_labl    write copyright label to screen                |
   |  fcr_labl   write copyright label to file                  |
   |  names      create an output file name from input file name|
   |                                                            |
   |------------------------------------------------------------|
*/
      #include <stdio.h>
      #include <stdlib.h>
      #include <float.h>
      #include <math.h>
      #include <string.h>
      #include <time.h>
      #include "../include/c_bsegy.h"

//...parameter definitions

//...function prototypes
      #include "../include/sub4.h"

      int  getparm(int argc,char *argv[],
         float *scalef, char infile1[], char infile2[],
         char ofile[], char lstfile[],char pid[]);

      void outlst(float scalef, char infile1[], char infile2[],
        char pid[], char ofile[], char lstfile[], FILE *h3);

      int main(int argc, char *argv[])
     {

//...declare variables
      FILE              *h1, *h2, *h3, *h4;
      int               jt,
                        jrec,
                        npts,
                        ntraces,
                        ibar,
                        invbar;
      float             fsamin,
                        bar,
                        *s1,
			*s4,
                        *s2;
      char              pid[5]="bsum",   //process id name
                        ofile[13],
                        lstfile[13],
			infile1[14],
                        infile2[14];
      struct header     hd;

//...specific to this application
      float             scalef;
      int		npts2,
			ntraces2;
      float		fsamin2;
			

//...get parameters
      if(getparm(argc,argv,&scalef,infile1,
         infile2,ofile,lstfile,pid)!=0)
      {
       goto quick_exit;
      }

//...open output listing file
        if( (h3=fopen(lstfile,"w"))==NULL )
        {
          fprintf(stderr,"\n ABORT !! Error opening file %s",lstfile);
          goto quick_exit;
        }

//...echo check to listing file
      outlst(scalef,infile1,infile2,pid,ofile,lstfile,h3);

//...open input file 1
        if( (h1=fopen(infile1,"rb"))==NULL )
        {
          fprintf(stderr,"\n ABORT !! Error opening file %s",infile1);
          goto exit_3;
        }

//...open input file 2
        if( (h4=fopen(infile2,"rb"))==NULL )
        {
          fprintf(stderr,"\n ABORT !! Error opening file %s",infile2);
          goto exit_3;
        }

//...open output file
        if( (h2=fopen(ofile,"wb"))==NULL )
        {
          fprintf(stderr,"\n ABORT !! Error opening file %s",ofile);
          goto exit_1;
        }

//...check input file 1 size
          if(in_chk(&ntraces,&npts,&fsamin,hd,h1)!=0)
         {
          goto exit_2;
         }

//...check input file 2 size
          if(in_chk(&ntraces2,&npts2,&fsamin2,hd,h4)!=0)
         {
          goto exit_2;
         }

//...check for conformable
	if(ntraces!=ntraces2) 
	{  fprintf(stderr,"ABORT ntraces not equal: %d  %d",ntraces,ntraces2);
	   goto exit_2; }

	if(npts!=npts2) 
	{  fprintf(stderr,"ABORT npts not equal: %d  %d",npts,npts2);
	   goto exit_2; }

	if(fsamin!=fsamin2) 
	{  fprintf(stderr,"ABORT fsamin not equal: %f  %f",fsamin,fsamin2);
	   goto exit_2; }

//...print input file data to lstfile
        fprintf(h3,"\n     Number of samples= %d",npts);
        fprintf(h3,"\n     Sample Interval = %5.4f sec.",fsamin);
        fprintf(h3,"\n     Number of traces = %d",ntraces);

//...dynamic allocation of seimic trace memory
        s1=calloc(npts,sizeof(float));
          if(s1==NULL)
         {
          fprintf(stderr,"ABORT:s1 can't allocate enough trace memory");
          fprintf(stderr," npts = %d",npts);
          goto exit_2;
         }
        s4=calloc(npts,sizeof(float));
          if(s1==NULL)
         {
          fprintf(stderr,"ABORT:s4 can't allocate enough trace memory");
          fprintf(stderr," npts = %d",npts);
          goto exit_s1;
         }
        s2=calloc(npts,sizeof(float));
          if(s2==NULL)
         {
          fprintf(stderr,"ABORT:s2 can't allocate enough trace memory");
          fprintf(stderr," npts = %d",npts);
          goto exit_s2;
         }
//-----------------main loop---------------------------
//...display progress bar grid

      bargrid(ntraces,&bar,&ibar,&invbar);

//...trace loop

      for (jrec=0;jrec<ntraces;jrec++)
     {

//...read trace
        if(c_bsegin(jrec,s4,npts,&hd,h4)!=0)
       {
        fprintf(stderr,"ABORT-- 4 c_bsegin");
        goto quit_it;
       }

        if(c_bsegin(jrec,s1,npts,&hd,h1)!=0)
       {
        fprintf(stderr,"ABORT-- 1 c_bsegin");
        goto quit_it;
       }

//...sum traces
         for (jt=0;jt<npts;jt++)
        {
         s2[jt]=s1[jt]+scalef*s4[jt];
        }

//...output trace
        if(c_bsegout(jrec,s2,npts,&hd,h2)!=0)
       {
        fprintf(stderr,"ABORT--c_bsegout");
        goto quit_it;
       }

//...display progress bar
      exbar(ntraces,bar,ibar,invbar,jrec+1);

     } //  end trace loop-----------------------------

      printf("\n   Listing ====>%s",lstfile);
      printf("\n   Output  ====>%s",ofile);
      printf("\n   Processed %d traces",ntraces);
	return(0);
quit_it:
        puts(" ");

        free(s2);
exit_s2:
	free(s4);
exit_s1:
        free(s1);
exit_2:
        fclose(h2);
exit_1:
        fclose(h1);
exit_3:
        fclose(h3);

quick_exit:
	return(1);

     } //  end of main

/* |------------------------------------------------------------|
   | Basic Seismic Utilities (BSU)                              | 
   | LANGUAGE: C                                                |
   | AUTHOR: P. MICHAELS <pm@cgiss.boisestate.edu>              |
   | FUNCTION NAME: getparm.c                                   |
   | Copyright (C) 2017 P. Michaels  (See GNU License)          |
   |------------------------------------------------------------|
   |                                                            |
   | purpose: gets input parameters from command                |
   |          line or prompts user from consol.                 |
   |                                                            |
   |                                                            |
   |------------------------------------------------------------|
*/
      int  getparm(int argc,char *argv[],
         float *scalef, char infile1[], char infile2[],
         char ofile[], char lstfile[],char pid[])
     {
      char              sfx1[4]="seg",
                        sfx2[4]="lst";
      char		help[3]="-h";
      char              msg1[46],msg2[46];

//...test for command line help ?
      if(argc==2 && !strcmp(argv[1],help))
     {
//...screen header and copyright notice
      cr_labl(pid);

//...help pannel
      sprintf(msg1,"Sums two seismic data sets                   ");
      sprintf(msg2,"Second data set scaled:  s3=s1*scalef*s2     ");
      hlp_labl(msg1,msg2);
      puts(" ");

//...help display
      puts(" ");
      printf("  %s    infile1   infile2   scalef  ",pid);
      puts(" ");
      puts("     infile1  =  first  input file name");
      puts("     infile2  =  second input file name");
      puts("     scalef   =  scale factor");
      puts("     output = input1 + scalef*input2");
      puts(" ");
      return(1);
     }
      else
     {
//...get parameters------------------------------
//...pars off character parameters
      if(argc>1)
       strncpy(infile1,argv[1],13);
       else
      {
       puts(" Enter 1st INPUT file name ");
       scanf(" %13s",infile1);
      }
	if (namecheck(12,infile1) != 0 ) return (1);

//...second input file
      if(argc>2)
       strncpy(infile2,argv[2],13);
       else
      {
       puts(" Enter 2nd INPUT file name ");
       scanf(" %13s",infile2);
      }
	if (namecheck(12,infile2) != 0 ) return (1);

//...convert string to float
      if(argc>3)
       *scalef=atof(argv[3]);
       else
      {
       puts(" Enter scale factor (float)");
       scanf(" %f",scalef);
      }

//...name output files
      names(infile1,pid,ofile,sfx1);
      names(infile1,pid,lstfile,sfx2);
       return(0);
     }  //  endif bhelp test
      } // end getparm

/* |------------------------------------------------------------|
   | Basic Seismic Utilities (BSU)                              | 
   | LANGUAGE: C                                                |
   | AUTHOR: P. MICHAELS <pm@cgiss.boisestate.edu>              |
   | FUNCTION NAME: outlst.c                                    |
   | Copyright (C) 2017 P. Michaels  (See GNU License)          |
   |------------------------------------------------------------|
   |                                                            |
   | purpose: prints copyright header, file names, and          |
   |          echo check of input parameters in newly           |
   |          opened output listing file.                       |
   |                                                            |
   |------------------------------------------------------------|
*/

      void outlst(float scalef, char infile1[], char infile2[],
        char pid[], char ofile[], char lstfile[], FILE *h3)
     {

//...print copyright label to outlst and screen

      fcr_labl(pid,h3);

//...echo check
      fputs("\n",h3);                      //blank line
      fprintf(h3,"\n     PROGRAM: %s.c ",pid);
      fputs("\n",h3);                      //blank line
      fputs("\n     FILES:       ",h3);
      fprintf(h3,"\n     1st Input   File= %s",infile1);
      fprintf(h3,"\n     2nd Input   File= %s",infile2);
      fprintf(h3,"\n     Output  File= %s",ofile);
      fprintf(h3,"\n     Listing File= %s",lstfile);
      fputs("\n",h3);                      //blank line
      fputs("\n     INPUT PARAMETERS: ",h3);
      fprintf(h3,"\n     scalef= %f ",scalef);
      fputs("\n",h3);                      //blank line
     }  //  end outlst
