// |$Id: bswp.c,v 1.1.1.1 2017/09/16 19:19:46 pm Exp $          |
/* |------------------------------------------------------------|
   | Basic Seismic Utilities (BSU)                              |
   | LANGUAGE: C                                                |
   | AUTHOR: P. MICHAELS    http://cgiss.boisestate.edu/~pm     | 
   | FUNCTION NAME: bswp.c                                      |
   | DIRECTORY: bsu-2.0.2/src/C/bsegy                           |
   |------------------------------------------------------------|
   | purpose: byte swap bsegy or SU data set                    |
   |          240 byte header, data trace (float, int4, or int2)|
   |------------------------------------------------------------|
   |  NOTICE:                                                   |
   |  Copyright (C) 2016 Paul Michaels                          |
   |  <pm@cgiss.boisestate.edu>                                 |
   |  This program is free software; you can                    |
   |  redistribute it and/or modify it under the terms          |
   |  of the GNU General Public License as published            |
   |  by the Free Software Foundation; either version           |
   |  3 of the License, or (at your option) any later           |
   |  version.  This program is distributed in the              |
   |  hope that it will be useful, but                          |
   |  WITHOUT ANY WARRANTY; without even the implied            |
   |  warranty of MERCHANTABILITY or FITNESS FOR A              |
   |  PARTICULAR PURPOSE.  See the GNU General Public           |
   |  License for more details.                                 |
   |  You should have received a copy of the GNU                |
   |  General Public License along with this program;           |
   |  if not, write to the Free Software Foundation,            |
   |  Inc., 675 Mass Ave, Cambridge, MA 02139, USA.             |
   |------------------------------------------------------------|
   |INLINE FUNCTIONS:                                           |
   |  getparm    get parameters from command line or interactive|
   |  outlst     echo check of input parameters to listing file |
   |                                                            |
   |------------------------------------------------------------|
   |sublibC4 functions:                                         |
   |  c_bsegin   read seismic trace                             |
   |  c_bsegout  write a seismic trace                          |
   |  in_chk     scan input file for samples, number traces etc.|
   |  bargrid    write progress bar grid to screen              |
   |  exbar      write progress bar to screen                   |
   |  cr_labl    write copyright label to screen                |
   |  fcr_labl   write copyright label to file                  |
   |  names      create an output file name from input file name|
   |                                                            |
   |------------------------------------------------------------|
*/
      #include <stdio.h>
      #include <stdlib.h>
      #include <float.h>
      #include <math.h>
      #include <string.h>
      #include <time.h>
      #include "../include/c_bsegy.h"

//...parameter definitions

//...function prototypes
      #include "../include/sub4.h"

      int  getparm(int argc,char *argv[],
        int *idfc, int *npts, char infile[],
        char ofile[], char lstfile[], char pid[]);

      void outlst(int idfc, char infile[],
        char pid[], char ofile[], char lstfile[], FILE *h3);

      void swaptrace(int idfc, void  *s2,
      int npts, struct header *hd);

	void swap4(void *x4);

	void swap2(void *x2);

      int  main(int argc, char *argv[])
     {

//...declare variables
      FILE              *h1, *h2, *h3;
//    int               jt,
      int               jrec,
                        npts,   //user supplied number samples per trace
//			nptshd, //header value, may be wrong endian
                        ntraces,
//                      ntraceshd,
                        ibar,
                        invbar;
//    float             fsamin,
      float             bar,
                        *s1,
                        *s2;
      char              pid[5]="bswp",   //process id name (4char)
                        ofile[13],
                        lstfile[13],
                        infile[14];
      int 		fpos,nbyte;
      struct header     hd;

//...specific to this application
      int               idfc;  

//...get parameters
      if(getparm(argc,argv,&idfc,&npts,  
         infile,ofile,lstfile,pid)!=0)
      {
       goto quick_exit;
      }

//...open output listing file
        if( (h3=fopen(lstfile,"w"))==NULL )
        {
          fprintf(stderr,"\n ABORT !! Error opening file %s",lstfile);
          goto quick_exit;
        }

//...echo check to listing file
      outlst(idfc,infile,pid,ofile,lstfile,h3);

//...open input file
        if( (h1=fopen(infile,"rb"))==NULL )
        {
          fprintf(stderr,"\n ABORT !! Error opening file %s",infile);
          goto exit_3;
        }

//...open output file
        if( (h2=fopen(ofile,"wb"))==NULL )
        {
          fprintf(stderr,"\n ABORT !! Error opening file %s",ofile);
          goto exit_1;
        }

/*
//...check input file size
          if(in_chk(&ntraceshd,&nptshd,&fsamin,hd,h1)!=0)
         {
          goto exit_2;
         }
*/
//...Determine the length of the file in bytes
        fseek(h1,0,SEEK_END);
        fpos=ftell(h1);
//      fprintf(h3,"\n bytes in file= %ld",fpos);
        fprintf(h3,"\n bytes in file= %d",fpos);

//...Compute number of traces
	switch (idfc) {
	case 1: {
	nbyte=4; break; } //esac 1
	case 2: {
	nbyte=4; break; } //esac 1
	case 3: {
	nbyte=2; break; } //esac 1
	} //endswitch
        ntraces=(fpos)/(240+nbyte*(npts));
        fprintf(h3,"\n number of traces = %d",ntraces); 


//...print input file data to lstfile
        fprintf(h3,"\n     Number of samples= %d",npts);
        fprintf(h3,"\n     Computed Number of traces = %d \n",ntraces);
fflush(h3);
//...dynamic allocation of seimic trace memory
        s1=calloc(npts,sizeof(float));
          if(s1==NULL)
         {
          fprintf(stderr,"ABORT:s1 can't allocate enough trace memory");
          fprintf(stderr," npts = %d",npts);
          goto exit_2;
         }
        s2=calloc(npts,sizeof(float));
          if(s2==NULL)
         {
          fprintf(stderr,"ABORT:s2 can't allocate enough trace memory");
          fprintf(stderr," npts = %d",npts);
          goto exit_s1;
         }
//-----------------main loop---------------------------
//...display progress bar grid

      bargrid(ntraces,&bar,&ibar,&invbar);

//...trace loop

      for (jrec=0;jrec<ntraces;jrec++)
     {

//...read trace
        if(c_bsegin(jrec,s1,npts,&hd,h1)!=0)
       {
        fprintf(stderr,"ABORT--c_bsegin");
        goto quit_it;
       }

//...byteswap trace
	swaptrace(idfc,s1,npts,&hd);

//...output trace
        if(c_bsegout(jrec,s1,npts,&hd,h2)!=0)
       {
        fprintf(stderr,"ABORT--c_bsegout");
        goto quit_it;
       }

//...display progress bar
      exbar(ntraces,bar,ibar,invbar,jrec+1);

     } //  end trace loop-----------------------------

      printf("\n   Listing ====>%s",lstfile);
      printf("\n   Output  ====>%s",ofile);
      printf("\n   Processed %d traces",ntraces);
        return(0);

quit_it:
        puts(" ");
        free(s2);
exit_s1:
        free(s1);
exit_2:
        fclose(h2);
exit_1:
        fclose(h1);
exit_3:
        fclose(h3);
quick_exit:
        return(1);
     } //  end of main
/* |------------------------------------------------------------|
   | Basic Seismic Utilities (BSU)                              | 
   | LANGUAGE: C                                                |
   | AUTHOR: P. MICHAELS <pm@cgiss.boisestate.edu>              |
   | FUNCTION NAME: getparm.c                                   |
   | Copyright (C) 2016 P. Michaels  (See GNU License)          |
   |------------------------------------------------------------|
   |                                                            |
   | purpose: gets input parameters from command                |
   |          line or prompts user from consol.                 |
   |                                                            |
   |                                                            |
   |------------------------------------------------------------|
*/
      int  getparm(int argc,char *argv[],
         int *idfc, int *npts, char infile[],
         char ofile[], char lstfile[],char pid[])
     {
      char              sfx1[4]="seg",
                        sfx2[4]="lst";
      char		help[3]="-h";
      char              msg1[46],msg2[46];

//...test for command line help ?
      if(argc==2 && !strcmp(argv[1],help))
     {
//...screen header and copyright notice
      cr_labl(pid);

//...help pannel
      sprintf(msg1,"Byte Swap Program                            ");
      sprintf(msg2,"Assumes BSEGY or SU data format              ");
      hlp_labl(msg1,msg2);
      puts(" ");

//...help display
      puts(" ");
      printf("  %s  infile   idfc  npts \n",pid);
      puts(" ");
      puts("     infile  = input file name (4char minimum)");
      puts(" ");
      puts("     Data format                         ");
      puts("     idfc   1=4 byte floating point    ");
      puts("            2=4 byte integer         ");
      puts("            3=2 byte integer         ");
      puts("     npts    =number of samples per trace ");
      puts(" ");
      return(1);
     }
      else
     {
//...get parameters------------------------------
//...pars off character parameters
      if(argc>1)
       strncpy(infile,argv[1],13);
       else
      {
       puts(" Enter INPUT file name ");
       scanf(" %13s",infile);
      }
	if (namecheck(12,infile) != 0 ) return (1);

//...convert string to integer
      if(argc>2)
       *idfc=atoi(argv[2]);
       else
      {
       puts(" Enter data format code (int)");
       puts("     idfc   1=4 byte floating point    ");
       puts("            2=4 byte integer         ");
       puts("            3=2 byte integer         ");
       scanf(" %d",idfc);
      }

//...convert string to integer
      if(argc>3)
       *npts=atoi(argv[3]);
       else
      {
       puts(" Enter number of samples per trace, npts");
       scanf(" %d",npts);
      }

//...name output files
      names(infile,pid,ofile,sfx1);
      names(infile,pid,lstfile,sfx2);
       return(0);
     }  //  endif bhelp test
      } // end getparm
/* |------------------------------------------------------------|
   | Basic Seismic Utilities (BSU)                              | 
   | LANGUAGE: C                                                |
   | AUTHOR: P. MICHAELS <pm@cgiss.boisestate.edu>              |
   | FUNCTION NAME: outlst.c                                    |
   | Copyright (C) 2017 P. Michaels  (See GNU License)          |
   |------------------------------------------------------------|
   |                                                            |
   | purpose: prints copyright header, file names, and          |
   |          echo check of input parameters in newly           |
   |          opened output listing file.                       |
   |                                                            |
   |------------------------------------------------------------|
*/
      void outlst(int idfc, char infile[],
        char pid[], char ofile[], char lstfile[], FILE *h3)
     {

//...print copyright label to outlst and screen

      fcr_labl(pid,h3);

//...echo check
      fputs("\n",h3);                      //blank line
      fprintf(h3,"\n     PROGRAM: %s.c ",pid);
      fputs("\n",h3);                      //blank line
      fputs("\n     FILES:       ",h3);
      fprintf(h3,"\n     Input   File= %s",infile);
      fprintf(h3,"\n     Output  File= %s",ofile);
      fprintf(h3,"\n     Listing File= %s",lstfile);
      fputs("\n",h3);                      //blank line
      fputs("\n     INPUT PARAMETERS: ",h3);
      fprintf(h3,"\n     idfc  = %5.5d ",idfc);
	switch (idfc)
	{
		case 1: {
		fprintf(h3," Data are 4 byte floating point \n");
		break;} //esac 1

		case 2: {
		fprintf(h3," Data are 4 byte integer (two's complement) \n");
		break;} //esac 1

		case 3: {
		fprintf(h3," Data are 2 byte integer (two's complement) \n");
		break;} //esac 1
	} //end switch
      fputs("\n",h3);                      //blank line
     }  //  end outlst
/* |------------------------------------------------------------|
   | Basic Seismic Utilities (BSU)                              | 
   | LANGUAGE: C                                                |
   | AUTHOR: P. MICHAELS <pm@cgiss.boisestate.edu>              |
   | FUNCTION NAME: swaptrace.c                                 |
   | Copyright (C) 2017 P. Michaels  (See GNU License)          |
   |------------------------------------------------------------|
   | purpose: output the jrec-th trace from data                |
   |          set in BSEGY  format.                             |
   |                                                            |
   | input:  h1      =pointer to input file                     |
   |         npts    =number of points in a trace               |
   |         idfc    =data format type                          |
   |                1=floating point 4 byte                     |
   |                2=integer        4 byte                     |
   |                3=integer        2 byte                     | 
   |                                                            |
   | output: hd      =pointer to header structure               |
   |         s2      =pointer to data array                     |
   |                                                            |
   |------------------------------------------------------------|
*/

      void swaptrace(int idfc, void  *s2,
      int npts, struct header *hd)
     {
	int		i4;
	short int	i2;
	void		*ptr;
	int		j;
//	unsigned int 	dst;
//	int		j4;

// Format 240 byte header
//initialize pointer
	ptr=hd;

//loop through header and swap
	/* 4 byte section*/
	for (j=0; j<28; j=j+4) {
	memcpy(&i4,(ptr + j),4);
	swap4(&i4);	
	memcpy((ptr+j),&i4,4);
	} //next j
	/* 2 byte section*/
	for (j=28; j<36; j=j+2) {
	memcpy(&i2,(ptr + j),2);
	swap2(&i2);	
	memcpy((ptr+j),&i2,2);
	} //next j
	/* 4 byte section*/
	for (j=36; j<68; j=j+4) {
	memcpy(&i4,(ptr + j),4);
	swap4(&i4);	
	memcpy((ptr+j),&i4,4);
	} //next j
	/* 2 byte section*/
	for (j=68; j<72; j=j+2) {
	memcpy(&i2,(ptr + j),2);
	swap2(&i2);	
	memcpy((ptr+j),&i2,2);
	} //next j
	/* 4 byte section*/
	for (j=72; j<88; j=j+4) {
	memcpy(&i4,(ptr + j),4);
	swap4(&i4);	
	memcpy((ptr+j),&i4,4);
	} //next j
	/* 2 byte section*/
	for (j=88; j<180; j=j+2) {
	memcpy(&i2,(ptr + j),2);
	swap2(&i2);	
	memcpy((ptr+j),&i2,2);
	} //next j
	/* 4 byte section*/
	for (j=180; j<184; j=j+4) {
	memcpy(&i4,(ptr + j),4);
	swap4(&i4);	
	memcpy((ptr+j),&i4,4);
	} //next j
	/* 2 byte section*/
	for (j=184; j<208; j=j+2) {
	memcpy(&i2,(ptr + j),2);
	swap2(&i2);	
	memcpy((ptr+j),&i2,2);
	} //next j
	j=208;
	memcpy(&i4,(ptr + j),4);
	swap4(&i4);	
	memcpy((ptr+j),&i4,4);
// end of header swap bsegy

// Convert data trace according to type (float, int4, int2)
	switch (idfc) {
	case 1: {
		for (j=0; j<npts; j++) {
		ptr=s2+j*4;
		swap4(ptr);
		} //next j
	break;
	} //esac 1
	case 2: {
		for (j=0; j<npts; j++) {
		ptr=s2+j*4;
  		swap4(ptr);
		} //next j
	break;
	} //esac 2

	case 3: {
		for (j=0; j<npts; j++) {
		ptr=s2+j*2;
		swap2(ptr);
		} //next j
	break;
	} //esac 3


	} //end switch


     } //end swaptrace
/* |------------------------------------------------------------|
   | Basic Seismic Utilities (BSU)                              | 
   | LANGUAGE: C                                                |
   | AUTHOR: P. MICHAELS <pm@cgiss.boisestate.edu>              |
   | FUNCTION NAME: swap2.c  Swap a 2 byte variable             |
   | Copyright (C) 2017 P. Michaels  (See GNU License)          |
   |------------------------------------------------------------|
   |                                                            |
   | Byte swap for endian conversion                            |
   |------------------------------------------------------------|
*/
/* 2 byte  swap  PM */
void swap2(void *x2)
{
unsigned char *tmp=(unsigned char *) x2;
unsigned char x;
x=tmp[0];
tmp[0]=tmp[1];
tmp[1]=x;
} //end swap2

/* |------------------------------------------------------------|
   | Basic Seismic Utilities (BSU)                              | 
   | LANGUAGE: C                                                |
   | AUTHOR: P. MICHAELS <pm@cgiss.boisestate.edu>              |
   | FUNCTION NAME: swap4.c  Swap a 4 byte variable             |
   | Copyright (C) 2017 P. Michaels  (See GNU License)          |
   |------------------------------------------------------------|
   |                                                            |
   | Byte swap for endian conversion                            |
   |------------------------------------------------------------|
*/
/* 4 byte  swap  PM */
void swap4(void *x4)
{
unsigned char *tmp=(unsigned char *) x4;
unsigned char x;
x=tmp[0];
tmp[0]=tmp[3];
tmp[3]=x;
x=tmp[1];
tmp[1]=tmp[2];
tmp[2]=x;
} //end swap4

