c |$Id: bedt.f,v 1.1.1.1 2017/09/16 19:19:46 pm Exp $          |
c --------------------------------------------------------------
c | Basic Seismic Utilities (BSU)                              |
c | LANGUAGE: Fortran77                                        |
c | AUTHOR: P. MICHAELS  http://cgiss.boisestate.edu/~pm       | 
c | Modified: bug fix istop=-1 added                           |
c | PROGRAM NAME: bedt.f                                       |
c | DIRECTORY: bsu-2.0.2/src/Fort/bsegy                        |
c |------------------------------------------------------------|
c |              Edit Program for data in BSEGY format         |
c | 1.resample      3.tmin,tmax                                |
c | 2.interpolate   4.extract traces or pad                    |
c |------------------------------------------------------------|
c |  NOTICE:                                                   |
c |  Copyright (C) 2017 Paul Michaels                          |
c |  <pm@cgiss.boisestate.edu>                                 |
c |  This program is free software; you can                    |
c |  redistribute it and/or modify it under the terms          |
c |  of the GNU General Public License as published            |
c |  by the Free Software Foundation; either version           |
c |  3 of the License, or (at your option) any later           |
c |  version.  This program is distributed in the              |
c |  hope that it will be useful, but                          |
c |  WITHOUT ANY WARRANTY; without even the implied            |
c |  warranty of MERCHANTABILITY or FITNESS FOR A              |
c |  PARTICULAR PURPOSE.  See the GNU General Public           |
c |  License for more details.                                 |
c |  You should have received a copy of the GNU                |
c |  General Public License along with this program;           |
c |  if not, write to the Free Software Foundation,            |
c |  Inc., 675 Mass Ave, Cambridge, MA 02139, USA.             |
c |------------------------------------------------------------|
c |  PARAMETERS:                                               |
c |  ndim = maximum number of samples in a seismic trace       |
c |------------------------------------------------------------|
c |  INCLUDE FILE REQUIRED:                                    |
c |  ../include/bsegy.inc    (modified segy header definitions)|
c |                (no reel header, floating point data)       |
c |------------------------------------------------------------|
c |  SUBROUTINES CALLED:                                       |
c | In-line===============                                     |
c |     GETPRM  get input parameters from command line or      |
c |             interactive query.  Should be inline with      |
c |             main program, specific to application.         |
c |     LSTPRM  echo check of input parameters from GETPRM.    |
c |             Values written to a history file (*.lst)       |
c |     SAMPLE  Sinc Interpolation function                    |
c |                                                            |
c | Located in bsu-2.0.2/src/sublibF4===========               |
c |     CHKTRC  get number of samples, sample interval, and    |
c |             number of traces in the input data file.       |
c |             Also returns values for progress bar, stdout   |
c |     BSEGIN  Reads a seismic trace.                         |
c |     BSEGOUT Write a seismic trace.                         |
c |     PLTBAR  Writes progress bar on stdout                  |
c |                                                            |
c --------------------------------------------------------------
c
      include '../include/bsegy.inc'
      parameter (ndim=32767)
c
      real*4            s1(ndim),s2(ndim)
      character*4       aplc
      character*40      infil, outfil, outlst
      common /header/ hd
c
c...assign I/O units, initialize....
      jrec=1
      iunit1=1
      iunit2=2
      icall1=0
      icall2=0
c
c...define application prefix
      write(aplc,'(''bedt'')')
c
c...get parameters-------------------------------------------
      call GETPRM(nargsx,infil,ndim,s1,
     +tmin1,tmax1,ifirst,ilast,idecm)
c
c...count traces and check dimensions------------------------
      call CHKTRC(iunit1,icall1,npts,s1,fsamin,infil,
     +bar,invbar,ibar,ntrace,ndim)
      dt=fsamin
c..INPUT PARAMETER CHECK---------------------------------
c
c...new sample rate dt2 (seconds) jisi (microseconds)==>header
c...if idecm.lt.0 then interpolation
      if(idecm.gt.0) then
      dt2=dt*float(idecm)
      jisi=isi*idecm
      else
      dt2=dt/abs(float(idecm))
      jisi=isi/iabs(idecm)
      endif
c
c...check parameters
      jtmin=int(tmin1/dt)+1
      jtmax=int(tmax1/dt)+1
c
c...static header in msec
      tjstat=int(-float(jtmin-1)*dt*1000.)
c
c
      if(idecm.gt.0) then
      jtlen=(jtmax-jtmin)/idecm + 1
      if(jtlen.lt.1) then
      write(*,'('' abort---decimation too large'')')
      stop
      endif
      else
      jtlen=(jtmax-jtmin)*iabs(idecm) +1
      if(jtlen.gt.ndim) then
      write(*,'('' abort---interpolation too large'')')
      stop
      endif
      endif
c
c      write(*,'('' new length='',i5)') jtlen
      fsamin2=dt2
      npts2=jtlen
c...END INPUT PARAMETER CHECK----------------------------
c
c...name outputfile
      write(outfil,'(a,a,''.seg'')') aplc(1:4),infil(1:4)
c
c...output listing-------------------------------------------
      call LSTPRM(io11,ntrace,aplc,outlst,outfil,fsamin,fsamin2,
     +npts,npts2,infil,tmin1,tmax1,ifirst,ilast,idecm,iantia)
c
c...clear output file...
      open(10,file=outfil,access='direct',recl=240)
      close(10,status='delete')
c
      icall1=0
      icall2=0
      jtrace=0
      istop=-1
c
c  read traces and edit
c...padding on left if needed
      if(ifirst.lt.0) then
      do 70 jt=1,jtlen
      s2(jt)=0.00
   70 continue
      istop=-ifirst
      do 71 jtrace=1,istop
      tsnl=jtrace
c
c...output trace
      iunit2=2
      CALL BSEGOUT(iunit2,
     +icall2,npts2,s2,fsamin2,outfil,jtrace)
c
   71 continue
      jtrace=istop
      endif
c
c
      ntr = 0
      isw33=0
c
c...data loop=======================================....
c
      do 100 jrec=1,ntrace
c
c...input a trace
      call BSEGIN(iunit1,ndim,
     +icall1,npts,s1,fsamin,infil,jrec,iexit)
c
      ntr = ntr + 1
c
      if(jrec.ge.ifirst .and. jrec.le.ilast) then
      isw33=isw33+1
      jtrace=jtrace + 1
c
c...anti alias filter trace if decimation
      if(idecm.gt.0) then
      if(iafilt.eq.1) then
      do 72 jt=1,npts
      data=0.0
      do 90 k=1,idecm
   90 data=data+s1(jt+k-1)
      data=data/float(idecm)
      do 92 k=1,idecm
   92 s1(jt+k-1)=data
   72 continue
      endif
      endif
c
c...if loop on trace >>>>>>>>>>>
      jpts=1
      jt=jtmin
      txintr=float(jt-1)*dt
c
  75  if(jpts.eq.jtlen) go to 76
c------------------------------
      data=0.0
      if(idecm.gt.0) then
      if(jt.ge.1 .and. jt.le.npts) data=s1(jt)
      jt=jt+idecm
      else
      if(txintr.ge.0.0 .and. txintr.lt.npts) then
      data=sample(npts,s1,dt,txintr)
      endif
      txintr=txintr+dt2
      endif
      s2(jpts)=data
      jpts=jpts+1
c------------------------------
      go to 75
  76  continue
c....end if loop >>>>>>>>>>>>>>>
      tsnl=tsnl+istop+1
c
c...output trace
      CALL BSEGOUT(iunit2,
     +icall2,npts2,s2,fsamin2,outfil,jtrace)
c
      endif
c
c...display progress....
      call pltbar(bar,invbar,ibar,jrec,ntrace)
c
c
c...END LOOP========================================....
  100 continue
c
c
c...padding on right if needed
      if(ilast.gt.ntrace) then
      do 80 jt=1,jtlen
      s2(jt)=0.00
   80 continue
      jstart=jtrace+1
      jstop=jstart+ilast-ntrace-1
      do 81 jtrace=jstart,jstop
      tsnl=jtrace
c
c...output trace
      CALL BSEGOUT(iunit2,
     +icall2,npts2,s2,fsamin2,outfil,jtrace)
c
   81 continue
      jtrace=jtrace-1
      endif
c
      write(6,240) ntr,outfil,outlst
240   format(/' Traces Processed = ',i5/
     +5x,'====> output in ',a/'====> listing in ',a)
c
c
      close(io11)
      close(iunit1)
      close(iunit2)
      stop
      end
c --------------------------------------------------------------
c | Basic Seismic Utilities                                    |
c | AUTHOR: P. MICHAELS  http://cgiss.boisestate.edu/~pm       | 
c | PROGRAM NAME: sample.f                                     |
c |------------------------------------------------------------|
c | function that does sinc interpolation for                  |
c | an segy trace                                              |
c |                                                            |
c |  s1 = trace to be interpolated                             |
c |  dt = sample rate of s1                                    |
c |   t = time at which sample desired                         |
c |                                                            |
c --------------------------------------------------------------
c
      real*4 function sample(npts,s1,dt,t)
      implicit real*4 (a-h,o-z)
      real*4 s1(*)
c
c...find preceeding index and x
      pi=3.1415926
      pidt=pi/dt
      j=int(t/dt+0.5) + 1
      val=0.0
c
c
      kstart=j-11
      kend=j+11
c
      if (kstart.lt.1) kstart=1
      if (kend.gt.npts) kend=npts
c
      do 5, k=kstart,kend
      delt=abs(float(k)*dt - t)
      arg=delt*pidt
      if(arg.ne.0.0) then
      ck=s1(k)*(sin(arg)/arg)
      else
      ck=s1(k)
      endif
c
      val=val+ck
5     continue
      sample=val
      end
c
c --------------------------------------------------------------
c | Basic Seismic Utilities                                    |
c | AUTHOR: P. MICHAELS  http://cgiss.boisestate.edu/~pm       | 
c | PROGRAM NAME: getprm.f                                     |
c |------------------------------------------------------------|
c | get input parameters or help                               |
c --------------------------------------------------------------
c
      subroutine GETPRM(nargsx,infil,ndim,s1,
     +tmin1,tmax1,ifirst,ilast,idecm)
c
      character*40      infil
      character*40      arg1,arg2,arg3,arg4,arg5,arg6,arg7
      real*4            s1(*)
c
c
c...get input/output file names...
c   |--------------------------------------------|
c   | file naming convention                     |
c   | application prefix 4 characters "bxxx"     |
c   | data set suffix 4 characters "dddd"        |
c   | file name = bxxxdddd.seg                   |
c   |--------------------------------------------|
      write(*,*)'|---------------------------------|'
      write(*,*)'| Copyright (C) 2017  P. Michaels |'
      write(*,*)'|       All rights reserved       |'
      write(*,*)'|see GNU General Public License   |'
      write(*,*)'|---------------------------------|'
c
ccc---LINUX VERSION
      nargsx=iargc()
      karg=-1
c...test for help
      if(nargsx.ge.(2+karg)) then
      call getarg(1,arg1)
      if(arg1(1:2).eq.'-h') then
      write(*,*)' '
      write(*,*) '|------------------------------------------|'
      write(*,*) '| Basic Seismic Utilities     FORTRAN      |'
      write(*,*) '| ONLINE HELP:                             |'
      write(*,*) '|------------------------------------------|'
      write(*,*) '| bedt: Edit seismic trace files in time   |'
      write(*,*) '| or in channels, resamples also.          |'
      write(*,*) '|------------------------------------------|'
      write(*,*)' '
      write(*,*)
     +'bedt infil tmin tmax ifirst ilast idecm iantia '
      write(*,*)' '
      write(*,*)'infil   =input file name to edit '
      write(*,*)'tmin    =minimum time to extract data '
      write(*,*)'tmax    =maximum time to extract data '
      write(*,*)'ifirst  =first trace to extract (<0 pads left) '
      write(*,*)'ilast   =last trace to extract (>ntraces pads right)'
      write(*,*)'idecm   =decimation factor (idecm>0)  '
      write(*,*)'        =interpolation factor (idecm<0)  '
      write(*,*)'    EXAMPLES:'
      write(*,*)'    idecm=1 keep same sample interval    '
      write(*,*)'    idecm=2 output every other sample'
      write(*,*)'    idecm=-2 output samples between originals '
      write(*,*)' '
      write(*,*)'iantia  =0 no anti-alias filter for resample'
      write(*,*)'        =1 use anti-alias filter for resample'
      write(*,*)' '
      stop
      endif
      endif
      if(nargsx.lt.(2+karg)) then
      write(*,*)'  enter: name of input file'
      read(*,'(a13)') infil
      else
      call getarg(1,infil)
      endif
      call namechk(12,infil)
c
c
c...input a trace to get sample interval
      jrec=1
      iunit1=1
      icall1=0
c
c...count traces and check dimensions------------------------
      call CHKTRC(iunit1,icall1,npts,s1,fsamin,infil,
     +bar,invbar,ibar,ntrace,ndim)
      close(iunit1)
c
      tmaxx=float(npts-1)*fsamin
c
c
      if(nargsx.lt.(3+karg)) then
      write(*,'('' enter tmin'')')
      read(*,*) tmin1
      else
      call getarg(2,arg2)
      read(arg2,'(f10.4)') tmin1
      endif
c
      if(nargsx.lt.(4+karg)) then
      write(*,'('' original tmax= '',f10.4)') tmaxx
      write(*,'('' enter new tmax'')')
      read(*,*) tmax1
      else
      call getarg(3,arg3)
      read(arg3,'(f10.4)') tmax1
      endif
c
      if(nargsx.lt.(5+karg)) then
      write(*,'('' enter first trace number'')')
      read(*,*) ifirst
      else
      call getarg(4,arg4)
      read(arg4,'(i4)') ifirst
      endif
c
      if(nargsx.lt.(6+karg)) then
      write(*,'('' original number of traces= '',i5)') ntrace
      write(*,'('' enter last trace number'')')
      read(*,*) ilast
      else
      call getarg(5,arg5)
      read(arg5,'(i4)') ilast
      endif
c
      if(nargsx.lt.(7+karg)) then
      write(*,'('' original sample interval= '',f10.5)') fsamin 
      write(*,'(''    enter (+) decimation factor'')')
      write(*,'('' or enter (-) interpolation factor'')')
      read(*,*) idecm
      else
      call getarg(6,arg6)
      read(arg6,'(i4)') idecm
      endif
c
      if(idecm.gt.1) then
      if(nargsx.lt.(8+karg)) then
      write(*,'('' 0 = no anti-alias filter'')')
      write(*,'('' 1 = use anti-alias filter )'')')
      write(*,'(''  enter option '')')
      read(*,*) iafilt
      else
      call getarg(7,arg7)
      read(arg7,'(i4)') iafilt
      endif
      endif
      return
      end
c
c --------------------------------------------------------------
c | Basic Seismic Utilities                                    |
c | AUTHOR: P. MICHAELS  http://cgiss.boisestate.edu/~pm       | 
c | PROGRAM NAME: lstprm.f                                     |
c |------------------------------------------------------------|
c | echo check of input parameters, writes to a history file   |
c --------------------------------------------------------------
c
      subroutine LSTPRM(io11,ntrace,aplc,outlst,outfil,fsamin,fsamin2,
     +npts,npts2,infil,tmin1,tmax1,ifirst,ilast,idecm,iantia)
      character*4       aplc
      character*40      outlst,outfil,infil
c
c...open output listing file...
      io11=11
      write(outlst,'(a,a,''.lst'')') aplc,infil(1:4)
      open(io11,file=outlst,access='sequential')
      write(io11,*)'|---------------------------------|'
      write(io11,*)'| Copyright (C) 2017  P. Michaels |'
      write(io11,*)'|       All rights reserved       |'
      write(io11,*)'|see GNU General Public License   |'
      write(io11,*)'|---------------------------------|'
c
      write(io11,'('' Program bedt  '')')
      write(io11,'('' Input File: '',a)') infil
      write(io11,'('' Output File: '',a)') outfil
      write(io11,'(''  Number of traces='',i5)') ntrace
      write(io11,'(''  Parameters:  '')')
      write(io11,'(''     tmin='',f10.4)') tmin1
      write(io11,'(''     tmax='',f10.4)') tmax1
      write(io11,'(''     ifirst='',i5)')  ifirst
      write(io11,'(''     ilast='',i5)')   ilast
      write(io11,'(''     idecm='',i5)')   idecm
      write(io11,'(''     iantia='',i5)')  iantia
      write(io11,'(''     original sample interval='',f10.7)') fsamin
      write(io11,'(''     new sample interval='',f10.7)') fsamin2
      write(io11,'(''     original number of points='',i5)') npts
      write(io11,'(''     new number of points='',i5)') npts2
      write(io11,'(50(1h-))')
      write(io11,'(''    '')')
c
      return
      end

