c |$Id: bpic.f,v 1.1.1.1 2017/09/16 19:19:46 pm Exp $          |
c --------------------------------------------------------------
c | Basic Seismic Utilities (BSU)                              |
c | LANGUAGE: Fortran77                                        |
c | AUTHOR: P. MICHAELS  http://cgiss.boisestate.edu/~pm       | 
c | PROGRAM NAME: bmst.f                                       |
c | DIRECTORY: bsu-2.0.2/src/Fort/bsegy                        |
c |------------------------------------------------------------|
c |        Automatic first break picker using forward/reverse  |
c |        power ratio as a feature.                           |
c |        Picks are stored in a file bpicxxxx.pic             |
c |        Or inserts or downloads picks to/from headers       |
c |------------------------------------------------------------|
c |  NOTICE:                                                   |
c |  Copyright (C) 2017 Paul Michaels                          |
c |  <pm@cgiss.boisestate.edu>                                 |
c |  This program is free software; you can                    |
c |  redistribute it and/or modify it under the terms          |
c |  of the GNU General Public License as published            |
c |  by the Free Software Foundation; either version           |
c |  3 of the License, or (at your option) any later           |
c |  version.  This program is distributed in the              |
c |  hope that it will be useful, but                          |
c |  WITHOUT ANY WARRANTY; without even the implied            |
c |  warranty of MERCHANTABILITY or FITNESS FOR A              |
c |  PARTICULAR PURPOSE.  See the GNU General Public           |
c |  License for more details.                                 |
c |  You should have received a copy of the GNU                |
c |  General Public License along with this program;           |
c |  if not, write to the Free Software Foundation,            |
c |  Inc., 675 Mass Ave, Cambridge, MA 02139, USA.             |
c |------------------------------------------------------------|
c |  PARAMETERS:                                               |
c |  ndim = maximum number of samples in a seismic trace       |
c |------------------------------------------------------------|
c |  INCLUDE FILE REQUIRED:                                    |
c |  ../include/bsegy.inc    (modified segy header definitions)|
c |                (no reel header, floating point data)       |
c |------------------------------------------------------------|
c |  SUBROUTINES CALLED:                                       |
c | In-line===============                                     |
c |     GETPRM  get input parameters from command line or      |
c |             interactive query.  Should be inline with      |
c |             main program, specific to application.         |
c |     LSTPRM  echo check of input parameters from GETPRM.    |
c |             Values written to a history file (*.lst)       |
c | Located in bsu-2.0.2/src/sublibF4===========               |
c |     CHKTRC  get number of samples, sample interval, and    |
c |             number of traces in the input data file.       |
c |             Also returns values for progress bar, stdout   |
c |     BSEGIN  Reads a seismic trace.                         |
c |     BSEGOUT Write a seismic trace.                         |
c |     PLTBAR  Writes progress bar on stdout                  |
c |                                                            |
c --------------------------------------------------------------
c
      include '../include/bsegy.inc'
      parameter (ndim=32767,npdim=2500)
c
      real*4            s1(ndim),s2(ndim)
      character*4       aplc
      character*40      infil, outfil, outlst,picfil
      real*4            fpic(npdim)
      integer*4         ichan(npdim)
      common /header/ hd
c
c...assign I/O units, initialize....
      jrec=1
      iunit1=1
      iunit2=2
      icall1=0
      icall2=0
      io12=12
c
c...define application prefix
      write(aplc,'(''bpic'')')
c
c...get parameters-------------------------------------------
      call GETPRM(nargsx,infil,aplc,io12,ichan,fpic,npicks,
     +picfil,iswop1,mix,thresh,isw1,talign,bulkst,npdim)
      write(*,'('' npicks='',i5)') npicks
c
c...count traces and check dimensions------------------------
      call CHKTRC(iunit1,icall1,npts,s1,fsamin,infil,
     +bar,invbar,ibar,ntrace,ndim)
c
c...check for valid limits
      tmax1=float(npts-1)*fsamin
      if(talign.lt.0) talign=0.
      if(talign.gt.tmax1) talign=tmax1 
      if(iswop1.eq.1) then
        if(npicks.ne.ntrace) then
        write(*,*)' ABORT!!!-number of picks not equal to traces'
        stop
        endif
      endif
c
c...name outputfile
      write(outfil,'(a,a,''.seg'')') aplc(1:4),infil(1:4)
c
c...open pick file
      open(io12,file=picfil,access='sequential')
c
c...output listing-------------------------------------------
      call LSTPRM(io11,ntrace,aplc,outlst,outfil,ichan,fpic,
     +infil,picfil,iswop1,mix,thresh,isw1,talign,bulkst,npicks)
c
c...clear output file...
      open(10,file=outfil,access='direct',recl=240)
      close(10,status='delete')
c
      icall1=0
      icall2=0
      ntr=0
      do 60 jt=1,npts
   60 s2(jt)=0.
c
c...data loop=======================================....
c
      do 100 jrec=1,ntrace
c
c...input a trace
      call BSEGIN(iunit1,ndim,
     +icall1,npts,s1,fsamin,infil,jrec,iexit)
c
      ntr = ntr + 1
c...branch to pick,upload, or download
c===========================================
      if(iswop1.eq.0) then
c
c...compute power ratio signal....
      sumb=0.
      sumf=0.
      do 70 jt=1,mix
   70 sumb=s1(jt)*s1(jt)+sumb
      jtf=mix+2
      do 75 jt=1,mix
      sumf=s1(jtf)*s1(jtf)+sumf
   75 jtf=jtf+1
      jstart=mix+1
      jend=npts-mix
      do 80 jt=1,(jstart-1)
      if(sumb.ne.0.) then
      s2(jt)=sumf/sumb
      else
      s2(jt)=1.0
      endif
   80 continue
      do 85 jt=jstart,jend
      if(sumb.ne.0.) then
      s2(jt)=sumf/sumb
      else
      s2(jt)=1.0
      endif
      sumb=sumb-s1(jt-mix)**2+s1(jt)**2
      sumf=sumf-s1(jt+1)**2+s1(jt+mix+1)**2
   85 continue
      fmax1=-1.0E+20
      do 95 jt=1,npts
      if(abs(s2(jt)).gt.fmax1) fmax1=abs(s2(jt))
   95 continue
      do 98 jt=1,npts
   98 s2(jt)=s2(jt)/fmax1
c
c...threshold picker....
      pick=0.
      do 120, jt=1,npts
      if(s2(jt).gt.thresh) then
      pick=float(jt-1)*fsamin
      go to 110
      endif
  120 continue
  110 continue
c
c...write autopick result to file
      write(io12,'(2x,i5,2x,f10.4)') jrec, pick
c
c...output trace forward/reverse power....
      if(isw1.eq.1) then
      CALL BSEGOUT(iunit2,
     +icall2,npts,s2,fsamin,outfil,jrec)
      endif
c
c...output trace pick aligned...
      if(isw1.eq.2) then
      nalign=int(talign/fsamin+0.5)+1
      npick=int(pick/fsamin+0.5)+1
      do 125 jt=1,npts
  125 s2(jt)=0.0
      do 130 jt=1,(npts-npick+1)
      kt=jt+npick-nalign
      if(kt.lt.1 .or. kt.gt.npts) then
      s2(jt)=0.0
      else
      s2(jt)=s1(kt)
      endif
  130 continue
c
      CALL BSEGOUT(iunit2,
     +icall2,npts,s2,fsamin,outfil,jrec)
      endif
      else
c===================
      if(iswop1.eq.1) then
c.....mst=microseconds
      mst=int(fpic(jrec)*1.0E+06)
c
      CALL BSEGOUT(iunit2,
     +icall2,npts,s1,fsamin,outfil,jrec)
      endif
c
      if(iswop1.eq.2) then
      fpic(jrec)=float(mst)/(1.0E+06)
      ichan(jrec)=jrec
      write(io12,'(2x,i5,2x,f10.4)') ichan(jrec), fpic(jrec)
      endif
c
      endif
c===================================
c
c...display progress....
      call pltbar(bar,invbar,ibar,jrec,ntrace)
c
c
c...END LOOP========================================....
  100 continue
c
c
      write(*,'('' '')')
      write(6,240) ntrace,outfil,outlst
240   format(' Traces Processed = ',i5/
     +5x,'====> output in ',a/'====> listing in ',a)
      if(iswop1.ne.1) then
      write(*,'(''  =====> output pic file in '' ,a)') picfil
      endif
c
      close(io11)
      close(io12)
      close(iunit1)
      close(iunit2)
      stop
      end
c
c --------------------------------------------------------------
c | Basic Seismic Utilities                                    |
c | AUTHOR: P. MICHAELS  http://cgiss.boisestate.edu/~pm       | 
c | PROGRAM NAME: getprm.f                                     |
c |------------------------------------------------------------|
c | get input parameters or help                               |
c --------------------------------------------------------------
c
      subroutine GETPRM(nargsx,infil,aplc,io12,ichan,fpic,npicks,
     +picfil,iswop1,mix,thresh,isw1,talign,bulkst,npdim)
c
      character*40      infil,picfil
      character*4       aplc
      character*40      arg1,arg2,arg3,arg4,arg5,arg6
      real*4            fpic(*)
      integer*4         ichan(*)
c
c
c...get input/output file names...
c   |--------------------------------------------|
c   | file naming convention                     |
c   | application prefix 4 characters "bxxx"     |
c   | data set suffix 4 characters "dddd"        |
c   | file name = bxxxdddd.seg                   |
c   |--------------------------------------------|
      write(*,*)'|---------------------------------|'
      write(*,*)'| Copyright (C) 2017  P. Michaels |'
      write(*,*)'|       All rights reserved       |'
      write(*,*)'|see GNU General Public License   |'
      write(*,*)'|---------------------------------|'
c
ccc---LINUX VERSION
      nargsx=iargc()
      karg=-1
c...test for help
      if(nargsx.ge.(2+karg)) then
      call getarg(1,arg1)
      if(arg1(1:2).eq.'-h') then
      write(*,*)' '
      write(*,*) '|------------------------------------------|'
      write(*,*) '| Basic Seismic Utilities                  |'
      write(*,*) '| ONLINE HELP:                             |'
      write(*,*) '|------------------------------------------|'
      write(*,*) '| bpic: Pick first breaks AUTOPIC          |'
      write(*,*) '|       or READ/WRITE headers<--> picfile  |'
      write(*,*) '|------------------------------------------|'
      write(*,*)' '
      write(*,*)'-----IF ISWOP1=0----AUTOPIC-------------------------'
      write(*,*)
     +'bpic infile iswop1 mix thresh outsw talign'
      write(*,*)' '
      write(*,*) 'infile:    name of input file  '
      write(*,*)' iswop1:    option 0=autopic  '
      write(*,*)'                   1=UPload pic file to header '
      write(*,*)'                   2=DOWNload headers to pic file '
      write(*,*) 'mix:       = width of power smoother (ex. 7)   '
      write(*,*) 'thresh: =pick threshold  0<threshold<1.0   '
      write(*,*) '             (ex. 0.2)'
      write(*,*) 'outsw:     =output switch '
      write(*,*) '          0=none'
      write(*,*) '          1=forward/reverse power traces'
      write(*,*) '          2=input aligned on pick times + talign'
      write(*,*) 'talign     =alignment time for output '
      write(*,*)' '
      write(*,*)'-----IF ISWOP1 = 1 or 2-----PIC FILE----------------'
      write(*,*)' '
      write(*,*)
     +'bpic infile iswop1 picfil bulkst'
      write(*,*)' '
      write(*,*)'infil '
      write(*,*)' iswop1:   option 0=autopic  '
      write(*,*)'                  1=UPload pic file to header '
      write(*,*)'                  2=DOWNload headers to pic file '
      write(*,*)' picfil    =name of pic file  '
      write(*,*)' bulkst    =bulk static shift to add to picks in sec'
      write(*,*)' '
      stop
      endif
      endif
c
      if(nargsx.lt.(2+karg)) then
      write(*,*)'  enter: name of input file'
      read(*,'(a13)') infil
      else
      call getarg(1,infil)
      endif
      call namechk(12,infil)
      write(picfil,'(a,a,''.pic'')') aplc(1:4),infil(1:4)
c
c
c...get parameters....
      if(nargsx.lt.(3+karg)) then
      write(*,'('' Select Option: 0=Autopic Data '')')
      write(*,'(''                1=Load pic file into headers'')')
      write(*,'(''                2=Download header to a pic file'')')
      read(*,*) iswop1
      else
      call getarg(2,arg2)
      read(arg2,'(i1)') iswop1
      endif
c
c/////////////////////////
      if(iswop1.eq.0) then
      if(nargsx.lt.(4+karg)) then
      write(*,'(''  enter mix parameter (example 7)'')')
      read(*,*) mix
      else
      call getarg(3,arg3)
      read(arg3,'(i1)') mix
      endif
c
      if(mix.lt.1) mix=1
      if(nargsx.lt.(5+karg)) then
      write(*,'(''  enter threshold: 0<thresh<1  (example 0.2) '')')
      read(*,*) thresh
      else
      call getarg(4,arg4)
      read(arg4,'(f7.4)') thresh
      endif
c
      if(thresh.lt.0) thresh=.0000001
      if(thresh.gt.1) thresh=1.0
      if(nargsx.lt.(6+karg)) then
      write(*,'(''  Output SEGY Data Set Selection:'')')
      write(*,'(''    0= none'')')
      write(*,'(''    1= forward/reverse power'')')
      write(*,'(''    2= flatten input data on pick times'')')
      read(*,*) isw1
      else
      call getarg(5,arg5)
      read(arg5,'(i1)') isw1
      endif
c
      if(nargsx.lt.(7+karg)) then
      if(isw1.eq.2) then
      write(*,'(''    enter alignment time'')')
      read(*,*) talign
      endif
      else
      if(isw1.eq.2) then
      call getarg(6,arg6)
      read(arg6,'(f7.4)') talign
      endif
      endif
c
c///////////////////////
      else
c----------------
      if(iswop1.eq.1) then
      if(nargsx.lt.(4+karg)) then
      write(*,*)'  Enter name of pic file'
      read(*,'(a)') picfil
      else
      call getarg(3,picfil)
      endif
c
      if(nargsx.lt.(5+karg)) then
      write(*,*)'  Enter bulk static to be added in sec.'
      read(*,*) bulkst
      else
      call getarg(4,arg4)
      read(arg4,'(f7.4)') bulkst
      endif
c
      endif
c-----------------
      endif
c//////////////////////////////////
c
      if(iswop1.eq.1) then
c...open pick file
      open(io12,file=picfil,access='sequential')
      ipicnt=0
   12 ipicnt=ipicnt+1
      if(ipicnt.gt.npdim) then
      write(*,*)'  ABORT---dimensions exceeded--too many picks'
      stop
      endif
      read(io12,*,end=14) ichan(ipicnt),fpic(ipicnt)
      fpic(ipicnt)=fpic(ipicnt)+bulkst
      go to 12
   14 npicks=ipicnt-1
      close(io12)
      endif
c
      return
      end
c
c --------------------------------------------------------------
c | Basic Seismic Utilities                                    |
c | AUTHOR: P. MICHAELS  http://cgiss.boisestate.edu/~pm       | 
c | PROGRAM NAME: lstprm.f                                     |
c |------------------------------------------------------------|
c | echo check of input parameters, writes to a history file   |
c --------------------------------------------------------------
c
      subroutine LSTPRM(io11,ntrace,aplc,outlst,outfil,ichan,fpic,
     +infil,picfil,iswop1,mix,thresh,isw1,talign,bulkst,npicks)
      character*4       aplc
      character*40      outlst,outfil,infil,picfil
      real*4            fpic(*)
      integer*4         ichan(*)
c
c...open output listing file...
      io11=11
      write(outlst,'(a,a,''.lst'')') aplc,infil(1:4)
      open(io11,file=outlst,access='sequential')
      write(io11,*)'|---------------------------------|'
      write(io11,*)'| Copyright (C) 2017  P. Michaels |'
      write(io11,*)'|       All rights reserved       |'
      write(io11,*)'|see GNU General Public License   |'
      write(io11,*)'|---------------------------------|'
c
      write(io11,'('' Program bpic  '')')
      write(io11,'('' Input File: '',a)') infil
      write(io11,'('' Output File: '',a)') outfil
      write(io11,'('' Pic File: '',a)') picfil
c
      write(io11,'(''  Number of traces='',i5)') ntrace
      write(io11,'(''  Parameters:  '')')
      write(io11,'(''   iswop1='',i5)') iswop1
c
      if(iswop1.eq.0) then
      write(io11,'(''      mix='',i5)') mix
      write(io11,'(''      thresh='',f10.4)') thresh
      if(isw1.eq.0) then
      write(io11,'(''      no SEGY output'')')
      endif
      if(isw1.eq.1) then
      write(io11,'(''      SEGY output: forward/reverse power'')')
      endif
      if(isw1.eq.2) then
      write(io11,'(''      SEGY output: pick aligned data'')')
      write(io11,'(''      Alignment time='',f10.4)') talign
      endif
      endif
      if(iswop1.eq.1) then
      write(io11,'(/'' Bulk Static='',f10.5/)') bulkst
      write(io11,*)'       Pic File Contents+bulkst'
      do 15 j=1,npicks
   15 write(io11,'(8x,i5,2x,f10.4)') ichan(j),fpic(j)
      endif
      write(io11,'(50(1h-))')
      write(io11,'(''    '')')
c
      return
      end
