c |$Id: brdc.f,v 1.1.1.1 2017/09/16 19:19:46 pm Exp $          |
c --------------------------------------------------------------
c | Basic Seismic Utilities (BSU)                              |
c | LANGUAGE: Fortran77                                        |
c | AUTHOR: P. MICHAELS  http://cgiss.boisestate.edu/~pm       | 
c | PROGRAM NAME: brdc.f                                       |
c | DIRECTORY: bsu-2.0.2/src/Fort/bsegy                        |
c |------------------------------------------------------------|
c |         REMOVES DC LEVEL FROM A TRACE                      |
c |         (or linear trend)                                  |
c |------------------------------------------------------------|
c |  NOTICE:                                                   |
c |  Copyright (C) 2017 Paul Michaels                          |
c |  <pm@cgiss.boisestate.edu>                                 |
c |  This program is free software; you can                    |
c |  redistribute it and/or modify it under the terms          |
c |  of the GNU General Public License as published            |
c |  by the Free Software Foundation; either version           |
c |  3 of the License, or (at your option) any later           |
c |  version.  This program is distributed in the              |
c |  hope that it will be useful, but                          |
c |  WITHOUT ANY WARRANTY; without even the implied            |
c |  warranty of MERCHANTABILITY or FITNESS FOR A              |
c |  PARTICULAR PURPOSE.  See the GNU General Public           |
c |  License for more details.                                 |
c |  You should have received a copy of the GNU                |
c |  General Public License along with this program;           |
c |  if not, write to the Free Software Foundation,            |
c |  Inc., 675 Mass Ave, Cambridge, MA 02139, USA.             |
c |------------------------------------------------------------|
c |  PARAMETERS:                                               |
c |  ndim = maximum number of samples in a seismic trace       |
c |------------------------------------------------------------|
c |  INCLUDE FILE REQUIRED:                                    |
c |  ../include/bsegy.inc    (modified segy header definitions)|
c |                (no reel header, floating point data)       |
c |------------------------------------------------------------|
c |  SUBROUTINES CALLED:                                       |
c | In-line===============                                     |
c |     GETPRM  get input parameters from command line or      |
c |             interactive query.  Should be inline with      |
c |             main program, specific to application.         |
c |     LSTPRM  echo check of input parameters from GETPRM.    |
c |             Values written to a history file (*.lst)       |
c | Located in bsu-2.0.2/src/sublibF4===========               |
c |     CHKTRC  get number of samples, sample interval, and    |
c |             number of traces in the input data file.       |
c |             Also returns values for progress bar, stdout   |
c |     BSEGIN  Reads a seismic trace.                         |
c |     BSEGOUT Write a seismic trace.                         |
c |     PLTBAR  Writes progress bar on stdout                  |
c |                                                            |
c --------------------------------------------------------------
c
      include '../include/bsegy.inc'
      parameter (ndim=32767)
c
      real*4            s1(ndim),s2(ndim)
      character*4       aplc
      character*40      infil, outfil, outlst
      common /header/ hd
c
c...assign I/O units, initialize....
      jrec=1
      iunit1=1
      iunit2=2
      icall1=0
      icall2=0
c
c...define application prefix
      write(aplc,'(''brdc'')')
c
c...get parameters-------------------------------------------
      call GETPRM(nargsx,infil,
     +iswdc,usrdc)                
c
c...count traces and check dimensions------------------------
      call CHKTRC(iunit1,icall1,npts,s1,fsamin,infil,
     +bar,invbar,ibar,ntrace,ndim)
c
c...name outputfile
      write(outfil,'(a,a,''.seg'')') aplc(1:4),infil(1:4)
c
c...output listing-------------------------------------------
      call LSTPRM(io11,ntrace,aplc,outlst,outfil,
     +infil,iswdc,usrdc)                
c
c...clear output file...
      open(10,file=outfil,access='direct',recl=240)
      close(10,status='delete')
c
      icall1=0
      icall2=0
      ntr=0
c
c...data loop=======================================....

c
      do 100 jrec=1,ntrace
c
c
c...input a trace
      call BSEGIN(iunit1,ndim,
     +icall1,npts,s1,fsamin,infil,jrec,iexit)
c
      ntr = ntr + 1
c
c...
c
c...determine dc level if none provided....
      if(iswdc.eq.0) then
      fmean=usrdc
      do 80 jt=1,npts
  80  s2(jt)=s1(jt)-fmean
      else
c---------or else find DC level----
        if(iswdc.eq.1) then
         fmean=0.0
         do 75 jt=1,npts
  75     fmean=s1(jt)+fmean
         fmean=fmean/float(npts)
         do 81 jt=1,npts
  81     s2(jt)=s1(jt)-fmean
         write(io11,'(5x,i5,''  Mean='',E14.7)') ntr, fmean
        else
c------------or else remove trend--
c
c...least squares fit to find slope
c
         fmean=0.0
         do 77 jt=1,npts
  77     fmean=s1(jt)+fmean
         fmean=fmean/float(npts)
         sumx=0.
         sumx2=0.
         sumy=0.
         sumxy=0.
         fn=float(npts)
         do 105 jt=1,npts
         x=float(jt)
         y=s1(jt)
         xy=x*y
         sumx=sumx+x
         sumy=sumy+y
         sumxy=sumxy+xy
105      sumx2=sumx2+x*x
         slope=(fn*sumxy-sumy*sumx)/(fn*sumx2-sumx*sumx)
         yintrc=(sumx2*sumy-sumx*sumxy)/(fn*sumx2-sumx*sumx)
c
c...remove trend and evaluate fit
         yvar=0.
         do 107 jt=1,npts
         x=float(jt)
         y=x*slope+yintrc
         s2(jt)=s1(jt)-y
 107     yvar=(s1(jt)-y)**2+yvar
         yvar=yvar/fn
            if(yvar.ne.0.0) then
            yvar=sqrt(yvar/fn)
            endif
         write(io11,'(5x,i5,
     +   e14.7,2x,e14.7,2x,f10.4)') ntr,
     +   slope,yintrc,yvar
c
        endif
c----------------------------------
      endif
c
c...output trace
      iunit2=2
      call BSEGOUT(iunit2,
     +icall2,npts,s2,fsamin,outfil,jrec)
c
c...display progress....
      call pltbar(bar,invbar,ibar,jrec,ntrace)
c
c
c...END LOOP========================================....
  100 continue
c
c
      write(*,'('' '')')
      write(6,240) ntrace,outfil,outlst
240   format(' Traces Processed = ',i5/
     +5x,'====> output in ',a/'====> listing in ',a)
c
      close(io11)
      close(iunit1)
      close(iunit2)
      stop
      end
c
c --------------------------------------------------------------
c | Basic Seismic Utilities                                    |
c | AUTHOR: P. MICHAELS  http://cgiss.boisestate.edu/~pm       | 
c | PROGRAM NAME: getprm.f                                     |
c |------------------------------------------------------------|
c | get input parameters or help                               |
c --------------------------------------------------------------
c
      subroutine GETPRM(nargsx,infil,
     +iswdc,usrdc)                
c
      character*40      infil
      character*40      arg1,arg2,arg3
c
c
c...get input/output file names...
c   |--------------------------------------------|
c   | file naming convention                     |
c   | application prefix 4 characters "bxxx"     |
c   | data set suffix 4 characters "dddd"        |
c   | file name = bxxxdddd.seg                   |
c   |--------------------------------------------|
      write(*,*)'|---------------------------------|'
      write(*,*)'| Copyright (C) 2017  P. Michaels |'
      write(*,*)'|       All rights reserved       |'
      write(*,*)'|see GNU General Public License   |'
      write(*,*)'|---------------------------------|'
c
ccc---LINUX VERSION
      nargsx=iargc()
      karg=-1
c...test for help
      if(nargsx.ge.(2+karg)) then
      call getarg(1,arg1)
      if(arg1(1:2).eq.'-h') then
      write(*,*)' '
      write(*,*) '|------------------------------------------|'
      write(*,*) '| Basic Seismic Utilities     FORTRAN      |'
      write(*,*) '| ONLINE HELP:                             |'
      write(*,*) '|------------------------------------------|'
      write(*,*) '| brdc: Removes DC level from sample values|'
      write(*,*) '| in seismic traces, or linear trend       |'
      write(*,*) '|------------------------------------------|'
      write(*,*)' '
      write(*,*)
     +'brdc infile iswdc  usrdc '
      write(*,*)' '
      write(*,*) 'infile:  name of input file to remove DC     '
      write(*,*) 'iswdc:   0=user supplied DC level to remove  '
      write(*,*) '         1=determine dc level from each trace'
      write(*,*) '         2=remove linear trend measured from trace'
      write(*,*) 'usrdc:   = user supplied value of DC to remove '
      write(*,*)' '
      stop
      endif
      endif
      if(nargsx.lt.(2+karg)) then
      write(*,*)'  enter: name of input file'
      read(*,'(a13)') infil
      else
      call getarg(1,infil)
      call namechk(12,infil)

      if(nargsx.ge.(3+karg)) then
      call getarg(2,arg2)
      read(arg2,'(i1)') iswdc
      endif
      if(nargsx.ge.(4+karg)) then
      call getarg(3,arg3)
      read(arg3,'(f10.4)') usrdc
      endif
      endif
c
      if(nargsx.le.(2+karg)) then
      write(*,*)' Option:   0=user enter one level all traces'
      write(*,*)'           1=find DC from data, each trace'
      write(*,*)'           2=find linear trend from data, each trace'
      write(*,*)'  enter option above'
      read(*,*) iswdc
      endif
      if(nargsx.le.(3+karg)) then
      if(iswdc.eq.0) then
      write(*,*)'  enter user supplied dc level'
      read(*,*) usrdc
      endif
      endif
c
      return
      end
c
c --------------------------------------------------------------
c | Basic Seismic Utilities                                    |
c | AUTHOR: P. MICHAELS  http://cgiss.boisestate.edu/~pm       | 
c | PROGRAM NAME: lstprm.f                                     |
c |------------------------------------------------------------|
c | echo check of input parameters, writes to a history file   |
c --------------------------------------------------------------
c
      subroutine LSTPRM(io11,ntrace,aplc,outlst,outfil,
     +infil,iswdc,usrdc)                
      character*4       aplc
      character*40      outlst,outfil,infil
c
c...open output listing file...
      io11=11
      write(outlst,'(a,a,''.lst'')') aplc,infil(1:4)
      open(io11,file=outlst,access='sequential')
      write(io11,*)'|---------------------------------|'
      write(io11,*)'| Copyright (C) 2017  P. Michaels |'
      write(io11,*)'|       All rights reserved       |'
      write(io11,*)'|see GNU General Public License   |'
      write(io11,*)'|---------------------------------|'
c
      write(io11,'('' Program brdc  '')')
      write(io11,'('' Input File: '',a)') infil
      write(io11,'('' Output File: '',a)') outfil
      write(io11,'(''  Number of traces='',i5)') ntrace
      write(io11,'(''  Parameters:  '')')
      write(io11,'(''     option: 0=user DC, 1=data DC, 2=trend'')')
      write(io11,'(''               iswdc='',i5)') iswdc
      if(iswdc.eq.0) then
      write(io11,'(''     user supplied dc level='',e14.7)') usrdc
      endif
      if(iswdc.eq.2) then
      write(io11,'(5x,''Trace      Slope          Intercept'',
     +''        RMS'')')
      endif
      write(io11,'(50(1h-))')
      write(io11,'(''    '')')
c
      return
      end
