c |$Id: bred.f,v 1.1.1.1 2017/09/16 19:19:46 pm Exp $          |
c --------------------------------------------------------------
c | Basic Seismic Utilities (BSU)                              |
c | LANGUAGE: Fortran77                                        |
c | AUTHOR: P. MICHAELS  http://cgiss.boisestate.edu/~pm       | 
c | PROGRAM NAME: bred.f                                       |
c | DIRECTORY: bsu-2.0.2/src/Fort/bsegy                        |
c |------------------------------------------------------------|
c |       Applies a linear or nmo static shift to data         |
c |------------------------------------------------------------|
c |  NOTICE:                                                   |
c |  Copyright (C) 2017 Paul Michaels                          |
c |  <pm@cgiss.boisestate.edu>                                 |
c |  This program is free software; you can                    |
c |  redistribute it and/or modify it under the terms          |
c |  of the GNU General Public License as published            |
c |  by the Free Software Foundation; either version           |
c |  3 of the License, or (at your option) any later           |
c |  version.  This program is distributed in the              |
c |  hope that it will be useful, but                          |
c |  WITHOUT ANY WARRANTY; without even the implied            |
c |  warranty of MERCHANTABILITY or FITNESS FOR A              |
c |  PARTICULAR PURPOSE.  See the GNU General Public           |
c |  License for more details.                                 |
c |  You should have received a copy of the GNU                |
c |  General Public License along with this program;           |
c |  if not, write to the Free Software Foundation,            |
c |  Inc., 675 Mass Ave, Cambridge, MA 02139, USA.             |
c |------------------------------------------------------------|
c |  PARAMETERS:                                               |
c |  ndim = maximum number of samples in a seismic trace       |
c |------------------------------------------------------------|
c |  INCLUDE FILE REQUIRED:                                    |
c |  ../include/bsegy.inc    (modified segy header definitions)|
c |                (no reel header, floating point data)       |
c |------------------------------------------------------------|
c |  SUBROUTINES CALLED:                                       |
c | In-line===============                                     |
c |     GETPRM  get input parameters from command line or      |
c |             interactive query.  Should be inline with      |
c |             main program, specific to application.         |
c |     LSTPRM  echo check of input parameters from GETPRM.    |
c |             Values written to a history file (*.lst)       |
c | Located in bsu-2.0.2/src/sublibF4===========               |
c |     CHKTRC  get number of samples, sample interval, and    |
c |             number of traces in the input data file.       |
c |             Also returns values for progress bar, stdout   |
c |     BSEGIN  Reads a seismic trace.                         |
c |     BSEGOUT Write a seismic trace.                         |
c |     PLTBAR  Writes progress bar on stdout                  |
c |     VSHFT   Applies a static shift using interpolation.    |
c |                                                            |
c --------------------------------------------------------------
c
      include '../include/bsegy.inc'
      parameter (ndim=32767)
c
      real*4            s1(ndim),s2(ndim)
      character*4       aplc
      character*40      infil, outfil, outlst, outdat
      common /header/ hd
c
c...assign I/O units, initialize....
      jrec=1
      iunit1=1
      iunit2=2
      icall1=0
      icall2=0
c
c...define application prefix
      write(aplc,'(''bred'')')
c
c...get parameters-------------------------------------------
      call GETPRM(nargsx,infil,
     +iopt,vred,t0,tshift)      
      dt=fsamin
c
c...count traces and check dimensions------------------------
      call CHKTRC(iunit1,icall1,npts,s1,fsamin,infil,
     +bar,invbar,ibar,ntrace,ndim)
c
c...name outputfile
      write(outfil,'(a,a,''.seg'')') aplc(1:4),infil(1:4)
c
c...output listing-------------------------------------------
      call LSTPRM(io11,ntrace,aplc,outlst,outfil,
     +infil,iopt,vred,t0,tshift)      
c
c...clear output file...
      open(10,file=outfil,access='direct',recl=240)
      close(10,status='delete')
c
c
c...open *.dat file
      io12=12
      write(outdat,'(a,''.dat'')') outfil(1:8)
      open(io12,file=outdat,access='sequential')
c
      icall1=0
      icall2=0
      ntr=0
c
c...data loop=======================================....
c
      do 100 jrec=1,ntrace
c
c...input a trace
      call BSEGIN(iunit1,ndim,
     +icall1,npts,s1,fsamin,infil,jrec,iexit)
c
      ntr = ntr + 1
c
c...find coordinates of shot and receiver
      call findxyz(sx,sy,sz,sd,rx,ry,rz,range,rangex)
c
      if (iopt.eq.1) then
c <<<<<<<<<<<<<< LINEAR OPTION >>>>>>>>>>>>>>>>>>>>>>>>>>
c
c...compute static
      ashift=range/vred - tshift
      pshift=-ashift
      write(io11,'(5x,i5,'' static (seconds)='',f10.4,
     +2x,''offset='',f10.4)') ntr,pshift,range
      write(io12,'(5x,f7.0,2x,f10.5)') float(ntr),pshift
      else
c /////////////// NMO OPTION /////////////////////////////
c
c...compute static
      txx=range/vred
      ashift=sign(1.,vred)*sqrt((txx*txx) + t0*t0) - tshift
      pshift=-ashift
      write(io11,'(5x,i5,'' static (seconds)='',f10.4,
     +2x,''offset='',f10.4)') ntr,pshift,range
      write(io12,'(5x,f7.0,2x,f10.5)') float(ntr),pshift
      endif
c<<<<<<<<<<<<//////////////////////////>>>>>>>>>>>>>>>>>>>
c
c...to undo an application, negate vred and tshift
c
      call VSHFT(npts,s1,s2,ashift,fsamin)
c
c
c...output trace
      CALL BSEGOUT(iunit2,
     +icall2,npts,s2,fsamin,outfil,jrec)
c
c...display progress....
      call pltbar(bar,invbar,ibar,jrec,ntrace)
c
c
c...END LOOP========================================....
  100 continue
c
c
      write(*,'('' '')')
      write(6,240) ntrace,outfil,outlst
240   format(' Traces Processed = ',i5/
     +5x,'====> output in ',a/'====> listing in ',a)
c
      close(io11)
      close(iunit1)
      close(iunit2)
      stop
      end
c
c --------------------------------------------------------------
c | Basic Seismic Utilities                                    |
c | AUTHOR: P. MICHAELS  http://cgiss.boisestate.edu/~pm       | 
c | PROGRAM NAME: getprm.f                                     |
c |------------------------------------------------------------|
c | get input parameters or help                               |
c --------------------------------------------------------------
c
      subroutine GETPRM(nargsx,infil,
     +iopt,vred,t0,tshift)      
c
      character*40      infil
      character*40      arg1,arg2,arg3,arg4,arg5
c
c
c...get input/output file names...
c   |--------------------------------------------|
c   | file naming convention                     |
c   | application prefix 4 characters "bxxx"     |
c   | data set suffix 4 characters "dddd"        |
c   | file name = bxxxdddd.seg                   |
c   |--------------------------------------------|
      write(*,*) '|---------------------------------|'
      write(*,*) '| Copyright (C) 2017  P. Michaels |'
      write(*,*) '|       All rights reserved       |'
      write(*,*) '|see GNU General Public License   |'
      write(*,*) '|---------------------------------|'
c
ccc---LINUX VERSION
      nargsx=iargc()
      karg=-1
c...test for help
      if(nargsx.ge.(2+karg)) then
      call getarg(1,arg1)
      if(arg1(1:2).eq.'-h') then
      write(*,*) ' '
      write(*,*) '|------------------------------------------|'
      write(*,*) '| Basic Seismic Utilities     FORTRAN      |'
      write(*,*) '| ONLINE HELP:                             |'
      write(*,*) '|------------------------------------------|'
      write(*,*) '| bred: Apply either linear or reflection  |'
      write(*,*) '| NMO (hyperbolic) static shifts to data.  |'
      write(*,*) '|------------------------------------------|'
      write(*,*) ' '
      write(*,*)
     +'bred  infile  iopt  vred  t0  tshift  '
      write(*,*) ' '
      write(*,*) 'infile:  =name of input file '
      write(*,*) 'iopt:    =statics option '
      write(*,*) '        1=linear trend '
      write(*,*) '        0=hyperbolic NMO trend '
      write(*,*) 'vred:    =reduction velocity '
      write(*,*) 't0:      =zero offset time (iopt=0 ONLY) '
      write(*,*) 'tshift:  =bulk static shift '
      write(*,*) ' '
      stop
      endif
      endif
      if(nargsx.lt.(2+karg)) then
      write(*,*)'  enter: name of input file'
      read(*,'(a13)') infil
      else
      call getarg(1,infil)
      endif
      call namechk(12,infil)
c
c...enter parameters
      if(nargsx.lt.(3+karg)) then
      write(*,'(''  1= linear reduction'')')
      write(*,'(''  0= NMO    reduction'')')
      write(*,'(''  enter option:'')')
      read(*,*) iopt
      else
      call getarg(2,arg2)
      read(arg2,'(i1)') iopt
      endif
c
      if(iopt.eq.1) then
      if(nargsx.lt.(4+karg)) then
      write(*,'('' enter reduction velocity '')')
      read(*,*) vred
      else
      call getarg(3,arg3)
      read(arg3,'(f10.0)') vred
      endif
      if(nargsx.lt.(5+karg)) then
      write(*,'('' enter bulk time shift (seconds)'')')
      read(*,*) tshift
      else
      call getarg(4,arg4)
      read(arg4,'(f10.4)') tshift
      endif
c
      else
      if(nargsx.lt.(4+karg)) then
      write(*,'('' enter NMO  velocity '')')
      read(*,*) vred
      else
      call getarg(3,arg3)
      read(arg3,'(f10.0)') vred
      endif
      if(nargsx.lt.(5+karg)) then
      write(*,'('' enter zero offset time, T0 (seconds)'')')
      read(*,*) t0
      else
      call getarg(4,arg4)
      read(arg4,'(f10.4)') t0
      endif
      if(nargsx.lt.(6+karg)) then
      write(*,'('' enter bulk time shift (seconds)'')')
      read(*,*) tshift
      else
      call getarg(5,arg5)
      read(arg5,'(f10.4)') tshift
      endif
c
      endif
      return
      end
c
c --------------------------------------------------------------
c | Basic Seismic Utilities                                    |
c | AUTHOR: P. MICHAELS  http://cgiss.boisestate.edu/~pm       | 
c | PROGRAM NAME: lstprm.f                                     |
c |------------------------------------------------------------|
c | echo check of input parameters, writes to a history file   |
c --------------------------------------------------------------
c
      subroutine LSTPRM(io11,ntrace,aplc,outlst,outfil,
     +infil,iopt,vred,t0,tshift)      
      character*4       aplc
      character*40      outlst,outfil,infil
c
c...open output listing file...
      io11=11
      write(outlst,'(a,a,''.lst'')') aplc,infil(1:4)
      open(io11,file=outlst,access='sequential')
      write(io11,*) '|---------------------------------|'
      write(io11,*) '| Copyright (C) 2017  P. Michaels |'
      write(io11,*) '|       All rights reserved       |'
      write(io11,*) '|see GNU General Public License   |'
      write(io11,*) '|---------------------------------|'
c
      write(io11,'('' Program bred'')')
      write(io11,'('' Input File: '',a)') infil
      write(io11,'('' Output File: '',a)') outfil
c
      write(io11,'(''  Number of traces='',i5)') ntrace
      write(io11,'(''  Parameters:   '')')
      if(iopt.eq.1) then
      write(io11,'(''     Linear Statics Applied  '')')
      write(io11,'(''     Reduction Velocity= '',f10.2)') vred
      write(io11,'(''     Bulk Static Shift= '',f10.4)') tshift
      else
      write(io11,'(''     NMO Hyperbolic Statics Applied  '')')
      write(io11,'(''     Reduction Velocity= '',f10.2)') vred
      write(io11,'(''     Zero Offset Time= '',f10.4)')  t0
      write(io11,'(''     Bulk Static Shift= '',f10.4)') tshift
      endif
      write(io11,'(''  '')')
      write(io11,'(50(1h-))')
      write(io11,'(''    '')')
c
      return
      end
