c |$Id: brot.f,v 1.1.1.1 2017/09/16 19:19:46 pm Exp $          |
c --------------------------------------------------------------
c | Basic Seismic Utilities (BSU)                              |
c | LANGUAGE: Fortran77                                        |
c | AUTHOR: P. MICHAELS  http://cgiss.boisestate.edu/~pm       | 
c | PROGRAM NAME: brot.f                                       |
c | DIRECTORY: bsu-2.0.2/src/Fort/bsegy                        |
c |------------------------------------------------------------|
c | Rotation of 2 components about an orthogonal axis          |
c | User specifies signals to rotate, and hence axis.          |
c |------------------------------------------------------------|
c |  NOTICE:                                                   |
c |  Copyright (C) 2017 Paul Michaels                          |
c |  <pm@cgiss.boisestate.edu>                                 |
c |  This program is free software; you can                    |
c |  redistribute it and/or modify it under the terms          |
c |  of the GNU General Public License as published            |
c |  by the Free Software Foundation; either version           |
c |  3 of the License, or (at your option) any later           |
c |  version.  This program is distributed in the              |
c |  hope that it will be useful, but                          |
c |  WITHOUT ANY WARRANTY; without even the implied            |
c |  warranty of MERCHANTABILITY or FITNESS FOR A              |
c |  PARTICULAR PURPOSE.  See the GNU General Public           |
c |  License for more details.                                 |
c |  You should have received a copy of the GNU                |
c |  General Public License along with this program;           |
c |  if not, write to the Free Software Foundation,            |
c |  Inc., 675 Mass Ave, Cambridge, MA 02139, USA.             |
c |------------------------------------------------------------|
c |  PARAMETERS:                                               |
c |  ndim = maximum number of samples in a seismic trace       |
c |------------------------------------------------------------|
c |  INCLUDE FILE REQUIRED:                                    |
c |  ../include/bsegy.inc    (modified segy header definitions)|
c |                (no reel header, floating point data)       |
c |------------------------------------------------------------|
c |  SUBROUTINES CALLED:                                       |
c | In-line===============                                     |
c |     GETPRM  get input parameters from command line or      |
c |             interactive query.  Should be inline with      |
c |             main program, specific to application.         |
c |     LSTPRM  echo check of input parameters from GETPRM.    |
c |             Values written to a history file (*.lst)       |
c | Located in bsu-2.0.2/src/sublibF4===========               |
c |     CHKTRC  get number of samples, sample interval, and    |
c |             number of traces in the input data file.       |
c |             Also returns values for progress bar, stdout   |
c |     BSEGIN  Reads a seismic trace.                         |
c |     BSEGOUT Write a seismic trace.                         |
c |     PLTBAR  Writes progress bar on stdout                  |
c |                                                            |
c --------------------------------------------------------------
c
      include '../include/bsegy.inc'
      parameter (ndim=32767)
c
      real*4            s1(ndim),s2(ndim),sx1(ndim),sy1(ndim)
      character*4       aplc
      character*40      infil, outfil, outlst
      common /header/ hd
c
c...assign I/O units, initialize....
      jrec=1
      iunit1=1
      iunit2=2
      icall1=0
      icall2=0
c
c...define application prefix
      write(aplc,'(''brot'')')
c
c...get parameters-------------------------------------------
      call GETPRM(nargsx,infil,
     +itrx,itry,iopt,iangle)    
c
c...count traces and check dimensions------------------------
      call CHKTRC(iunit1,icall1,npts,s1,fsamin,infil,
     +bar,invbar,ibar,ntrace,ndim)
c
c...name outputfile
      write(outfil,'(a,a,''.seg'')') aplc(1:4),infil(1:4)
c
c...output listing-------------------------------------------
      call LSTPRM(io11,ntrace,aplc,outlst,outfil,
     +infil,itrx,itry,iangle)    
c
c...clear output file...
      open(10,file=outfil,access='direct',recl=240)
      close(10,status='delete')
c
      icall1=0
      icall2=0
      ntr=0
c
c...data loop=======================================....
c
c...determine rotation angle for iopt 1 or 2
      if(iopt.eq.1) then
      jrec=3
      CALL BSEGIN(iunit1,ndim,
     +icall1,npts,s1,fsamin,infil,jrec,iexit)
      iswcaz=geoazi
      jrec=6
      CALL BSEGIN(iunit1,ndim,
     +icall1,npts,s1,fsamin,infil,jrec,iexit)
      irefaz=geoazi
      iangle=iswcaz-irefaz
      endif
c
      if(iopt.eq.2) then
      jrec=2
      CALL BSEGIN(iunit1,ndim,
     +icall1,npts,s1,fsamin,infil,jrec,iexit)
      iswcaz=geoazi
      jrec=5
      CALL BSEGIN(iunit1,ndim,
     +icall1,npts,s1,fsamin,infil,jrec,iexit)
      irefaz=geoazi
      iangle=iswcaz-irefaz
      endif
c
      write(io11,'(1x,''iangle='',i5,'' degrees'')')
     +iangle
c
      rangle=float(iangle)*(3.1415926/180.)
c
c
c...read traces, compute/////////////////////////////////////////....
c
      cosa=cos(rangle)
      sina=sin(rangle)
c
c...rotate x-axis and y-axis traces
c
c...input x-axis trace
      jrec=itrx
      CALL BSEGIN(iunit1,ndim,
     +icall1,npts,sx1,fsamin,infil,jrec,iexit)
c
c...input y-axis trace
      jrec=itry
      CALL BSEGIN(iunit1,ndim,
     +icall1,npts,sy1,fsamin,infil,jrec,iexit)
c
c...comput new x-axis trace
      do 101 jt=1,npts
      s2(jt)=sx1(jt)*cosa - sy1(jt)*sina
  101 continue
c
c...output new x-axis trace
      iunit2=2
      jrec=itrx
c...call bsegin to restore headers
      CALL BSEGIN(iunit1,ndim,
     +icall1,npts,s1,fsamin,infil,jrec,iexit)
c...update geoazi header
      geoazi=geoazi-iangle
      if(geoazi.gt.359) geoazi=geoazi-360
      CALL BSEGOUT(iunit2,
     +icall2,npts,s2,fsamin,outfil,jrec)
c
c...comput new y-axis trace
      do 102 jt=1,npts
      s2(jt)=sx1(jt)*sina + sy1(jt)*cosa
  102 continue
c
c...output new y-axis trace
      jrec=itry
c...call bsegin to restore headers
      CALL BSEGIN(iunit1,ndim,
     +icall1,npts,s1,fsamin,infil,jrec,iexit)
c...update geoazi header
      geoazi=geoazi-iangle
      if(geoazi.gt.359) geoazi=geoazi-360
      CALL BSEGOUT(iunit2,
     +icall2,npts,s2,fsamin,outfil,jrec)
c
      jrec = 0
      ntr = 0
c
c
      do 100 jrec=1,ntrace
c
c...input a trace
      call BSEGIN(iunit1,ndim,
     +icall1,npts,s1,fsamin,infil,jrec,iexit)
c
      ntr = ntr + 1
c
      if(jrec.ne.itrx .and. jrec.ne.itry) then
c
c...output trace
      iunit2=2
      call BSEGOUT(iunit2,
     +icall2,npts,s1,fsamin,outfil,jrec)
      endif
c
c...display progress....
      call pltbar(bar,invbar,ibar,jrec,ntrace)
c
c...END LOOP========================================....
  100 continue
c
c
      write(*,'('' '')')
      write(6,240) ntrace,outfil,outlst
240   format(' Traces Processed = ',i5/
     +5x,'====> output in ',a/'====> listing in ',a)
c
      close(io11)
      close(iunit1)
      close(iunit2)
      stop
      end
c
c --------------------------------------------------------------
c | Basic Seismic Utilities                                    |
c | AUTHOR: P. MICHAELS  http://cgiss.boisestate.edu/~pm       | 
c | PROGRAM NAME: getprm.f                                     |
c |------------------------------------------------------------|
c | get input parameters or help                               |
c --------------------------------------------------------------
c
      subroutine GETPRM(nargsx,infil,
     +itrx,itry,iopt,iangle)    
c
      character*40      infil
      character*40      arg1,arg2,arg3,arg4,arg5
c
c
c...get input/output file names...
c   |--------------------------------------------|
c   | file naming convention                     |
c   | application prefix 4 characters "bxxx"     |
c   | data set suffix 4 characters "dddd"        |
c   | file name = bxxxdddd.seg                   |
c   |--------------------------------------------|
      write(*,*)'|---------------------------------|'
      write(*,*)'| Copyright (C) 2017  P. Michaels |'
      write(*,*)'|       All rights reserved       |'
      write(*,*)'|see GNU General Public License   |'
      write(*,*)'|---------------------------------|'
c
ccc---LINUX VERSION
      nargsx=iargc()
      karg=-1
c...test for help
      if(nargsx.ge.(2+karg)) then
      call getarg(1,arg1)
      if(arg1(1:2).eq.'-h') then
      write(*,*)' '
      write(*,*) '|------------------------------------------|'
      write(*,*) '| Basic Seismic Utilities     FORTRAN      |'
      write(*,*) '| ONLINE HELP:                             |'
      write(*,*) '|------------------------------------------|'
      write(*,*) '| brot: Rotate mulit-component trace data  |'
      write(*,*) '|      (use headers or user supplied value)|'
      write(*,*) '|------------------------------------------|'
      write(*,*)' '
      write(*,*)
     +'brot infil itrx itry iopt iangle  '
      write(*,*)' '
      write(*,*)'infil:   =input file name '
      write(*,*)'itrx:    =trace number for x-axis (example, 2) '
      write(*,*)'itry:    =trace number for y-axis (example, 3) '
      write(*,*)'iopt:    =rotation option '
      write(*,*)'        0=user supplied rotation angle  '
      write(*,*)'        1=Transverse Alignment: chan 3 aligned to 6'
      write(*,*)'        2=Radial Alignment:     chan 2 aligned to 5 '
      write(*,*)'iangle:  =user supplied rotation angle from x-axis '
      write(*,*)' '
      stop
      endif
      endif
c
c...enter parameters
      if(nargsx.lt.(2+karg)) then
      write(*,*)'  enter: name of input file'
      read(*,'(a13)') infil
      else
      call getarg(1,infil)
      endif
      call namechk(12,infil)
c
      if(nargsx.lt.(3+karg)) then
      write(*,*)' enter trace number, x-coord'
      read(*,*) itrx
      else
      call getarg(2,arg2)
      read(arg2,'(i3)') itrx
      endif
c
      if(nargsx.lt.(4+karg)) then
      write(*,*)' enter trace number, y-coord'
      read(*,*) itry
      else
      call getarg(3,arg3)
      read(arg3,'(i3)') itry
      endif
c
      if(nargsx.lt.(5+karg)) then
      write(*,*)' enter option:'
      write(*,*)'     0= user supplied rotation angle'
      write(*,*)'     1= Transverse Alignment (ch.3 with ch.6)'
      write(*,*)'     2= Radial     Alignment (ch.2 with ch.5)'
      read(*,*) iopt
      else
      call getarg(4,arg4)
      read(arg4,'(i3)') iopt
      endif
c
      if(nargsx.lt.(6+karg) .and. iopt.eq.0) then
      write(*,*)' enter rotation angle (deg.) 0<angle<360 '
      read(*,*) iangle
      else
      call getarg(5,arg5)
      read(arg5,'(i3)') iangle
      endif
      return
      end
c
c --------------------------------------------------------------
c | Basic Seismic Utilities                                    |
c | AUTHOR: P. MICHAELS  http://cgiss.boisestate.edu/~pm       | 
c | PROGRAM NAME: lstprm.f                                     |
c |------------------------------------------------------------|
c | echo check of input parameters, writes to a history file   |
c --------------------------------------------------------------
c
      subroutine LSTPRM(io11,ntrace,aplc,outlst,outfil,
     +infil,itrx,itry,iangle)    
      character*4       aplc
      character*40      outlst,outfil,infil
c
c...open output listing file...
      io11=11
      write(outlst,'(a,a,''.lst'')') aplc,infil(1:4)
      open(io11,file=outlst,access='sequential')
      write(io11,*)'|---------------------------------|'
      write(io11,*)'| Copyright (C) 2017  P. Michaels |'
      write(io11,*)'|       All rights reserved       |'
      write(io11,*)'|see GNU General Public License   |'
      write(io11,*)'|---------------------------------|'
c
      write(io11,'('' Program brot  '')')
      write(io11,'('' Input File: '',a)') infil
      write(io11,'('' Output File: '',a)') outfil
c
      write(io11,'(''  Number of traces='',i5)') ntrace
      write(io11,'(''  Parameters:  '')')
      write(io11,'(''  Number of traces='',i5)') ntrace
      write(io11,'(''  Parameters:    '')')
      write(io11,'(''     Trace: x-axis = '',i5)') itrx
      write(io11,'(''     Trace: y-axis = '',i5)') itry
      write(io11,'(''     Rotation Angle= '',i5)') iangle
      write(io11,'(50(1h-))')
      write(io11,'(''    '')')
c
      return
      end
