c |$Id: bxcr.f,v 1.1.1.1 2017/09/16 19:19:46 pm Exp $          |
c --------------------------------------------------------------
c | Basic Seismic Utilities (BSU)                              |
c | LANGUAGE: Fortran77                                        |
c | AUTHOR: P. MICHAELS  http://cgiss.boisestate.edu/~pm       | 
c | PROGRAM NAME: bxcr.f                                       |
c | DIRECTORY: bsu-2.0.2/src/Fort/bsegy                        |
c |------------------------------------------------------------|
c |         CROSS CORRELATES  TWO BSEGY DATA                   |
c |         SETS AND OUTPUTS BSEGY CROSS CORRELATION           |
c |------------------------------------------------------------|
c |  NOTICE:                                                   |
c |  Copyright (C) 2017 Paul Michaels                          |
c |  <pm@cgiss.boisestate.edu>                                 |
c |  This program is free software; you can                    |
c |  redistribute it and/or modify it under the terms          |
c |  of the GNU General Public License as published            |
c |  by the Free Software Foundation; either version           |
c |  3 of the License, or (at your option) any later           |
c |  version.  This program is distributed in the              |
c |  hope that it will be useful, but                          |
c |  WITHOUT ANY WARRANTY; without even the implied            |
c |  warranty of MERCHANTABILITY or FITNESS FOR A              |
c |  PARTICULAR PURPOSE.  See the GNU General Public           |
c |  License for more details.                                 |
c |  You should have received a copy of the GNU                |
c |  General Public License along with this program;           |
c |  if not, write to the Free Software Foundation,            |
c |  Inc., 675 Mass Ave, Cambridge, MA 02139, USA.             |
c |------------------------------------------------------------|
c |  PARAMETERS:                                               |
c |  ndim = maximum number of samples in a seismic trace       |
c |------------------------------------------------------------|
c |  INCLUDE FILE REQUIRED:                                    |
c |  ../include/bsegy.inc    (modified segy header definitions)|
c |                (no reel header, floating point data)       |
c |------------------------------------------------------------|
c |  SUBROUTINES CALLED:                                       |
c | In-line===============                                     |
c |     GETPRM  get input parameters from command line or      |
c |             interactive query.  Should be inline with      |
c |             main program, specific to application.         |
c |     LSTPRM  echo check of input parameters from GETPRM.    |
c |             Values written to a history file (*.lst)       |
c | Located in bsu-2.0.2/src/sublibF4===========               |
c |     CHKTRC  get number of samples, sample interval, and    |
c |             number of traces in the input data file.       |
c |             Also returns values for progress bar, stdout   |
c |     BSEGIN  Reads a seismic trace.                         |
c |     BSEGOUT Write a seismic trace.                         |
c |     PLTBAR  Writes progress bar on stdout                  |
c |     XCOR    Computes cross or auto correlation, 2 sided    |
c |                                                            |
c | Bug Fix: infil-->infil2 in cross correlation line 140 here |
c |                                                            |
c --------------------------------------------------------------
c
c
      include '../include/bsegy.inc'
      parameter (ndim=32767)
c
      real*4            s1(ndim),s2(ndim),s3(ndim)
      character*4       aplc
      character*40      infil, outfil, outlst, infil2
      common /header/ hd
c
c...assign I/O units, initialize....
      jrec=1
      iunit1=1
      iunit2=2
      iunit3=3
      icall1=0
      icall2=0
      icall3=0
c
c...define application prefix
      write(aplc,'(''bxcr'')')
c
c...get parameters-------------------------------------------
      call GETPRM(nargsx,infil,
     +infil2,t1,t2,tlagmx)
c
c...count traces and check dimensions------------------------
      call CHKTRC(iunit1,icall1,npts,s1,fsamin,infil,
     +bar,invbar,ibar,ntrace,ndim)
c
      if(infil.ne.infil2) then
      call CHKTRC(iunit2,icall2,npts2,s2,fsamin2,infil2,
     +bar2,invbar2,ibar2,ntrace2,ndim)
      else
      ntrace2=ntrace
      fsamin2=fsamin
      npts2=npts
      endif
c
c...check validity of parameters
      tmax1=fsamin*(npts-1)
      if(t1.lt.0) t1=0
      if(t2.gt.tmax1) t2=tmax1
      tzero=tlagmx/2
c
      if(ntrace.ne.ntrace2) then
      write(*,*) 'ABORT: Number Traces Not Equal'
      stop
      endif
      if(fsamin.ne.fsamin2) then
      write(*,*) 'ABORT: Sample intervals not equal'
      stop
      endif
      if(npts.ne.npts2) then
      write(*,*) 'ABORT: Number of samples not equal'
      stop
      endif
c
c...name outputfile
      write(outfil,'(a,a,''.seg'')') aplc(1:4),infil(1:4)
c
c...output listing-------------------------------------------
      call LSTPRM(io11,ntrace,aplc,outlst,outfil,
     +infil,infil2,t1,t2,tlagmx,tzero)
c
c...clear output file...
      open(10,file=outfil,access='direct',recl=240)
      close(10,status='delete')
c
      icall1=0
      icall2=0
      icall3=0
      ntr=0
c
c...data loop=======================================....
c
      do 100 jrec=1,ntrace
c
c...input a trace
      CALL BSEGIN(iunit1,ndim,
     +icall1,npts,s1,fsamin,infil,jrec,iexit)
c
      if(infil.ne.infil2) then
      CALL BSEGIN(iunit2,ndim,
     +icall2,npts,s2,fsamin,infil2,jrec,iexit)
      else
      do 37 jt=1,npts
   37 s2(jt)=s1(jt)
      endif
c
      ntr = ntr + 1
c
      call xcor(npts,t1,t2,tlagmx,tzero,fsamin,s1,s2,s3)
      npts3=tlagmx/fsamin
c
c...output trace
      CALL BSEGOUT(iunit3,
     +icall3,npts3,s3,fsamin,outfil,jrec)
c
c...display progress....
      call pltbar(bar,invbar,ibar,jrec,ntrace)
c
c
c...END LOOP========================================....
  100 continue
c
c
      write(*,'('' '')')
      write(6,240) ntrace,outfil,outlst
240   format(' Traces Processed = ',i5/
     +5x,'====> output in ',a/'====> listing in ',a)
c
      close(io11)
      close(iunit1)
      close(iunit2)
      close(iunit3)
      stop
      end
c --------------------------------------------------------------
c | Basic Seismic Utilities                                    |
c | AUTHOR: P. MICHAELS  http://cgiss.boisestate.edu/~pm       | 
c | PROGRAM NAME: getprm.f                                     |
c |------------------------------------------------------------|
c | get input parameters or help                               |
c --------------------------------------------------------------
c
      subroutine GETPRM(nargsx,infil,
     +infil2,t1,t2,tlagmx)
c
      character*40      infil,infil2
      character*40      arg1,arg3,arg4,arg5
c
c
c...get input/output file names...
c   |--------------------------------------------|
c   | file naming convention                     |
c   | application prefix 4 characters "bxxx"     |
c   | data set suffix 4 characters "dddd"        |
c   | file name = bxxxdddd.seg                   |
c   |--------------------------------------------|
      write(*,*) '|---------------------------------|'
      write(*,*) '| Copyright (C) 2017  P. Michaels |'
      write(*,*) '|       All rights reserved       |'
      write(*,*) '|see GNU General Public License   |'
      write(*,*) '|---------------------------------|'
c
ccc---LINUX VERSION
      nargsx=iargc()
      karg=-1
c...test for help
      if(nargsx.ge.(2+karg)) then
      call getarg(1,arg1)
      if(arg1(1:2).eq.'-h') then
      write(*,*) ' '
      write(*,*) '|------------------------------------------|'
      write(*,*) '| Basic Seismic Utilities     FORTRAN      |'
      write(*,*) '| ONLINE HELP:                             |'
      write(*,*) '|------------------------------------------|'
      write(*,*) '| bxcr: Cross-correlate two bsegy data     |'
      write(*,*) '| sets and output the result in bsegy fmt. |'
      write(*,*) '|------------------------------------------|'
      write(*,*) ' '
      write(*,*)
     +'bxcr infile1 infile2 t1 t2 tlagmx '
      write(*,*) ' '
      write(*,*) 'infile1: name of input file #1 '
      write(*,*) 'infile2: name of input file #2 '
      write(*,*) 't1:     =start time of cross correlation gate (sec.)'
      write(*,*) 't2:     =end time of cross correlation gate (sec.) '
      write(*,*) 'tlagmx: =maximum cross correlation lag time (sec.)'
      write(*,*) ' '
      stop
      endif
      endif
c
      if(nargsx.lt.(2+karg)) then
      write(*,*)'  enter: name of input file #1'
      read(*,'(a13)') infil
      else
      call getarg(1,infil)
      endif
      call namechk(12,infil)
c
      if(nargsx.lt.(3+karg)) then
      write(*,*)'  enter: name of input file #2'
      read(*,'(a13)') infil2
      else
      call getarg(2,infil2)
      endif
      call namechk(12,infil2)
c
      if(nargsx.lt.(4+karg)) then
      write(*,'(''  enter t1 = start time of correlation gate'')')
      read(*,*) t1
      else
      call getarg(3,arg3)
      read(arg3,'(f7.4)') t1
      endif
      if(nargsx.lt.(5+karg)) then
      write(*,'(''  enter t2 = end time of correlation gate'')')
      read(*,*) t2
      else
      call getarg(4,arg4)
      read(arg4,'(f7.4)') t2
      endif
c
      if(nargsx.lt.(6+karg)) then
      write(*,'(''  enter tlagmx = maximum lag in seconds'')')
      read(*,*) tlagmx
      else
      call getarg(5,arg5)
      read(arg5,'(f7.4)') tlagmx
      endif
      return
      end
c
c --------------------------------------------------------------
c | Basic Seismic Utilities                                    |
c | AUTHOR: P. MICHAELS  http://cgiss.boisestate.edu/~pm       | 
c | PROGRAM NAME: lstprm.f                                     |
c |------------------------------------------------------------|
c | echo check of input parameters, writes to a history file   |
c --------------------------------------------------------------
c
      subroutine LSTPRM(io11,ntrace,aplc,outlst,outfil,
     +infil,infil2,t1,t2,tlagmx,tzero)
      character*4       aplc
      character*40      outlst,outfil,infil,infil2
c
c...open output listing file...
      io11=11
      write(outlst,'(a,a,''.lst'')') aplc,infil(1:4)
      open(io11,file=outlst,access='sequential')
      write(io11,*) '|---------------------------------|'
      write(io11,*) '| Copyright (C) 2017  P. Michaels |'
      write(io11,*) '|       All rights reserved       |'
      write(io11,*) '|see GNU General Public License   |'
      write(io11,*) '|---------------------------------|'
c
      write(11,'('' Program bxcr'')')
      write(11,'('' Input File #1: '',a)') infil
      write(11,'('' Input File #2: '',a)') infil2
      write(11,'('' Output File: '',a)') outfil
      write(io11,'(''  Number of traces='',i5)') ntrace
      write(io11,'(''  Parameters:  '')')
      write(io11,'(''   t1 =  '',f10.2)') t1  
      write(io11,'(''   t2 =  '',f10.2)') t2  
      write(io11,'(''   tlagmx='',f10.2)') tlagmx
      write(io11,'(''     Zero lag time on output='',f10.4)') tzero
      write(io11,'(50(1h-))')
      write(io11,'(''    '')')
c
      return
      end

