c |$Id: genbhodV.f,v 1.1.1.1 2017/09/16 19:19:46 pm Exp $      |
c --------------------------------------------------------------
c | Basic Seismic Utilities (BSU)                              |
c | LANGUAGE: Fortran77                                        |
c | AUTHOR: P. MICHAELS  http://cgiss.boisestate.edu/~pm       | 
c | PROGRAM NAME: genbhodV.f  EXPERIMENTAL                     |
c | DIRECTORY: bsu-2.0.2/src/Fort/bsegy                        |
c |------------------------------------------------------------|
c |   This is an interactive program that generates the scripts|
c |  for a batch processing which will lead to determination   |
c |  of down-hole tool orientation by PCA analysis.  This is   |
c |  EXPERIMENTAL, and is designed for a VERTICAL IMPACT source|
c |  which generates Rayleigh and converted S-waves.           |
c | !! NO DIFFERENCE - SUM ENHANCEMENT SINCE VERTICAL IMPACT !!|
c |                                                            |
c |    There are many assumptions here.  First, it is assumed  |
c |  that a stationary 3-C reference phone is at the surface.  |
c |  Note that the bowspring indicates the direction of the    |
c |  R-component down-hole, but that is only for my tool.  Your|
c |  tool may be different.  Take it appart to find out.       |
c |    Further, it is assumed that the field records have 6    |
c |  or possibly 7 channels.  Channels are defined as          |
c |     CH     Description                                     |
c |     1      Down-hole vertical   (up)                       |
c |     2      Down-hole radial (aligned with bowspring)       |
c |     3      Down-hole transverse (orthogonal to 2)          |
c |     4      Reference vertical   (up)                       |
c |     5      Reference radial     (0 degree azimuth)         |
c |     6      Reference transverse (270 degree azimuth)       |
c |     7      Load Cell signal (if present, ignored in flows) |
c |                                                            |
c |    This is not really a problem if you acquire data with a |
c |  different setup in the field, but you must work out your  |
c |  own sign conventions and prompts.  The program bscl.f     |
c |  is used to scale the data on the vertical component before|
c |  PCA analysis on the horizontal (this is not essential, but|
c |  rather convenient).  Bhod does the Principal Component    |
c |  Analysis (PCA).  It is also assumed that you observed the |
c |  tool orientation on exiting the borehole with the down-   |
c |  hole phone, and can use that to resolve the 180 degree    |
c |  ambiguity inherent in the PCA analysis (Michaels, 2001)   |
c |    WHAT IS THE THEORY HERE?  The Rayleigh wave gives a good|
c |  horizontal signal, and in the context of the experimental |
c |  set up, one may be able to infer tool orientation.  It    |
c |  appears to work well with the reference phone, but your   |
c |  mileage will vary down-hole, depending on the depth to    |
c |  which the Rayleigh motion extends.  Nodal points will also|
c |  limit one at some depths (so extraplolate manually).      |
c |                                                            |
c |  Michaels, P., 2001, "Use of Principal Component Analysis  |
c |   to Determine Down-Hole Tool Orientation and Enhance      |
c |   SH-Waves", Journ. Environmental and Engineering          |
c |   Geophysics, Vol. 6, No. 4, December, p175-183.           |
c |                                                            |
c |------------------------------------------------------------|
c |  NOTICE:                                                   |
c |  Copyright (C) 2017 Paul Michaels                          |
c |  <pm@cgiss.boisestate.edu>                                 |
c |  This program is free software; you can                    |
c |  redistribute it and/or modify it under the terms          |
c |  of the GNU General Public License as published            |
c |  by the Free Software Foundation; either version           |
c |  3 of the License, or (at your option) any later           |
c |  version.  This program is distributed in the              |
c |  hope that it will be useful, but                          |
c |  WITHOUT ANY WARRANTY; without even the implied            |
c |  warranty of MERCHANTABILITY or FITNESS FOR A              |
c |  PARTICULAR PURPOSE.  See the GNU General Public           |
c |  License for more details.                                 |
c |  You should have received a copy of the GNU                |
c |  General Public License along with this program;           |
c |  if not, write to the Free Software Foundation,            |
c |  Inc., 675 Mass Ave, Cambridge, MA 02139, USA.             |
c |------------------------------------------------------------|
c
c
      character*8  prfx
      call GETPRM(nargsx)
      write(*,*) 'enter 1char_ALPHA PREFIX '
      read(*,'(a)') prfx
      do 5, j=1,8
      if(prfx(j:j).eq.' ') then
      klast=j-1
      go to 6
      endif
    5 continue
    6 continue
      write(*,*) ' enter FIRST FILE NUMBER (<=3digits)'
      read(*,*) ifrst
c
      write(*,*) ' enter LAST FILE NUMBER (<=3digits)'
      read(*,*) ilast
c
      write(*,*) ' enter azimuth of bowspring(R-comp)'
      read(*,*) azctl
c
c.============================DOWNHOLE 
      open(unit=2,access='sequential',file='gobhodo')
      open(unit=3,access='sequential',file='gorunbhod')
      nlines=(ilast-ifrst)+1

      do 10 j=1,nlines
      jn=j+ifrst-1
      write(2,'(''bscl '',a,i3.3,''.seg 1 1 3'')') prfx(1:klast),jn
      write(2,100) prfx(1:klast), jn,jn
  100 format('mv  bscl',a,i3.3,'.seg  h',i3.3,'.seg')
c
c saz=0.0 on following for source impact south of borehole, converted waves
c saz=180.0 if source were north of borehole (see bhod.f argument list)
      write(3,'(''bhod h'',i3.3,
     +''.seg 2 3 50 0.0'',f6.1,'' +90.0'')')
     +jn,azctl
   10 continue
      write(*,*) ' OUTPUT====> Downhole: gobhodo'
      close(2)
      close(3)
c.============================REFERENCE
      open(unit=2,access='sequential',file='gobhodoR')
      open(unit=3,access='sequential',file='gorunbhodR')
      nlines=(ilast-ifrst)+1

      do 20 j=1,nlines
      jn=j+ifrst-1
      write(2,'(''bscl '',a,i3.3,''.seg 4 1 3'')') prfx(1:klast),jn
      write(2,200) prfx(1:klast), jn,jn
  200 format('mv bscl',a,i3.3,'.seg  r',i3.3,'.seg')
c
      write(3,'(''bhod r'',i3.3,
     +''.seg 5 6 50 0.0'',f6.1,'' -90.0'')')
     +jn,azctl
   20 continue
      write(*,*) ' OUTPUT====> Reference: gobhodoR'
      close(2)
      close(3)
c.============================
      write(*,*) ' OUTPUT====> Downhole: gorunbhod'
      write(*,*) ' OUTPUT====> Reference: gorunbhodR'
      write(*,*) ' ----------------------------------------'
      write(*,*) 'REMEMBER to change permissions on the    '
      write(*,*) 'above files to execute. '
      write(*,*) ' ----------------------------------------'
      write(*,*) ' IF examining the Down-hole Phone        '
      write(*,*) '                                         '
      write(*,*) ' 1. Run gobhodo in directory with 6 chan '
      write(*,*) ' records (3 down, 3 reference phones)    '
      write(*,*) '                                         '
      write(*,*) ' 2. Run gorunbhod in directory with files'
      write(*,*) ' that are named hxxx.seg                 '
      write(*,*) '                                         '
      write(*,*) ' ----------------------------------------'
      write(*,*) ' IF examining the Reference Phone        '
      write(*,*) '                                         '
      write(*,*) ' 1. Run gobhodoR in the directory with   '
      write(*,*) ' the 6 channel records.                  '
      write(*,*) '                                         '
      write(*,*) ' 2. Run gorunbhodR in the directory with '
      write(*,*) ' files that are named rxxx.seg           '
      write(*,*) '   EXPERIMENTAL APPROACH on Rayleigh Wave'
      write(*,*) ' ----------------------------------------'
      stop
      end
c
c --------------------------------------------------------------
c | Basic Seismic Utilities                                    |
c | AUTHOR: P. MICHAELS  http://cgiss.boisestate.edu/~pm       | 
c | PROGRAM NAME: getprm.f                                     |
c |------------------------------------------------------------|
c | get input parameters or help                               |
c --------------------------------------------------------------
c
      subroutine GETPRM(nargsx)
c
      character*40      arg1
c
      write(*,*) '|---------------------------------|'
      write(*,*) '| Copyright (C) 2017  P. Michaels |'
      write(*,*) '|       All rights reserved       |'
      write(*,*) '|see GNU General Public License   |'
      write(*,*) '|---------------------------------|'
      write(*,*) ' '
      write(*,*) 'WARNING: !! '
      write(*,*) 'See Source Code, genbhodV.f, or BSU documentation '
      write(*,*) '(man pages and BSU user Guide)                    '
      write(*,*) 'before you use this program. It is hardwired for'
      write(*,*) 'a specific type of acquisition.'
      write(*,*) ' '
c
ccc---LINUX VERSION
      nargsx=iargc()
      karg=-1
c...test for help
      if(nargsx.ge.(2+karg)) then
      call getarg(1,arg1)
      write(*,*) ' '
      write(*,*) '|------------------------------------------|'
      write(*,*) '| Basic Seismic Utilities     FORTRAN      |'
      write(*,*) '| ONLINE HELP:       GENBHODV  EXPERIMENTAL|'
      write(*,*) '|------------------------------------------|'
      write(*,*) '| There are NO COMMAND LINE ARGUMENTS for  |'
      write(*,*) '| this program, it is interactive.         |'
      write(*,*) '|------------------------------------------|'
      write(*,*) ' '
      endif
      return
      end
