c |$Id: genref.f,v 1.2 2018/04/07 20:43:15 pm Exp $        |
c --------------------------------------------------------------
c | Basic Seismic Utilities (BSU)                              |
c | LANGUAGE: Fortran77                                        |
c | AUTHOR: P. MICHAELS  http://cgiss.boisestate.edu/~pm       | 
c | PROGRAM NAME: genref.f                                     |
c | DIRECTORY: bsu-2.0.2/src/Fort/bsegy                        |
c |------------------------------------------------------------|
c | genref.for   program to generate data sets for REFLECTION  |
c |            CDP Roll-a-long pattern             GEOMETERY   |
c |------------------------------------------------------------|
c |  NOTICE:                                                   |
c |  Copyright (C) 2017 Paul Michaels                          |
c |  <pm@cgiss.boisestate.edu>                                 |
c |  This program is free software; you can                    |
c |  redistribute it and/or modify it under the terms          |
c |  of the GNU General Public License as published            |
c |  by the Free Software Foundation; either version           |
c |  3 of the License, or (at your option) any later           |
c |  version.  This program is distributed in the              |
c |  hope that it will be useful, but                          |
c |  WITHOUT ANY WARRANTY; without even the implied            |
c |  warranty of MERCHANTABILITY or FITNESS FOR A              |
c |  PARTICULAR PURPOSE.  See the GNU General Public           |
c |  License for more details.                                 |
c |  You should have received a copy of the GNU                |
c |  General Public License along with this program;           |
c |  if not, write to the Free Software Foundation,            |
c |  Inc., 675 Mass Ave, Cambridge, MA 02139, USA.             |
c |------------------------------------------------------------|
c | This file is designed for Bison Format seismic file        |
c | recording.  It can be used for surface seismic acquisition |
c | geometry setting when an *.nez file is not available       |
c | from the surveyor.  This program generates bash scripts    |
c | similar to the down-hole equivalent (genvsp.f)             |
c |------------------------------------------------------------|
c | 1. Generates a *.nez file (TOPCON Compatible) 5A12         |
c |    index  y-coord  x-coord  z-coord  name                  |
c |                                                            |
c | 2. Generates a file, GEOM    with calls to TOPCON Program  |
c |                                                            |
c | 3. Generates a file, GO1    (calls bhed)                   |
c |                                                            |
c | 4. Generates a file, GEOM2          (calls go1)            |
c --------------------------------------------------------------
c
      parameter (ndim=200,mdim=500)
      real*4            xs(ndim),ys(ndim),zs(ndim)
      real*4            xr(ndim),yr(ndim),zr(ndim)
      character*4       lineid
      character*5       spname
      character*5       vpname,prefix
      character*10      neznm
      character*8       fname,lname,bname
      integer*2         sazi,sver,i180,i90,i270,razi,tazi
c
      i180=180
      i90=90
      i270=270
c
c...interactive:
      io11=11
      write(*,*) '|---------------------------------|'
      write(*,*) '| Copyright (C) 2017  P. Michaels |'
      write(*,*) '|       All rights reserved       |'
      write(*,*) '|see GNU General Public License   |'
      write(*,*) '|---------------------------------|'
c
      write(*,*) ' '
      write(*,*) ' CDP Roll-a-long Pattern Generator '
      write(*,*) ' Bison Format Data'
      write(*,*) ' '
      write(*,*) '-----------SOURCES-----------------------------'
      write(*,*) '  Enter 6 char. name for nez file (ex. STP001)'
      read(*,'(a)') neznm(1:6)
      write(neznm(7:10),'(''.nez'')')
      open(io11,file=neznm,access='sequential')
c
      write(*,*) '  Enter 4 char. LINEID'
      read(*,'(a)') lineid
c
      write(*,*) '  Enter Z-Datum: Elevation'
      read(*,*) zdatum
c
c...Sources
      write(*,*) '  Enter number of shots '
      read(*,*) nshot
      js=1
      write(*,*) '  Enter Shot Record Names 8char: First'
      read(*,'(a,a)') fname
c
      write(*,*) '  Enter Shot Record Names 8char: Last'
      read(*,'(a,a)') lname
c
      write(*,*) ' Enter First Shot Station Number'
      read(*,*) ifsn
c
      write(*,*) '  Enter First Source: x, y, z'
      read(*,*) xs(1),ys(1),zs(1)
      zs(1)=zdatum+zs(1)
c
      write(*,*) '  Enter Last Source: x, y, z'
      read(*,*) xs(nshot),ys(nshot),zs(nshot)
      zs(nshot)=zdatum+zs(nshot)
c
      delxs=(xs(nshot)-xs(1))/float(nshot-1)
      delys=(ys(nshot)-ys(1))/float(nshot-1)
      delzs=(zs(nshot)-zs(1))/float(nshot-1)
c
c...generate evenly spaced shot coordinates
      do 10 js=1,nshot
      if(js.gt.1 .or. js.lt.nshot) then
      xs(js)=xs(1)+delxs*float(js-1)
      ys(js)=ys(1)+delys*float(js-1)
      zs(js)=zs(1)+delzs*float(js-1)
      endif
      write(spname,'(''SP'',i3.3)') js
      write(io11,901) js,ys(js),xs(js),zs(js),spname
  901 format(i4.4,8x,f10.4,2x,f10.4,2x,f10.4,2x,a)
   10 continue
c
c///////////////Receivers
      write(*,*) ' Enter number of receivers in a shot gather'
      read(*,*) nrec1
      write(*,*) ' Enter TOTAL NUMBER of stations on line'
      read(*,*) nrec
c
      write(*,*) '  Enter First Geophone Station: x, y, z'
      read(*,*) xr(1),yr(1),zr(1)
      zr(1)=zdatum+zr(1)
c
      write(*,*) '  Enter Last Geophone Station: x, y, z'
      read(*,*) xr(nrec),yr(nrec),zr(nrec)
      zr(nrec)=zdatum+zr(nrec)
c
      delxr=(xr(nrec)-xr(1))/float(nrec-1)
      delyr=(yr(nrec)-yr(1))/float(nrec-1)
      delzr=(zr(nrec)-zr(1))/float(nrec-1)
c
      write(*,*) ' Enter first shot NEAR GEOPHONE station'
      read(*,*) jfirst
      write(*,*) ' Enter first shot FAR GEOPHONE station'
      read(*,*) jlast
c
      do 20 js=1,nrec
      if(js.gt.1 .or. js.lt.nshot) then
      xr(js)=xr(1)+delxr*float(js-1)
      yr(js)=yr(1)+delyr*float(js-1)
      zr(js)=zr(1)+delzr*float(js-1)
      endif
      write(vpname,'(''VP'',i3.3)') js
      write(io11,900) js,yr(js),xr(js),zr(js),vpname
  900 format(i4.4,8x,f10.4,2x,f10.4,2x,f10.4,2x,a)
   20 continue
c
      close(io11)
c==================================================
c
c...GEOM FILE
      open(io11,file='geom',access='sequential')
c
      jcount=jfirst
      sdepth=0.
      nchan=nrec1
      razi=0
      tazi=0
      sazi=0
      sver=0
c
c...shot loop
      read(fname(6:8),'(i3)') ifcnt
      read(lname(6:8),'(i3)') ilcnt
      read(fname(1:5),'(a)') prefix
      ncount=ifcnt
      jifsn=ifsn
      do 70 js=1,nshot
c
      write(bname,'(a,i3.3)') prefix,ncount
      ifchn=jcount
      ilchn=jcount+nchan-1
c
      write(io11,902) neznm,bname,lineid,sdepth,jifsn,nchan,
     1ifchn,ilchn,ncount,sdepth,sazi,sver,razi,tazi
c
  902 format('topcon ',a,1x,a,1x,a,1x,f3.1,1x,i2,1x,i2,1x,
     1i3.3,1x,i3.3,1x,i3,1x,f3.0,1x,i3,1x,i3,1x,i3,1x,i3)
c
      jcount=jcount+1
      ncount=ncount+1
      jifsn=jifsn+1
c
   70 continue
c
      close(io11)
c
c...GEOM2 FILE
      open(io11,file='geom2',access='sequential')
      read(fname(6:8),'(i3)') ifcnt
      read(lname(6:8),'(i3)') ilcnt
      do 200 jc=ifcnt,ilcnt
      write(io11,903) jc
  903 format(' go1 ',i3.3)
  200 continue
      close(io11)
c
c...GO1.BAT FILE
      open(io11,file='go1',access='sequential')
      write(io11,904) prefix
  904 format(' bis2seg ',a,'$1')
      write(io11,905) prefix,prefix
  905 format(' bhed ',a,'$1.seg ',a,'$1.xyz 0')
      write(io11,906) prefix(1:4),prefix(1:1)
  906 format('mv bhed',a,'.seg ',a,'$1.seg')
      write(io11,907) prefix
  907 format('rm ',a,'$1.seg')
      stop
      end
