c |$Id: genvsp.f,v 1.1.1.1 2017/09/16 19:19:46 pm Exp $        |
c --------------------------------------------------------------
c | Basic Seismic Utilities (BSU)                              |
c | LANGUAGE: Fortran77                                        |
c | AUTHOR: P. MICHAELS  http://cgiss.boisestate.edu/~pm       | 
c | PROGRAM NAME: genvsp.f                                     |
c | DIRECTORY: bsu-2.0.2/src/Fort/bsegy                        |
c |------------------------------------------------------------|
c | Generates bash scripts to set geometry for down-hole data  |
c | acquired with a bison seismograph.                         |
c |------------------------------------------------------------|
c |  NOTICE:                                                   |
c |  Copyright (C) 2017 Paul Michaels                          |
c |  <pm@cgiss.boisestate.edu>                                 |
c |  This program is free software; you can                    |
c |  redistribute it and/or modify it under the terms          |
c |  of the GNU General Public License as published            |
c |  by the Free Software Foundation; either version           |
c |  3 of the License, or (at your option) any later           |
c |  version.  This program is distributed in the              |
c |  hope that it will be useful, but                          |
c |  WITHOUT ANY WARRANTY; without even the implied            |
c |  warranty of MERCHANTABILITY or FITNESS FOR A              |
c |  PARTICULAR PURPOSE.  See the GNU General Public           |
c |  License for more details.                                 |
c |  You should have received a copy of the GNU                |
c |  General Public License along with this program;           |
c |  if not, write to the Free Software Foundation,            |
c |  Inc., 675 Mass Ave, Cambridge, MA 02139, USA.             |
c |------------------------------------------------------------|
c | genvsp.f     program to generate data sets for VSP geometry|
c |------------------------------------------------------------|
c | 1. Generates a *.nez file (TOPCON Compatible) 5A12         |
c |    index  y-coord  x-coord  z-coord  name                  |
c |                                                            |
c | 2. Generates a file, "geom"  with calls to TOPCON Program  |
c |                                                            |
c | 3. Generates a file, "go1" which calls bis2seg and bhed    |
c |                                                            |
c | 4. Generates a file, "geom2" which calls go1 for each      |
c |   shot record in the survey.                               |
c |------------------------------------------------------------|
c | NOTES:                                                     |
c | This program will work with 6 or 7 channel field records   |
c | This program will handle SEG-2 or BISON format data        |
c |------------------------------------------------------------|
c | With BISON, output files include: *.nez, geom, geom2, go1  |
c | Files geom2 and go1 are needed for an extra step, running  |
c | bis2seg which ultimately produces the *.seg files          |
c |------------------------------------------------------------|
c | With SEG-2, output files include: *.nez, geom              |
c | Fewer files since topcon2 is a C-language program designed |
c | to use both header and data in the SEG-2 files, going      |
c | directly to the desired *.seg file output.                 |
c |                                                            |
c --------------------------------------------------------------
c
      parameter (ndim=10,mdim=500)
      real*4            xs(ndim),ys(ndim),zs(ndim),dz(ndim)
      real*4            zmax(ndim),zmin(ndim),signd(ndim)
      integer*2         ichnlsw
      character*4       spname(ndim),lineid
      character*5       prefix
      character*6       vpname
      character*10      neznm
      character*8       fname(ndim),lname(ndim),bname
      integer*2         sazi(ndim),sver(ndim),i180,i90,i270,razi,tazi
c
      i180=180
      i90=90
      i270=270
c
c...interactive:
      io11=11
      write(*,*) '|---------------------------------|'
      write(*,*) '| Copyright (C) 2017  P. Michaels |'
      write(*,*) '|       All rights reserved       |'
      write(*,*) '|see GNU General Public License   |'
      write(*,*) '|---------------------------------|'
c
      write(*,*) ' '
      write(*,*) ' Down-hole VSP  Pattern Generator '
      write(*,*) ' For Setting Geometry '
      write(*,*) ' Handles both Bison and SEG-2 File Formats '
      write(*,*) ' '
c...set switch for channel order
      write(*,*) ' Set Channel Order Switch'
      write(*,*) '  1=ascending  1,2,3=downhole 4,5,6=reference'
      write(*,*) ' -1=descending 6,5,4=downhole 3,2,1=reference'
      write(*,*) '  2=ascending  1,2,3=down 4,5,6=ref,7=load_cell'
      write(*,*) ' -2=descending 7=load_cell,6,5,4=down 3,2,1=ref'
      write(*,*) ' '
      read(*,*) ichnlsw 
      write(*,*) ' '
      write(*,*) '-----------BOREHOLE----------------------------'
      write(*,*) '  Enter 6 char. name for nez file (ex. STP001)'
      read(*,'(a)') neznm(1:6)
      write(neznm(7:10),'(''.nez'')')
      open(io11,file=neznm,access='sequential')
c
      write(*,*) '  Enter 4 char. LINEID'
      read(*,'(a)') lineid
c
      write(*,*) '  Enter Z-Datum: Casing Elevation'
      read(*,*) zdatum
c
      write(*,*) ' '
      write(*,*) '  BOREHOLE LOCATION:'
      write(*,*) '  Borehole is origin of the local coordinate system'
      write(*,*) '  Source and Reference phone locations are x,y '
      write(*,*) '  relative to borehole.'
      write(*,*) ' '
      write(*,*) '  Following entries will shift every x,y input to'
      write(*,*) '  a final global coordinate system:'
      write(*,*) '  Enter Global x-coord. of borehole'
      read(*,*)  gxcord
      write(*,*) '  Enter Global y-coord. of borehole'
      read(*,*)  gycord
c
c...Sources
      write(*,*) '  Enter number of sources '
      read(*,*) nshot
      do 10 js=1,nshot
      write(*,*) 'FOR THIS SOURCE:'
      write(*,*) '  Enter Shot Record Names 8char: First'
      read(*,'(a,a)') fname(js)
      write(*,*) '  Enter Shot Record Names 8char: Last'
      read(*,'(a,a)') lname(js)
c
      write(*,'(5x,a,a)') fname(js),lname(js)
      write(*,*) '  Enter Source: x, y, z_sub_CE (positive down)'
      read(*,*) xs(js),ys(js),zs(js)
c
c...convert to global coord.
      xs(js)=xs(js)+gxcord
      ys(js)=ys(js)+gycord
      zs(js)=zdatum-zs(js)
c
      write(spname(js),'(''SP'',i2.2)') js
      write(io11,900) js,ys(js),xs(js),zs(js),spname(js)
  900 format(i4.4,8x,f10.4,2x,f10.4,2x,f10.4,2x,a)
      write(*,*)'  Enter Source Polarization: azi, ver'
      read(*,*) sazi(js),sver(js)
   10 continue
c
c...Reference Phones
      write(*,*) ' '
      write(*,*) '-----------REFERENCE RECEIVER------------------'
      write(*,*) '  Enter Reference: x, y, z_sub_CE (positive down)'
      read(*,*) refx,refy,refz
c
c...convert to global coord.
      refx=refx+gxcord
      refy=refy+gycord
      refz=zdatum-refz
c
      write(*,*) '  Enter Reference Polarizations: R-azi, T-azi'
      read(*,*) razi,tazi
c
c...Borehole Phones
      write(*,*) ' '
      write(*,*) '-----------BOREHOLE PHONES---------------------'
c
      write(*,*) '  Enter Bulk Shift (Added To Geophone Depth ONLY)'
      read(*,*) zshift
c
      do 20 js=1,nshot
      write(*,901) spname(js),sazi(js),sver(js)
  901 format('  For Shot: ',a,'  AZI=',i3,' VER=',i3/)
c
      write(*,*) '  Enter Station Spacing:  dz'
      read(*,*) dz(js)
      write(*,*) '  Enter First Station Depth:  zmax'
      read(*,*) zmax(js)
      write(*,*) '  Enter Last Station Depth:  zmin'
      read(*,*) zmin(js)
      signd(js)=+1.0
      if(zmax(js).lt.zmin(js)) signd(js)=-1.0
   20 continue
c
c...write rest of nez file=============================
      jcount=1
      yr=0.0+gycord
      xr=0.0+gxcord
      do 30 js=1,nshot
      nrec=(signd(js)*(zmax(js)-zmin(js))/dz(js)+0.5)+1.0
      zelev=zdatum-zmax(js)-zshift
c
      do 40 jz=1,nrec
c
      do 50 jj=1,3
      write(vpname,'(''VP'',i4.4)') jcount
      write(io11,900) jcount,yr,xr,zelev,vpname
      jcount=jcount+1
   50 continue
c
      do 60 jj=1,3
      write(vpname,'(''VP'',i4.4)') jcount
      write(io11,900) jcount,refy,refx,refz,vpname
      jcount=jcount+1
   60 continue
c
      if(int(abs(ichnlsw)).ge.2) then
c
c...write out load_cell coordinate, at shot point=data channel
      write(vpname,'(''VP'',i4.4)') jcount
      write(io11,900) jcount,ys(js),xs(js),zs(js),vpname
      jcount=jcount+1
      endif
c
      zelev=zelev+dz(js)*signd(js)
   40 continue
   30 continue
c
      close(io11)
c
c...GEOM FILE
      open(io11,file='geom',access='sequential')
c
      jcount=1
      sdepth=0.
      if (int(abs(ichnlsw)).le.1) then
      nchan=6
      else
      nchan=7
      endif
c
c...shot loop
      do 70 js=1,nshot
      nrec=(signd(js)*(zmax(js)-zmin(js))/dz(js)+0.5)+1.0
      write(*,'(2x,''Number of receivers = '',i5)') nrec
      write(*,'(45(1h-))')
      write(*,*) ' CHECK DATA TYPE '
      write(*,*) '                 '
c
c...determine if bison or seg-2 data
      if(fname(js)(6:8).eq."dat") then
      read(fname(js)(1:4),'(i4)') ifcnt
      read(lname(js)(1:4),'(i4)') ilcnt
      idtype=0 
      else
      read(fname(js)(6:8),'(i3)') ifcnt
      read(lname(js)(6:8),'(i3)') ilcnt
      read(fname(js)(1:5),'(a)') prefix
      idtype=1
      endif
c
      if(idtype.eq.0) then
      write(*,*) ' Files like 1001.dat detected, ID=SEG-2'
      else
      write(*,*) ' Files like XXXX0001 detected, ID=BISON'
      endif
      write(*,*) ' Is above ID Correct, or overide needed?'
      write(*,*) ' 1=YES correct    0=NO incorrect '
      read(*,*) iswovr
      if(iswovr.eq.0 ) then
      write(*,*) 'Choose Option (2 recommended)'
      write(*,*) '  Results unpredictable if you proceed'
      write(*,*) '  0= Data Files are SEG-2'
      write(*,*) '  1= Data Files are BISON'
      write(*,*) '  2= ABORT (Non-Standard File Names) '
      read(*,*) idtype
      if(idtype.gt.1 .or. idtype.lt.0) then
      stop
      endif
      endif 
c
      iskip=2
      if(sver(js).eq.i180) iskip=1
c
      ncount=ifcnt
      do 80 jz=1,nrec
      if(idtype.eq.1) then
      write(bname,'(a,i3.3)') prefix,ncount
      else
      write(bname,'(i4.4,''.dat'')') ncount
      endif
c
      ifchn=jcount
      ilchn=jcount+nchan-1
c
      if(idtype.eq.1) then
      write(io11,902) neznm,bname,lineid,sdepth,js,nchan,
     1ifchn,ilchn,ncount,sdepth,sazi(js),sver(js),razi,tazi
c
  902 format('topcon ',a,1x,a,1x,a,1x,f3.1,1x,i2,1x,i2,1x,
     1i4.4,1x,i4.4,1x,i4,1x,f3.0,1x,i3,1x,i3,1x,i3,1x,i3)
      else
      if(ichnlsw.eq.1) then
      write(io11,802) neznm,bname,lineid,sdepth,js,nchan,
     1ifchn,ilchn,ncount,sdepth,sazi(js),sver(js),razi,tazi
      else
      write(io11,802) neznm,bname,lineid,sdepth,js,nchan,
     1ilchn,ifchn,ncount,sdepth,sazi(js),sver(js),razi,tazi
      endif
c
  802 format('topcon2 ',a,1x,a,1x,a,1x,f3.1,1x,i2,1x,i2,1x,
     1i4.4,1x,i4.4,1x,i4,1x,f3.0,1x,i3,1x,i3,1x,i3,1x,i3)
      endif
c
      jcount=jcount+nchan
      ncount=ncount+iskip
c
   80 continue
   70 continue
c
      close(io11)
c
      if(idtype.eq.1) then
c
c...GEOM2 FILE
      open(io11,file='geom2',access='sequential')
      read(fname(1)(6:8),'(i3)') ifcnt
      read(lname(nshot)(6:8),'(i3)') ilcnt
      do 200 jc=ifcnt,ilcnt
      write(io11,903) jc
  903 format('go1 ',i3.3)
  200 continue
      close(io11)
c
c...GO1 FILE
      open(io11,file='go1',access='sequential')
      write(io11,904) prefix
  904 format(' bis2seg ',a,'$1')
      write(io11,905) prefix,prefix
  905 format(' bhed ',a,'$1.seg ',a,'$1.xyz 0')
      if(prefix(1:1).ne.'b' .and. prefix(1:1).ne.'B') then
      write(io11,906) prefix(1:4),prefix(1:1)
      else
      write(io11,908) prefix(1:4)
      endif
  906 format('mv bhed',a,'.seg ',a,'$1.seg')
  908 format('mv bhed',a,'.seg x$1.seg')
      write(io11,907) prefix
  907 format('rm  ',a,'$1.seg')
      endif
      stop
      end
