c |$Id: seg2txt.f,v 1.1.1.1 2017/09/16 19:19:46 pm Exp $       |
c --------------------------------------------------------------
c | Basic Seismic Utilities (BSU)                              |
c | LANGUAGE: Fortran77                                        |
c | AUTHOR: P. MICHAELS  http://cgiss.boisestate.edu/~pm       | 
c | PROGRAM NAME: seg2txt.f                                    |
c | DIRECTORY: bsu-2.0.2/src/Fort/bsegy                        |
c |------------------------------------------------------------|
c | Converts bsegy data to ascii text file                     |
c | columns of text as follows:                                |
c | sample_time data  data  data ...... data  data  char_strng |
c |                                                            |
c | Text files are big, so user limited to mdim traces per     |
c | run of this program (original:mdim=20, see parameter below)|
c | You may choose to increase or decrease this and recompile. |
c |------------------------------------------------------------|
c |  NOTICE:                                                   |
c |  Copyright (C) 2017 Paul Michaels                          |
c |  <pm@cgiss.boisestate.edu>                                 |
c |  This program is free software; you can                    |
c |  redistribute it and/or modify it under the terms          |
c |  of the GNU General Public License as published            |
c |  by the Free Software Foundation; either version           |
c |  3 of the License, or (at your option) any later           |
c |  version.  This program is distributed in the              |
c |  hope that it will be useful, but                          |
c |  WITHOUT ANY WARRANTY; without even the implied            |
c |  warranty of MERCHANTABILITY or FITNESS FOR A              |
c |  PARTICULAR PURPOSE.  See the GNU General Public           |
c |  License for more details.                                 |
c |  You should have received a copy of the GNU                |
c |  General Public License along with this program;           |
c |  if not, write to the Free Software Foundation,            |
c |  Inc., 675 Mass Ave, Cambridge, MA 02139, USA.             |
c |------------------------------------------------------------|
c |  PARAMETERS:                                               |
c |  ndim = maximum number of samples in a seismic trace       |
c |  mdim = maximum number of traces output in text file       |
c |------------------------------------------------------------|
c |  INCLUDE FILE REQUIRED:                                    |
c |  ../include/bsegy.inc    (modified segy header definitions)|
c |                (no reel header, floating point data)       |
c |------------------------------------------------------------|
c |  SUBROUTINES CALLED:                                       |
c | In-line===============                                     |
c |     GETPRM  get input parameters from command line or      |
c |             interactive query.  Should be inline with      |
c |             main program, specific to application.         |
c |     LSTPRM  echo check of input parameters from GETPRM.    |
c |             Values written to a history file (*.lst)       |
c | Located in bsu-2.0.2/src/sublibF4===========               |
c |     CHKTRC  get number of samples, sample interval, and    |
c |             number of traces in the input data file.       |
c |             Also returns values for progress bar, stdout   |
c |     BSEGIN  Reads a seismic trace.                         |
c |     BSEGOUT Write a seismic trace.                         |
c |     PLTBAR  Writes progress bar on stdout                  |
c |                                                            |
c --------------------------------------------------------------
c
      include '../include/bsegy.inc'
      parameter (ndim=32767,mdim=200,kdim=32767)
      real*4 s1(ndim),signal(kdim,mdim),time(kdim)
      integer    timelist
c
      character*40      infil , outfil, aplc, outlst
      character*4      chrstr /' '/
      common /header/ hd
c
c...assign I/O units, initialize....
      jrec=1
      iunit1=1
      iunit2=2
      icall1=0
      icall2=0
c
c...define application prefix
      write(aplc,'(''s2tx'')')
c
c...get parameters-------------------------------------------
      call GETPRM(nargsx,infil,s1,ndim,
     +ftmin,tmax,jrec1,jrec2,timelist,ntrace,fsamin,npts)  
c
c
c...count traces and check dimensions------------------------
      jtrace=jrec2-jrec1+1
      bar=float(jtrace)/50.
      ibar=int(bar+0.5)
      invbar=int(1./bar+0.5)

c
c...check parameter ranges
      itmin=int(ftmin/fsamin)+1
      if(itmin.lt.1) itmin=1
      itmax=int(tmax/fsamin)+1
      if(itmax.gt.npts) itmax=npts
      npts2=(itmax-itmin)+1
c
      if(jtrace.gt.mdim) then
      write(*,*) 'ABORT:  too many traces, increase mdim'
      write(*,*) 'and recompile, or select fewer traces '
      stop
      endif
      ntr2=jrec2-jrec1+1
c
c...name outputfile
      write(outfil,'(a,a,''.txt'')') aplc(1:4),infil(1:4)
c
c...output listing-------------------------------------------
      call LSTPRM(io11,ntrace,aplc,outlst,outfil,
     +infil,ftmin,tmax,jrec1,jrec2,timelist)  
c
c...clear output file...
      open(10,file=outfil,access='direct',recl=240)
      close(10,status='delete')
c
      icall1=0
      icall2=0
      ntr=0
c
c...open text file for output
      open(iunit2,file=outfil,access='sequential')
c
c...fill time variable
      jtmp=1
      do 70 jt=itmin,itmax
      time(jtmp)=float(jtmp-1+itmin-1)*fsamin
      jtmp=jtmp+1
  70  continue
c
c...data loop=======================================....
      jtr=0
      do 100 jrec=jrec1,jrec2
c
c...input a trace
c
      CALL BSEGIN(iunit1,ndim,
     +icall1,npts,s1,fsamin,infil,jrec,iexit)
c
      jtr = jtr + 1
c
c-----------------computation block----------------------
c
c...signal variable
      jtmp=1
      do 71 jt=itmin,itmax
      signal(jtmp,jtr)=s1(jt)
      jtmp=jtmp+1
  71  continue
      jtmax=jtmp-1
c
c...display progress....
      call pltbar(bar,invbar,ibar,jrec,ntrace)
c
100   continue
c
c...output trace
c $ supresses cr lf
      do 150 jt=1,jtmax
      if (timelist .eq.1 ) then
      write(iunit2,900) time(jt)
      endif
      do 145, kkk=1,ntr2
 145  write(iunit2,901) signal(jt,kkk)
      write(iunit2,902) chrstr
 900  format(f8.6,$)
 901  format(1x,E14.7,$)
 902  format(a)
150   continue
c
c
c...END LOOP========================================....
c
c...get time and date....
      write(*,'('' '')')
      write(6,240) jtr,outfil,outlst
240   format(' Traces Processed = ',i5/
     +5x,'====> output in ',a/'====> listing in ',a)
c
c
      close(io11)
      close(iunit1)
      close(iunit2)
      stop
      end
c
c --------------------------------------------------------------
c | Basic Seismic Utilities                                    |
c | AUTHOR: P. MICHAELS  http://cgiss.boisestate.edu/~pm       | 
c | PROGRAM NAME: getprm.f                                     |
c |------------------------------------------------------------|
c | get input parameters or help                               |
c --------------------------------------------------------------
c
      subroutine GETPRM(nargsx,infil,s1,ndim,
     +ftmin,tmax,jrec1,jrec2,timelist,ntrace,fsamin,npts)  
c
      character*40      infil
      character*40      arg1,arg2,arg3,arg4,arg5,arg6
      real*4            s1(*)
      integer           timelist
c
c
c...get input/output file names...
c   |--------------------------------------------|
c   | file naming convention                     |
c   | application prefix 4 characters "bxxx"     |
c   | data set suffix 4 characters "dddd"        |
c   | file name = bxxxdddd.seg                   |
c   |--------------------------------------------|
      write(*,*) '|---------------------------------|'
      write(*,*) '| Copyright (C) 2017  P. Michaels |'
      write(*,*) '|       All rights reserved       |'
      write(*,*) '|see GNU General Public License   |'
      write(*,*) '|---------------------------------|'
c
ccc---LINUX VERSION
      nargsx=iargc()
      karg=-1
c...test for help
      if(nargsx.ge.(2+karg)) then
      call getarg(1,arg1)
      if(arg1(1:2).eq.'-h') then
      write(*,*) ' '
      write(*,*) '|------------------------------------------|'
      write(*,*) '| Basic Seismic Utilities     FORTRAN      |'
      write(*,*) '| ONLINE HELP:                             |'
      write(*,*) '|------------------------------------------|'
      write(*,*) '| seg2text:  Converst bsegy data to ASCII  |'
      write(*,*) '| text.  Sample_Time and data only.        |'
      write(*,*) '|------------------------------------------|'
      write(*,*) ' '
      write(*,*)
      write(*,*)
     +'seg2txt infil tmin tmax fstrc lstrc timelist  '
      write(*,*) ' '
      write(*,*) 'infil =input file name '
      write(*,*) 'tmin  =minimum time '
      write(*,*) 'tmax  =maximum time '
      write(*,*) 'fstrc =first trace  '
      write(*,*) 'lstrc =last trace '
      write(*,*) 'timelist = 0 do NOT add column of sample times '
      write(*,*) '         = 1 Do add first column of sample times '
      write(*,*) ' '
      write(*,*) 'EXAMPLE: '
      write(*,*) ' seg2txt 1000.seg .05 .10 2  5 0   '
      write(*,*) 'would generate text file with 5-2+1=4 columns'
      write(*,*) 'No additional time column added '
      write(*,*) ' '
      stop
      endif
      endif
c
      if(nargsx.lt.(2+karg)) then
      write(*,*) '  enter: name of input file'
      read(*,'(a13)') infil
      else
      call getarg(1,infil)
      endif
      call namechk(12,infil)
c
c...<<<<<<<<<<<<<<<< select data to plot>>>>>>>>>>>>>>>>>>>>
c...check key parameters to aid in interactive input
      iunit1=1
      icall1=0
      call CHKTRC(iunit1,icall1,npts,s1,fsamin,infil,
     +bar,invbar,ibar,ntrace,ndim)
c
      tmaxx=float(npts-1)*fsamin
c
      if(nargsx.lt.(3+karg)) then
      write(*,'(''  Maximum trace time= '',f10.5)') tmaxx
      write(*,'(''  enter tmin'')')
      read(*,*) ftmin
      else
      call getarg(2,arg2)
      read(arg2,'(f10.4)') ftmin
      endif
c
      if(nargsx.lt.(4+karg)) then
      write(*,'(''  Maximum trace time= '',f10.5)') tmaxx
      write(*,'(''  enter tmax'')')
      read(*,*) tmax
      else
      call getarg(3,arg3)
      read(arg3,'(f10.4)') tmax
      endif
c
      if(nargsx.lt.(5+karg)) then
      write(*,'(''  Total Number of Traces= '',i5)') ntrace
      write(*,'(''  enter first trace number'')')
      read(*,*) jrec1
      else
      call getarg(4,arg4)
      read(arg4,'(i2)') jrec1
      endif
c
      if(nargsx.lt.(6+karg)) then
      write(*,'(''  Total Number of Traces= '',i5)') ntrace
      write(*,'(''  enter last trace number'')')
      read(*,*) jrec2
      else
      call getarg(5,arg5)
      read(arg5,'(i2)') jrec2
      endif
c
      if(nargsx.lt.(7+karg)) then
      write(*,'('' enter timelist '')')
      write(*,'(''    0= No sample time column'')')
      write(*,'(''    1= Add sample time column'')')
      read(*,*) timelist
      else
      call getarg(6,arg6)
      read(arg6,'(i2)') timelist
      endif
c
      return
      end
c --------------------------------------------------------------
c | Basic Seismic Utilities                                    |
c | AUTHOR: P. MICHAELS  http://cgiss.boisestate.edu/~pm       | 
c | PROGRAM NAME: lstprm.f                                     |
c |------------------------------------------------------------|
c | echo check of input parameters, writes to a history file   |
c --------------------------------------------------------------
c
      subroutine LSTPRM(io11,ntrace,aplc,outlst,outfil,
     +infil,ftmin,tmax,jrec1,jrec2,timelist)
      integer           timelist  
      character*4       aplc
      character*40      outlst,outfil,infil
c
c...open output listing file...
      io11=11
      write(outlst,'(a,a,''.lst'')') aplc,infil(1:4)
      open(io11,file=outlst,access='sequential')
      write(io11,*) '|---------------------------------|'
      write(io11,*) '| Copyright (C) 2017  P. Michaels |'
      write(io11,*) '|       All rights reserved       |'
      write(io11,*) '|see GNU General Public License   |'
      write(io11,*) '|---------------------------------|'
c
      write(io11,'('' Program seg2txt'')')
      write(io11,'('' Input File: '',a)') infil
      write(io11,'('' Output File: '',a)') outfil
c
      write(io11,'(''  Number of traces='',i5)') ntrace
      write(io11,'(''  Parameters:      '')')
      write(io11,'(''     Tmin=  '',f10.4)') ftmin
      write(io11,'(''     Tmax=  '',f10.4)') tmax
      write(io11,'(''     First Trace=  '',i5)') jrec1
      write(io11,'(''     Last Trace=  '',i5)')  jrec2
      if (timelist .eq. 1) then
      write(io11,'('' Column 1 added, sample times '')')  
      else
      write(io11,'('' No sample time column added '')')  
      endif
c
      write(io11,'(50(1h-))')
      write(io11,'(''    '')')
c
      return
      end

