c --------------------------------------------------------------
c | author: p. michaels                      date: June 2000   |
c |                                                            |
c | gendis: generates a disper namelist                        | 
c |         disper.f computes rayleigh wave dispersion         |
c |  Modified for input file with Vs and Depth in columns      |
c |                                                            |
c |------------------------------------------------------------|
c |  NOTICE:                                                   |
c |  Copyright (C) 2017 Paul Michaels                          |
c |  <pm@cgiss.boisestate.edu>                                 |
c |  This program is free software; you can                    |
c |  redistribute it and/or modify it under the terms          |
c |  of the GNU General Public License as published            |
c |  by the Free Software Foundation; either version           |
c |  3 of the License, or (at your option) any later           |
c |  version.  This program is distributed in the              |
c |  hope that it will be useful, but                          |
c |  WITHOUT ANY WARRANTY; without even the implied            |
c |  warranty of MERCHANTABILITY or FITNESS FOR A              |
c |  PARTICULAR PURPOSE.  See the GNU General Public           |
c |  License for more details.                                 |
c |  You should have received a copy of the GNU                |
c |  General Public License along with this program;           |
c |  if not, write to the Free Software Foundation,            |
c |  Inc., 675 Mass Ave, Cambridge, MA 02139, USA.             |
c |------------------------------------------------------------|
c
      implicit real*4 (a-h,o-z)
      parameter (ndim=1000)
      real*4 mu(ndim),lame(ndim),rho(ndim),h(ndim)
      character*40 ofile,infile
c
c...read general parameters
c
      write(*,*)'  enter: name of output file ( < 40 char)'
      read(*,'(a)') ofile
c
      write(*,*)'  enter: sample rate'
      read(*,*) fsamin 
c 
      write(*,*)'  enter: tmax for trace' 
      read(*,*) tmax 
c 
      write(*,*)'  enter: minimum frequency' 
      read(*,*) fmin
c 
      write(*,*)'  enter: maximum frequency' 
      read(*,*) fmax 
c  
      write(*,*)'  enter: maximum mode #'
      read(*,*) modemx
c 
c     write(*,*)'  enter: deltz step size' 
c     read(*,*) deltz 
c
      write(*,*)' enter density Kg/m3'
      read(*,*) rho1
c
      write(*,*)' enter p-wave velocity m/s'
      read(*,*) alpha
c
      write(*,*)'===Enter name of input file with Vs, Z pairs'
      read(*,'(a)') infile
c
c...open file for input
      open(11,file=infile,access='sequential')
c 
c...read model 
c     write(*,*)'  enter: number of control' 
c     read(*,*) nlayer
c
      nlayer=0
      do 100 j=1,1000
c      read(*,*) beta,alpha,rho1,depth
      read(11,*,end=101) beta,depth
      nlayer=nlayer+1
      if(beta.eq.0.) then
      write(*,*)'ABORT:: Fix first value of beta=0'
      stop
      endif 
      rho(j)=rho1
      h(j)=depth
      mu(j)=fmu(beta,rho1)
      lame(j)=flame(alpha,beta,rho1)
  100 continue
  101 continue
c
c...calculate dz (assume dz constant)
      deltz=abs(h(2)-h(1))
      call namel3(mu,lame,rho,nlayer,deltz,
     +h,fmin,fmax,fsamin,tmax,modemx,ofile)
c
      write(*,'(''  output====>'',a)') ofile
      stop
      end
c#
c --------------------------------------------------------------
c | author: p. michaels                      date: 24 sep 92   |
c |                                                            |
c | genlist function block                                     | 
c |------------------------------------------------------------|
c
c ====================== shear modulus ========================
      real*4 function fmu(beta,rho)
      fmu=beta*beta*rho
      end
c ====================== lames modulus ========================
      real*4 function flame(alpha,beta,rho)
      flame=(alpha*alpha*rho)-2.00*fmu(beta,rho)
      end
c ===================== maximum layer thickness ===============
      real*4 function hmax(beta,fmax)
      hmax=4.00*(beta/fmax)
      end
c#
c --------------------------------------------------------------
c | author: p. michaels                      date: 24 sep 92   |
c |                                                            |
c | namel: generates a disper namelist                         | 
c |                                                            |
c |------------------------------------------------------------|
c
      subroutine namel3(mu,lame,rho,nlayer,deltz,
     +h,fmin,fmax,fsamin,tmax,modemx,ofile)
      implicit real*4 (a-h,o-z)
      real*4 mu(*),lame(*),rho(*),h(*)
      character*40 ofile
c
c
c...open file
      open(12,file=ofile,access='sequential')
c
c...calculate parameters
      zero=0.0
c
      npts=tmax/fsamin
      npts=2*npts
      npr2=2
      do 5 j=1,50
      if(npr2.gt.npts) go to 10
      npr2=npr2*2
    5 continue
   10 continue
      tmax=npr2*fsamin
      write(*,'('' twice npts='',i5)') npr2
      write(*,'('' twice tmax='',f10.4)') tmax
      delf=1.00/tmax
      flo=fmin
c
      nfreq=(fmax-fmin)/delf
c
c...write namelist
      write(12,*)' &disper'
      write(12,'(''  nlay='',i5,'','')') nlayer
      write(12,'("rho=",$)')
      write(12,100) (rho(kk),kk=1,nlayer)
  100 format(5(1x,E10.4,','))
      write(12,'("mu=",$)')
      write(12,110) (mu(kk),kk=1,nlayer)
  110 format(5(1x,E10.4,','))
      write(12,'("lame=",$)')
      write(12,110) (lame(kk),kk=1,nlayer)
      write(12,'("zi=",$)')
      write(12,120) (h(kk),kk=1,nlayer)
  120 format(5(1x,E10.4,','))
      write(12,*)'  iskip=1,'
      write(12,130) deltz
  130 format('  deltz=',f10.4,',')
      write(12,140) modemx
  140 format('  modemx=',i1,',')
      write(12,150) nfreq,flo,delf
  150 format('  nfreq=',i3,', flo=',e15.7,', delf=',e15.8,',')
      write(12,*)'  jsmax=300, ksw=0,'
      write(12,*)'  pvlcty=0.0, pfreq=0.0, zend=100.0,'
      write(12,160)
  160 format('  ofile=''disper.tmp'',')
      write(12,170)
  170 format(' '/
     +'  octav1=''phase.m'', octav2=''mat2.m'',')
      write(12,180)
  180 format('  curve=''earth.crv'', /')
      close(12)
      close(11)
      return
      end
