C  |------------------------------------------------------------|
C  | Paul Michaels <pm@cgiss.boisestate.edu>                    |
C  | $Id: halfsp.f,v 1.1.1.1 2017/09/16 19:19:46 pm Exp $       |
C  | TEST PROGRAM FOR HALFSP.F                                  |
C  |------------------------------------------------------------|
C  |  NOTICE:                                                   |
C  |  Copyright (C) 2017 Paul Michaels                          |
C  |  <pm@Cgiss.boisestate.edu>                                 |
C  |  This program is free software; you can                    |
C  |  redistribute it and/or modify it under the terms          |
C  |  of the GNU General Public License as published            |
C  |  by the Free Software Foundation; either version           |
C  |  3 of the License, or (at your option) any later           |
C  |  version.  This program is distributed in the              |
C  |  hope that it will be useful, but                          |
C  |  WITHOUT ANY WARRANTY; without even the implied            |
C  |  warranty of MERCHANTABILITY or FITNESS FOR A              |
C  |  PARTICULAR PURPOSE.  See the GNU General Public           |
C  |  License for more details.                                 |
C  |  You should have received a copy of the GNU                |
C  |  General Public License along with this program;           |
C  |  if not, write to the Free Software Foundation,            |
C  |  Inc., 675 Mass Ave, Cambridge, MA 02139, USA.             |
C  |------------------------------------------------------------|
C
      WRITE(*,'(''  ENTER RHO,VP,VS'')')
      READ(*,*) RHO,VP,VS
      FMU=VS*VS*RHO
      FLAME=VP*VP*RHO-2.0*FMU
      WRITE(*,'('' ENTER FREQ,NZ,Z0,ZEND'')')
      READ(*,*) FREQ,NZ,Z0,ZEND
      PI=3.14159265
      WFREQ=2.0E+00*FREQ*PI
      DZ=(ZEND-Z0)/FLOAT(NZ)
      C=0.0E+00
      Z=0.0E+00
      OPEN(UNIT=10,FILE='halfsp.tmp',ACCESS='SEQUENTIAL')
C
      CALL HALFSP(RHO,FMU,FLAME,WFREQ,Z,C,R1,R2,R3,R4)
      WRITE(10,'(''  HALFSP.F OUTPUT:'')')
      WRITE(*,'(''  PHASE VEL='',F10.4)') C
      WRITE(10,'(''  RHO='',E10.4)') RHO
      WRITE(10,'(''  MU='',E10.4)') FMU
      WRITE(10,'(''  LAME='',E10.4)') FLAME
      WRITE(10,'(''  FREQ='',E10.4)') FREQ
      B=SQRT(FMU/RHO)
      A=SQRT((FLAME+2.0E+00*FMU)/RHO)
      WRITE(10,'(''  P-WAVE VELOCITY='',E10.4)') A
      WRITE(10,'(''  S-WAVE VELOCITY='',E10.4)') B
      WRITE(10,'(''  RAYLEIGH WAVE PHASE VEL='',F10.4)') C
      WRITE(10,'('' R1=Horiz. Displacement R2=Vertical Displacement'')')
      WRITE(10,'('' R3=Horiz. Stress       R4=Vertical Stress'')')
      WRITE(10,901)
  901 FORMAT(//5X,'DEPTH',6X,'R1',9X,'R2',9X,'R3',9X,'R4'/)
C
      NZP1=NZ+1
      Z=Z0
      DO 10 J=1,NZP1
      CALL HALFSP(RHO,FMU,FLAME,WFREQ,Z,C,R1,R2,R3,R4)
      WRITE(10,900) Z,R1,R2,R3,R4
   10 Z=Z+DZ
  900 FORMAT(2X,F8.1,2X,4(E14.7,2X))
      STOP
      END
C# |------------------------------------------------------------|
C  | HALFSP.F                     ORIGINAL  DATE: 27 SEP  91    |
C  | $Id: halfsp.f,v 1.1.1.1 2017/09/16 19:19:46 pm Exp $           |
C  | AUTHOR: P. MICHAELS                                        |
C  |------------------------------------------------------------|
C  | HALFSP: CALCULATES THE THEORETICAL PHASE VELOCITY AND      |
C  |         DISPLACEMENT,STRESS EIGENFUNCTIONS AT DEPTH, Z.    |
C  |                                                            |
C  | R1=HORIZONTAL DISPLACEMENT      (AKI & RICHARDS SIGN       |
C  | R2=VERTICAL DISPLACEMENT         CONVENTION)               |
C  | R3=HORIZONTAL STRESS                                       |
C  | R4=VERTICAL STRESS                                         |
C  | C=PHASE VELOCITY FOR RAYLEIGH WAVES                        |
C  |                                                            |
C  | RHO=HALF SPACE DENSITY                                     |
C  | FMU=HALF SPACE SHEAR MODULUS                               |
C  | FLAME= HALF SPACE LAME'S CONSTANT                          |
C  |------------------------------------------------------------|
C
      SUBROUTINE HALFSP(RHO,FMU,FLAME,WFREQ,Z,C,R1,R2,R3,R4)
      IMPLICIT REAL*4 (A-H,O-Z)
C
C...COMPUTE SHEAR VELOCITY
      B=SQRT(FMU/RHO)
      D=FLAME/FMU
      IF(C.EQ.0.0E+00) THEN
C
C   COMPUTE PHASE VELOCITY
C...SET UP COEF. FOR CUBIC EQUATION
      A1=-8.0E+00
      A2=((8.0E+00)*(D*3.0E+00+4.0E+00))/(D+2.0E+00)
      A3=((-16.0E+00)*(D+1.0E+00))/(D+2.0E+00)
C
C...SOLVE CUBIC EQUATION
      PI=3.14159265
      Q=(A1*A1-3*A2)/9.0E+00
      R=(2.0E+00*A1*A1*A1-9.0E+00*A1*A2+27.0E+00*A3)/54.0E+00
      TEST=Q*Q*Q - R*R
      IF(TEST.GE.0.0E+00) THEN
      T1=ACOS(R/SQRT(Q*Q*Q))
      T2=-2.0E+00*SQRT(Q)
      T3=3.0E+00
      X1=T2*COS(T1/T3)-A1/T3
      X2=T2*COS((T1+2.0E+00*PI)/T3)-A1/T3
      X3=T2*COS((T1+4.0E+00*PI)/T3)-A1/T3
      ELSE
      T1=(SQRT(R*R-Q*Q*Q)+ABS(R))**.333333333
      T3=3.0E+00
      X1=-SIGN(1.0,R)*(T1+Q/T1)-A1/T3
      ENDIF
      C=SQRT(B*B*X1)
      ENDIF
C
C...COMPUTE EIGENFUNCTIONS
      FK=WFREQ/C
      T1=(C*C)/(B*B)
      G1=SQRT(1.0-T1)*FK
      G2=SQRT(1.0-T1/(D+2.0E+00))*FK
      T2=T1-2.0E+00
      E1=EXP(-G1*Z)
      E2=EXP(-G2*Z)
C
      R1=FK*E2 + FK*(T2/2.0E+00)*E1
C
      R2=G2*E2 + ((FK*FK)/(2.0E+00*G1))*T2*E1
C
      R3=FMU*FK*(-2.0E+00*G2*E2 + ((T2*T2*FK*FK)/(G1*2.0E+00))*E1)
C
      R4=(FK*FK*FMU*T2)*(E2-E1)
C
      RETURN
      END
