//  $Id: timing.c,v 1.1.1.1 2017/09/16 19:19:46 pm Exp $

/* Suitable for Fortran77 and Fortran90 */

#include <stdio.h>
#include <sys/times.h> /* for times system call */
#include <sys/time.h>  /* for gettimeofday system call */
#include <unistd.h>
#include <time.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>


/*
---------------------------------------------------------------------------
clock_t times(struct tms *buffer);

times() fills the structure pointed to by buffer with
time-accounting information.  The structure defined in 
<sys/times.h> is as follows:

struct tms {
    clock_t tms_utime;       user time 
    clock_t tms_stime;       system time 
    clock_t tms_cutime;      user time, children 
    clock_t tms_cstime;      system time, children 

The time is given in units of 1/CLK_TCK seconds where the
value of CLK_TCK can be determined using the sysconf() function
with the agrgument _SC_CLK_TCK.
---------------------------------------------------------------------------
*/
								  

float cputime();
float systime();
float gettime();
void timeofday(char *str);

float cputime_() { return cputime();}
float systime_() { return systime();}
float gettime_() { return gettime();}
void  timeofday_(char *tod1) { timeofday(tod1) ;}
// void  strlen_(const char *str, int *len1 )  { *len1=strlen(str); }

float cputime()
{ 
	struct tms buffer;
	float cputime;

	times(&buffer);
	cputime = (buffer.tms_utime)/ (float) sysconf(_SC_CLK_TCK);
	return (cputime);
}


float systime()
{ 
	struct tms buffer;
	float systime;

	times(&buffer);
	systime = (buffer.tms_stime)/ (float) sysconf(_SC_CLK_TCK);
	return (systime);
}


#define SECONDS_IN_A_YEAR 365*24*3600

float gettime()
{
    struct timeval now;
    double offset; 
    long years;
    double valu;
    float value;


    gettimeofday(&now, (struct timezone *)0);
    years = (long)now.tv_sec/(SECONDS_IN_A_YEAR);
    /*offset to get seconds into the year*/
    offset = years*SECONDS_IN_A_YEAR - 1;
    valu = (double) (((double) now.tv_sec +
            (double)now.tv_usec/1000000.0) - offset);
     /*offset to avoid underflow when converting to float*/
    offset = (double) floor((valu/10000.0))*10000.0;
    value  = (float) (valu-offset);
    return value;
}

void timeofday(char *tod1)
{
	time_t now;
	struct tm *ptr;
        static char *tt;

//  get current time
//convert to string
	time(&now);
	ptr=localtime(&now);
	tt=asctime(ptr);
        sprintf(tod1,"%s",tt);
}

