// |$Id: c_bsegout.c,v 1.1.1.1 2017/09/16 19:19:46 pm Exp $     |
/* |------------------------------------------------------------|
   | Basic Seismic Utilities (BSU)                              |
   | LANGUAGE: C                                                |
   | AUTHOR: P. MICHAELS    http://cgiss.boisestate.edu/~pm     | 
   | FUNCTION NAME: c_bsegout.c                                 |
   | DIRECTORY: bsu-2.0.2/src/sublibC4                          |
   |------------------------------------------------------------|
   |  NOTICE:                                                   |
   |  Copyright (C) 2017 Paul Michaels                          |
   |  <pm@cgiss.boisestate.edu>                                 |
   |  This program is free software; you can                    |
   |  redistribute it and/or modify it under the terms          |
   |  of the GNU General Public License as published            |
   |  by the Free Software Foundation; either version           |
   |  3 of the License, or (at your option) any later           |
   |  version.  This program is distributed in the              |
   |  hope that it will be useful, but                          |
   |  WITHOUT ANY WARRANTY; without even the implied            |
   |  warranty of MERCHANTABILITY or FITNESS FOR A              |
   |  PARTICULAR PURPOSE.  See the GNU General Public           |
   |  License for more details.                                 |
   |  You should have received a copy of the GNU                |
   |  General Public License along with this program;           |
   |  if not, write to the Free Software Foundation,            |
   |  Inc., 675 Mass Ave, Cambridge, MA 02139, USA.             |
   |------------------------------------------------------------|
   | purpose: output the jrec-th trace from output              |
   |          data set in bsegy format.                         |
   |                                                            |
   | input:  h2      =pointer to output file                    |
   |         jrec    =trace number to be output                 |
   |         npts    =number of points in a trace               |
   |         hd      =pointer to header structure               |
   |         s2      =data trace array pointer                  |
   |                                                            |
   | output: writes 240 byte header and data trace              |
   |         to file h2.                                        |
   |                                                            |
   |------------------------------------------------------------|
*/
      #include <stdio.h>
      #include <stdlib.h>
      #include "../C/include/c_bsegy.h"

      int c_bsegout(int jrec, float s2[], int npts,
        struct header *hd, FILE *h2)
     {
      long         offset;


      offset=(240+(long) npts*sizeof(float))*(jrec);
        if(fseek(h2,offset,SEEK_SET) !=0)
       {
        puts("ABORT--BSEGOUT: can't position indicator");
        return(1);
       }

//...write header
        if(fwrite(hd,sizeof(char),240,h2)!=240)
            {
             puts("ABORT--BSEGOUT: Error writing header ");
             return(1);
            }

//...write data
        if(fwrite(s2,sizeof(float),npts,h2)!=npts)
            {
             puts("ABORT--BSEGOUT: Error writing data ");
             return(1);
            }
      return(0);
     }  //  end bsegout
