// |$Id: findxyz.c,v 1.1.1.1 2017/09/16 19:19:46 pm Exp $       |
/* |------------------------------------------------------------|
   | Basic Seismic Utilities (BSU)                              |
   | LANGUAGE: C                                                |
   | AUTHOR: P. MICHAELS    http://cgiss.boisestate.edu/~pm     | 
   | FUNCTION NAME: findxyz                                     |
   | DIRECTORY: bsu-2.0.2/src/sublibC4                          |
   |------------------------------------------------------------|
   |  NOTICE:                                                   |
   |  Copyright (C) 2017 Paul Michaels                          |
   |  <pm@cgiss.boisestate.edu>                                 |
   |  This program is free software; you can                    |
   |  redistribute it and/or modify it under the terms          |
   |  of the GNU General Public License as published            |
   |  by the Free Software Foundation; either version           |
   |  3 of the License, or (at your option) any later           |
   |  version.  This program is distributed in the              |
   |  hope that it will be useful, but                          |
   |  WITHOUT ANY WARRANTY; without even the implied            |
   |  warranty of MERCHANTABILITY or FITNESS FOR A              |
   |  PARTICULAR PURPOSE.  See the GNU General Public           |
   |  License for more details.                                 |
   |  You should have received a copy of the GNU                |
   |  General Public License along with this program;           |
   |  if not, write to the Free Software Foundation,            |
   |  Inc., 675 Mass Ave, Cambridge, MA 02139, USA.             |
   |------------------------------------------------------------|
   |                                                            |
   | purpose: decode x,y,z coordinates from segy                |
   |          header values                                     |
   |                                                            |
   |------------------------------------------------------------|
*/
      #include <stdio.h>
      #include <stdlib.h>
      #include <math.h>
      #include "../C/include/c_bsegy.h"

      void findxyz(float *sx, float *sy, float *sz, float *sd,
         float *rx, float *ry, float *rz, float *range,
         struct header hd)
     {

//...source coordinates
       if(hd.smul2<0)
      {
        *sx=(float)hd.ishlo / (float)fabs(hd.smul2);
        *sy=(float)hd.ishla / (float)fabs(hd.smul2);
      }
       else
      {
        *sx=(float)hd.ishlo * (float)fabs(hd.smul2);
        *sy=(float)hd.ishla * (float)fabs(hd.smul2);
      }  //endif

       if(hd.smul1<0)
      {
        *sz=(float)hd.ishe / (float)fabs(hd.smul1);
        *sd=(float)hd.ishd / (float)fabs(hd.smul1);
      }
       else
      {
        *sz=(float)hd.ishe * (float)fabs(hd.smul1);
        *sd=(float)hd.ishd * (float)fabs(hd.smul1);
      }  //endif

//...reciever coordinates
       if(hd.smul2<0)
      {
        *rx=(float)hd.irlo / (float)fabs(hd.smul2);
        *ry=(float)hd.irla / (float)fabs(hd.smul2);
      }
       else
      {
        *rx=(float)hd.irlo * (float)fabs(hd.smul2);
        *ry=(float)hd.irla * (float)fabs(hd.smul2);
      }  //endif

       if(hd.smul1<0)
      {
        *rz=(float)hd.irel / (float)fabs(hd.smul1);
      }
       else
      {
        *rz=(float)hd.irel * (float)fabs(hd.smul1);
      }  //endif

//...compute range between source and receiver
      *range=(*rx-*sx)*(*rx-*sx);
      *range=*range+(*ry-*sy)*(*ry-*sy);
      *range=*range+(*rz-*sz+*sd)*(*rz-*sz+*sd);
      if(*range!=0.) *range=sqrt(*range);

     }  // end findxyz
