// |$Id: fmax_min.c,v 1.1.1.1 2017/09/16 19:19:46 pm Exp $      |
/* |------------------------------------------------------------|
   | Basic Seismic Utilities (BSU)                              |
   | LANGUAGE: C                                                |
   | AUTHOR: P. MICHAELS    http://cgiss.boisestate.edu/~pm     | 
   | FUNCTION NAME: fmax_min.c                                  |
   | DIRECTORY: bsu-2.0.2/src/sublibC4                          |
   |------------------------------------------------------------|
   |  NOTICE:                                                   |
   |  Copyright (C) 2017 Paul Michaels                          |
   |  <pm@cgiss.boisestate.edu>                                 |
   |  This program is free software; you can                    |
   |  redistribute it and/or modify it under the terms          |
   |  of the GNU General Public License as published            |
   |  by the Free Software Foundation; either version           |
   |  3 of the License, or (at your option) any later           |
   |  version.  This program is distributed in the              |
   |  hope that it will be useful, but                          |
   |  WITHOUT ANY WARRANTY; without even the implied            |
   |  warranty of MERCHANTABILITY or FITNESS FOR A              |
   |  PARTICULAR PURPOSE.  See the GNU General Public           |
   |  License for more details.                                 |
   |  You should have received a copy of the GNU                |
   |  General Public License along with this program;           |
   |  if not, write to the Free Software Foundation,            |
   |  Inc., 675 Mass Ave, Cambridge, MA 02139, USA.             |
   |------------------------------------------------------------|
   |                                                            |
   | purpose: find maximum and minimum values in a              |
   |          trace                                             |
   |                                                            |
   |INPUTS:                                                     |
   |  s1=input trace   npts=number samples isw1=switch          |
   |         isw1=0  max and min of s1, signed data             |
   |         isw1=1  max and min of fabs(s1) absolute           |
   |                                         value              |
   |                                                            |
   |OUTPUTS: pointers                                           |
   | *fmax=max value         *fmin=min value                    |
   | *kmx=index max value    *kmn=index min value               |
   |------------------------------------------------------------|
*/
      #include <stdio.h>
      #include <stdlib.h>
      #include <math.h>

      void fmax_min(int npts, float s1[],float *fmax,
         float *fmin, int isw1, int *kmx,  int *kmn )
     {

//...declare variables
      int               j;

//...initialize
      *fmax=-9.e+30;
      *fmin=+9.e+30;
      *kmx=0;
      *kmn=0;

//...find max and min
      for (j=0;j<npts;j++)
     {
       if(isw1==0)
      {
      if(s1[j]>*fmax) { *fmax=s1[j]; *kmx=j; }
      if(s1[j]<*fmin) { *fmin=s1[j]; *kmn=j; }
      }
       else
      {
      if(fabs(s1[j])>*fmax) { *fmax=fabs(s1[j]); *kmx=j; }
      if(fabs(s1[j])<*fmin) { *fmin=fabs(s1[j]); *kmn=j; }
      }  //endif
     }  //endfor

     } //  end fmax_min
