// |$Id: in_chk.c,v 1.1.1.1 2017/09/16 19:19:46 pm Exp $        |
/* |------------------------------------------------------------|
   | Basic Seismic Utilities (BSU)                              |
   | LANGUAGE: C                                                |
   | AUTHOR: P. MICHAELS    http://cgiss.boisestate.edu/~pm     | 
   | FUNCTION NAME: in_chk.c                                    |
   | DIRECTORY: bsu-2.0.2/src/sublibC4                          |
   |------------------------------------------------------------|
   |  NOTICE:                                                   |
   |  Copyright (C) 2017 Paul Michaels                          |
   |  <pm@cgiss.boisestate.edu>                                 |
   |  This program is free software; you can                    |
   |  redistribute it and/or modify it under the terms          |
   |  of the GNU General Public License as published            |
   |  by the Free Software Foundation; either version           |
   |  3 of the License, or (at your option) any later           |
   |  version.  This program is distributed in the              |
   |  hope that it will be useful, but                          |
   |  WITHOUT ANY WARRANTY; without even the implied            |
   |  warranty of MERCHANTABILITY or FITNESS FOR A              |
   |  PARTICULAR PURPOSE.  See the GNU General Public           |
   |  License for more details.                                 |
   |  You should have received a copy of the GNU                |
   |  General Public License along with this program;           |
   |  if not, write to the Free Software Foundation,            |
   |  Inc., 675 Mass Ave, Cambridge, MA 02139, USA.             |
   |------------------------------------------------------------|
   |                                                            |
   | purpose: gets fundamental parameters from header           |
   |          of newly opened bsegy data set.                   |
   |                                                            |
   | input: h1 (pointer to input file)                          |
   |                                                            |
   | output: npts    =number of samples per trace               |
   |         fsamin  =sample interval in seconds                |
   |         ntraces =number of traces in file                  |
   |                                                            |
   |------------------------------------------------------------|
*/
      #include <stdio.h>
      #include <stdlib.h>
      #include "../C/include/c_bsegy.h"

      int in_chk(int *ntraces, int *npts,
        float *fsamin,  struct header hd,  FILE *h1)
     {
        long            fsize;

//...read header
        if(fread(&hd,sizeof(char),240,h1)!=240)
            {
             puts("ABORT--Error reading input file ");
             return(1);
            }

      *npts=(int) hd.length;
      *fsamin=((float)hd.isi)/1.0E+6;

        if(fseek(h1,0,SEEK_END) !=0)
       {
        puts("ABORT--in_chk can't position at end of file");
        return(1);
       }

      fsize=ftell(h1);
      *ntraces=(int) fsize/(240 +(long) (*npts)*sizeof(float));

        if(fseek(h1,0,SEEK_SET) !=0)
       {
        puts("ABORT--in_chk can't position at start of file");
        return(1);
       }

        if (*npts<1 || *ntraces<1)
       {
        puts("ABORT--not enough data");
	fprintf(stdout,"npts=%d ntraces=%d \n",*npts,*ntraces);
        return(1);
       }

      return(0);
     }  //  end in_chk
