c |$Id: boxit.f,v 1.1.1.1 2017/09/16 19:19:46 pm Exp $          |
c --------------------------------------------------------------
c | Basic Seismic Utilities                                    |
c | AUTHOR: P. MICHAELS  http://cgiss.boisestate.edu/~pm       | 
c | PROGRAM NAME: boxit.f                                      |
c | DIRECTORY: bsu-2.0.2/src/sublibF4                          |
c |------------------------------------------------------------|
c |  NOTICE:                                                   |
c |  Copyright (C) 2017 Paul Michaels                          |
c |  <pm@cgiss.boisestate.edu>                                 |
c |  This program is free software; you can                    |
c |  redistribute it and/or modify it under the terms          |
c |  of the GNU General Public License as published            |
c |  by the Free Software Foundation; either version           |
c |  3 of the License, or (at your option) any later           |
c |  version.  This program is distributed in the              |
c |  hope that it will be useful, but                          |
c |  WITHOUT ANY WARRANTY; without even the implied            |
c |  warranty of MERCHANTABILITY or FITNESS FOR A              |
c |  PARTICULAR PURPOSE.  See the GNU General Public           |
c |  License for more details.                                 |
c |  You should have received a copy of the GNU                |
c |  General Public License along with this program;           |
c |  if not, write to the Free Software Foundation,            |
c |  Inc., 675 Mass Ave, Cambridge, MA 02139, USA.             |
c |------------------------------------------------------------|
c | boxit: Apply box car filter to a trace using add/subtract  |
c |        trick.                                              |
c |  npts=number of values in x,y, and z                       |
c |     x= array to be filtered (input)                        |
c |     y= filtered output                                     |
c |     z= scratch array                                       |
c |                                                            |
c |     isign  -1=low pass smoother (box car)                  |
c |             0=high pass (1-sinc**ntimes)                   |
c |            +1=high pass ((1-sinc)**ntimes)                 |
c |                                                            |
c |     iwide= width of box car in samples                     |
c |     ntimes=number of times to apply box car                |
c |                                                            |
c |------------------------------------------------------------|
c
      subroutine boxit(npts,x,y,z,iwide,isign,ntimes)
      real*4            x(*),y(*),z(*)
      if(npts.le.0) return
c
c...make iwide odd if it is even
      iwide=(iwide/2)*2+1
      ihw=(iwide+1)/2
      ihwm1=ihw-1
c
c...normalize for gain=1
      sfact=1.0E+00/float(iwide)
      srfact=float(iwide)
      fact=1.0E+00
      do 1 k=1,ntimes
    1 fact=fact*sfact
c
c...load scratch buffer
      do 5 j=1,npts
    5 z(j)=x(j)
c
c...start number of applications loop/////////////
      do 80 jtime=1,ntimes
c
c...small iwide
      if(iwide.gt.1) go to 2
      do 3 j=1,npts
    3 y(j)=z(j)
      go to 500
    2 continue
c
c...small data set
      if(npts.gt.iwide) go to 4
      sum=0.000
      do 7 j=1,npts
    7 sum=sum+z(j)
      do 8 j=1,npts
    8 y(j)=sum
      go to 500
    4 continue
c
c...taper in (set all values = sum for 1/2 operator)
      sum=0.000
      do 10 k=1,iwide
   10 sum=sum+z(k)
      do 20 k=1,ihw
   20 y(k)=sum
c
c...main application: add next, subtract first
      jend=npts-ihw+1
      jstart=ihw+1
c
      do 30 j=jstart,jend
      sum=sum-z(j-ihw)
      sum=sum+z(j+ihwm1)
   30 y(j)=sum
c
c...taper off (set all values = sum for 1/2 operator)
      jstart=jend+1
      do 40 j=jstart,npts
   40 y(j)=sum
c
c
  500 continue
c
c
c...high(1) or low pass?
      if(isign.lt.1) go to 60
      do 50 j=1,npts
   50 y(j)=z(j)*srfact-y(j)
c
c...prepare for next iteration
   60 continue
      do 70 j=1,npts
   70 z(j)=y(j)
c
   80 continue
c...end ntimes loop//////////////
c
c...rescale to gain = 1
      do 90 j=1,npts
   90 y(j)=y(j)*fact
c
c...high pass(0) or low pass ?
      if(isign.ne.0) return
      do 100 j=1,npts
  100 y(j)=x(j)-y(j)
c
      return
      end
