c |$Id: eureka.f,v 1.1.1.1 2017/09/16 19:19:46 pm Exp $          |
c --------------------------------------------------------------
c | Basic Seismic Utilities                                    |
c | AUTHOR: E. A. Robinson (1967, p.44)                        |
c | PROGRAM NAME: eureka.f                                     |
c | DIRECTORY: bsu-2.0.2/src/sublibF4                          |
c |------------------------------------------------------------|
c | Copyright (c) 1967 by Holden-Day Inc. 500 Sansome St.,     |
c | San Francisco, all rights reserved                         |
c |                                                            |
c | Holden-Day Inc., Liquidation Trust, 1259 S.W. 14th Street  |
c | Boca Raton, FL 33486 Phone: (561)750-9229 Fax: 561.394.6809|
c |                                                            |
c |    This subroutine is included for distribution with Basic |
c | Seismic Utilities (BSU) by joint permission of the author, |
c | Enders Robinson, and Holden-Day Inc. Liquidation Trust,    |
c | Frederick H. Murphy, Trustee.  Being part of the BSU       |
c | package, it is governed by the terms of the GPL license,   |
c | and is distributed in the hope that it will be useful, but |
c | WITHOUT ANY WARRANTY; without even the implied warranty    |
c | of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.    |
c | See the GNU General Public License for more details.       |
c | Paul Michaels, February 2002 <pm@cgiss.boisestate.edu>     |
c |                                                            |
c |------------------------------------------------------------|
c |Minor modifications, Fortran IV to Fortran 77 by P. Michaels|
c |------------------------------------------------------------|
c |                      REFERENCE                             |
c | Robinson, E.A., 1967, "Multichannel Time Series Analysis   |
c |   with Digital Computer Programs", Holden-Day, 298p.       |
c |                                                            |
c |------------------------------------------------------------|
c |  eureka: solves normal equations for shape using the       |
c |          Levinson recursion algorithm.                     |
c |                                                            |
c | r=array of autocorrelation coefficients                    |
c | g=rhs of normal equations (cross correlation input*desired)|
c | f=solution, filter coefficients fo,f1,f2,f3.....           |
c | a=prediction error operator 1,a1,a2,a3,....                |
c |                                                            |
c |see: Levinson, N.,(1946),"The Wiener RMS (root mean square) |
c |       error criterion in filter design and prediction"     |
c |       J. Math. Phys. 25, 261-278.                          |
c |                                                            |
c |------------------------------------------------------------|

      subroutine eureka(lr,r,g,f,a)
      implicit real*4 (a-h,o-z)
c     real*4 r(lr),g(lr),f(lr),a(lr)
      real*4 r(*),g(*),f(*),a(*)
      v=r(1)
      d=r(2)
      a(1)=1.
      f(1)=g(1)/v
      q=f(1)*r(2)
      if(lr.eq.1) return
      do 4 l=2,lr
      a(l)=-d/v
      if(l.eq.2) go to 2
      l1=(l-2)/2
      l2=l1+1
      do 1 j=2,l2
      hold=a(j)
      k=l-j+1
      a(j)=a(j)+a(l)*a(k)
    1 a(k)=a(k)+a(l)*hold
      if(2*l1.eq.l-2) go to 2
      a(l2+1)=a(l2+1)+a(l)*a(l2+1)
    2 v=v+a(l)*d
      f(l)=(g(l)-q)/v
      l3=l-1
      do 3 j=1,l3
      k=l-j+1
    3 f(j)=f(j)+f(l)*a(k)
      if(l.eq.lr) return
      d=0.0
      q=0.0
      do 4 i=1,l
      k=l-i+2
      d=d+a(i)*r(k)
    4 q=q+f(i)*r(k)
      return
      end
