c |$Id: l2norm.f,v 1.1.1.1 2017/09/16 19:19:46 pm Exp $          |
c --------------------------------------------------------------
c | Basic Seismic Utilities                                    |
c | AUTHOR: P. MICHAELS  http://cgiss.boisestate.edu/~pm       | 
c | PROGRAM NAME: l2norm.f                                     |
c | DIRECTORY: bsu-2.0.2/src/sublibF4                          |
c |------------------------------------------------------------|
c |  NOTICE:                                                   |
c |  Copyright (C) 2017 Paul Michaels                          |
c |  <pm@cgiss.boisestate.edu>                                 |
c |  This program is free software; you can                    |
c |  redistribute it and/or modify it under the terms          |
c |  of the GNU General Public License as published            |
c |  by the Free Software Foundation; either version           |
c |  3 of the License, or (at your option) any later           |
c |  version.  This program is distributed in the              |
c |  hope that it will be useful, but                          |
c |  WITHOUT ANY WARRANTY; without even the implied            |
c |  warranty of MERCHANTABILITY or FITNESS FOR A              |
c |  PARTICULAR PURPOSE.  See the GNU General Public           |
c |  License for more details.                                 |
c |  You should have received a copy of the GNU                |
c |  General Public License along with this program;           |
c |  if not, write to the Free Software Foundation,            |
c |  Inc., 675 Mass Ave, Cambridge, MA 02139, USA.             |
c |------------------------------------------------------------|
c | Normalizes a trace by L2 norm.  The norm is calculated over|
c | aperture specified by tmin1 and tmax1.                     |
c |                                                            |
c | npts = number of samples in trace                          |
c | s1   = input trace (un-normalized)                         |
c | s2   = output trace (normalized)                           |
c | fsamin= sample interval in seconds                         |
c | tmin1= start time of gate for norm calculation (seconds)   |
c | tmax1= end time of gate for norm calculation   (seconds)   |
c |                                                            |
c | Note: Entire trace (both inside and outside of gate) is    |
c | scaled by the same value.  Checks are imposed to make sure |
c | that the gate bounds remain within the range of samples.   |
c |                                                            |
c --------------------------------------------------------------
      subroutine L2NORM(npts,s1,s2,fsamin,tmin1,tmax1)
      real*4            s1(*),s2(*)
      real*8            sum,fleng
c
c...check gate limits and find sample numbers
      tmax11=float(npts-1)*fsamin
      istart=int(tmin1/fsamin+0.5)
      istop=int(tmax1/fsamin+0.5)
      if(istart.lt.1) istart=1
      if(istop.gt.npts) istop=npts
      if(istart.ge.npts) istart=npts-1
      if(istop.le.istart) istop=istart+1
      fleng=dfloat(istop-istart+1)
c
c...compute L2 norm and scale trace...
      sum=0.0
      do 75 jt=istart,istop
c     sum=(s1(jt)*s1(jt)) + sum
      sum=dprod(s1(jt),s1(jt)) + sum
  75  continue
c
      sum=sqrt(sum/fleng)
      if(sum.eq.0.0D+00) sum=1.0
      sums=real(sum,4)
      do 80 jt=1,npts
  80  s2(jt)=(s1(jt)/sums)
      return
      end
